Type: Package
Package: retroharmonize
Title: Ex Post Survey Data Harmonization
Version: 0.2.8
Date: 2026-05-19
Authors@R: c(
    person(given = "Daniel", 
           family = "Antal", 
           email = "daniel.antal@dataobservatory.eu",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7513-6760")),
      person(given = "Marta",
             family = "Kolczynska",
             role = c("ctb"),
             email = "mkolczynska@gmail.com",
             comment = c(ORCID = "0000-0003-4981-0437"))
      )
Maintainer: Daniel Antal <daniel.antal@dataobservatory.eu>
Description: Assist in reproducible retrospective (ex-post) harmonization
    of data, particularly individual level survey data, by providing tools
    for organizing metadata, standardizing the coding of variables, and
    variable names and value labels, including missing values, and
    documenting the data transformations, with the help of comprehensive
    s3 classes.
License: GPL-3
URL: https://retroharmonize.dataobservatory.eu/
BugReports: https://github.com/dataobservatory-eu/retroharmonize/issues
Depends: R (>= 3.5.0)
Imports: assertthat, cli, dataset, dplyr (>= 1.0.0), fs, glue, haven,
        here, labelled, magrittr, purrr, rlang, snakecase, stats,
        stringr, tibble, tidyr, tidyselect, utils, vctrs
Suggests: covr, ggplot2, knitr, markdown, png, rmarkdown, pillar,
        spelling, statcodelists, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-20 19:04:59 UTC; DanielAntal
Author: Daniel Antal [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7513-6760>),
  Marta Kolczynska [ctb] (ORCID: <https://orcid.org/0000-0003-4981-0437>)
Repository: CRAN
Date/Publication: 2026-05-21 07:20:09 UTC
Built: R 4.5.2; ; 2026-05-25 02:35:21 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",retroharmonize_labelled_spss_survey)
S3method("names<-",retroharmonize_labelled_spss_survey)
S3method(format,retroharmonize_labelled_spss_survey)
S3method(is.na,retroharmonize_labelled_spss_survey)
S3method(levels,retroharmonize_labelled_spss_survey)
S3method(mean,retroharmonize_labelled_spss_survey)
S3method(median,retroharmonize_labelled_spss_survey)
S3method(obj_print_footer,retroharmonize_labelled_spss_survey)
S3method(obj_print_header,retroharmonize_labelled_spss_survey)
S3method(print,retroharmonize_labelled_spss_survey)
S3method(print,survey_df)
S3method(quantile,retroharmonize_labelled_spss_survey)
S3method(sum,retroharmonize_labelled_spss_survey)
S3method(summary,retroharmonize_labelled_spss_survey)
S3method(summary,survey)
S3method(vec_cast,character.retroharmonize_labelled_spss_survey)
S3method(vec_cast,double.retroharmonize_labelled_spss_survey)
S3method(vec_cast,integer.retroharmonize_labelled_spss_survey)
S3method(vec_cast,retroharmonize_labelled_spss_survey.retroharmonize_labelled_spss_survey)
S3method(vec_ptype2,double.retroharmonize_labelled_spss_survey)
S3method(vec_ptype2,integer.retroharmonize_labelled_spss_survey)
S3method(vec_ptype2,retroharmonize_labelled_spss_survey.double)
S3method(vec_ptype2,retroharmonize_labelled_spss_survey.integer)
S3method(vec_ptype2,retroharmonize_labelled_spss_survey.retroharmonize_labelled_spss_survey)
S3method(vec_ptype_abbr,retroharmonize_labelled_spss_survey)
S3method(vec_ptype_full,retroharmonize_labelled_spss_survey)
S3method(weighted.mean,retroharmonize_labelled_spss_survey)
export("%>%")
export(as_character)
export(as_factor)
export(as_labelled_spss_survey)
export(as_numeric)
export(codebook_surveys_create)
export(codebook_waves_create)
export(collect_na_labels)
export(collect_val_labels)
export(concatenate)
export(create_codebook)
export(crosswalk)
export(crosswalk_surveys)
export(crosswalk_table_create)
export(document_survey_item)
export(document_surveys)
export(document_waves)
export(harmonize_na_values)
export(harmonize_survey_values)
export(harmonize_survey_variables)
export(harmonize_values)
export(harmonize_var_names)
export(harmonize_waves)
export(here)
export(is.crosswalk_table)
export(is.labelled_spss_survey)
export(is.na_range_to_values)
export(is.survey)
export(is.survey_df)
export(label_normalize)
export(labelled_spss_survey)
export(merge_surveys)
export(merge_waves)
export(metadata_create)
export(metadata_survey_create)
export(na_range_to_values)
export(pull_survey)
export(read.csv)
export(read_csv)
export(read_dta)
export(read_rds)
export(read_spss)
export(read_surveys)
export(subset_save_surveys)
export(subset_surveys)
export(subset_waves)
export(survey)
export(survey_df)
export(val_label_normalize)
export(var_label_normalize)
import(rlang)
import(vctrs)
importFrom(assertthat,assert_that)
importFrom(cli,cat_line)
importFrom(dataset,'identifier<-')
importFrom(dataset,datacite)
importFrom(dataset,dataset_df)
importFrom(dataset,defined)
importFrom(dataset,dublincore)
importFrom(dataset,is.dataset_df)
importFrom(dataset,subject_create)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,distinct_all)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,tally)
importFrom(dplyr,ungroup)
importFrom(fs,dir_exists)
importFrom(fs,file_exists)
importFrom(fs,is_file)
importFrom(fs,path_ext)
importFrom(fs,path_ext_remove)
importFrom(fs,path_file)
importFrom(glue,glue)
importFrom(haven,format_tagged_na)
importFrom(haven,is.labelled)
importFrom(haven,labelled)
importFrom(haven,labelled_spss)
importFrom(haven,read_dta)
importFrom(haven,read_spss)
importFrom(here,here)
importFrom(labelled,"var_label<-")
importFrom(labelled,`var_label<-`)
importFrom(labelled,labelled)
importFrom(labelled,na_range)
importFrom(labelled,na_values)
importFrom(labelled,to_character)
importFrom(labelled,to_labelled)
importFrom(labelled,val_labels)
importFrom(labelled,var_label)
importFrom(magrittr,"%>%")
importFrom(purrr,map)
importFrom(purrr,reduce)
importFrom(purrr,safely)
importFrom(purrr,set_names)
importFrom(rlang,.data)
importFrom(rlang,set_names)
importFrom(snakecase,to_snake_case)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,weighted.mean)
importFrom(stringr,str_extract)
importFrom(tibble,as_tibble)
importFrom(tibble,deframe)
importFrom(tibble,rowid_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,nest)
importFrom(tidyr,unnest)
importFrom(tidyr,unnest_longer)
importFrom(tidyselect,all_of)
importFrom(tidyselect,everything)
importFrom(utils,bibentry)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,person)
importFrom(utils,read.csv)
importFrom(vctrs,vec_data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# retroharmonize <a href='https://retroharmonize.dataobservatory.eu/'><img src='man/figures/logo.png' align="right" height="139" /></a>

![retroharmonize](man/figures/logo20.png) <!-- badges: start -->
[![Project Status: Active. The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/retroharmonize)](https://cran.r-project.org/package=retroharmonize)
[![CRAN_time_from_release](https://www.r-pkg.org/badges/ago/retroharmonize)](https://cran.r-project.org/package=retroharmonize)
[![metacran
downloads](https://cranlogs.r-pkg.org/badges/retroharmonize)](https://cran.r-project.org/package=retroharmonize)
[![Codecov](https://codecov.io/gh/dataobservatory-eu/retroharmonize/branch/main/graph/badge.svg)](https://app.codecov.io/gh/dataobservatory-eu/retroharmonize)
[![R-CMD-check](https://github.com/dataobservatory-eu/retroharmonize/workflows/R-CMD-check/badge.svg)](https://github.com/dataobservatory-eu/retroharmonize/actions)
<!-- badges: end -->

The goal of `retroharmonize` is to facilitate retrospective (ex-post)
harmonization of survey data in a reproducible manner. The package
provides tools for organizing the metadata, standardizing the coding of
variables, variable names and value labels, including missing values,
and for documenting all transformations, with the help of comprehensive
S3 classes.

Currently being generalized from problems solved in the not yet released
[eurobarometer](https://github.com/antaldaniel/eurobarometer) package
([doi](https://doi.org/10.5281/zenodo.3825700).)

## Installation

The package is available on [CRAN](https://CRAN.R-project.org):

``` r
install.packages("retroharmonize")
```

The development version can be installed from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("rOpenGov/retroharmonize")
```

You can download the manual in PDF for the [0.2.4
release](https://retroharmonize.dataobservatory.eu/retroharmonize_0.2.4.pdf)
but it is significantly different from 0.2.5.

## Survey harmonization

Surveys, i.e., systematic primary observation and data collections are
important data sources of both social and natural sciences. They are in
most cases the primary data sources of scientific research. Drawing
information from several surveys, conducted in different locations or in
different time can greatly enhance the inferential capacity of the
surveys, but it requires significant data processing and statistical
processing work. Our R software package offers a practical and
comprehensive solution to harmonizing the datasets and their codebooks.

Statistical matching is a related concepts that can take a harmonized
dataset further, for example, with creating new, statistically better,
unified weights. For these problems,
[StatMatch](https://CRAN.R-project.org/package=StatMatch) is a mature
solution in R.

Survey data harmonization refers to procedures that improve the data
comparability or the inferential capacity of multiple surveys. The
*retroharmonize* package support various data processing, documentation,
file/type conversion aspects of various retrospective survey
harmonization workflows (i.e. harmonization tasks related to surveys
that already have already been conducted, recorded into a coded file.)

From a technical perspective, the aim of the survey harmonization is to
create a single, tidy, joined harmonized dataset in the form of a data
frame that contains a row identifier, which is truly unique across all
observations, and which also contains the concatenated and harmonized
variables. We do this in a way that provides an unambiguous mapping of
numerical coded and labelled data, including special and missing data.
This way we avoid coercion that may lead to logical errors due to
syntactically correct, but logically inconsistent variable labelling in
across differently coded source files. Taking the harmonization to the
level of type harmonization to *numeric* and *factor* classes allows the
use of R’s powerful statistical packages that require numeric or factor
type input, and a wide range of survey output harmonization (harmonized
statistics and indicators.

For an extended overview of these problems with illustrations please
refer to the vignette [Survey
Harmonization](https://retroharmonize.dataobservatory.eu/articles/survey_harmonization.html).

### 1. Importing

Survey data, i.e., data derived from questionnaires or systematic data
collection, such as inspecting objects in nature, recording prices at
shops are usually stored databases, and converted to complex files
retaining at least coding, labelling metadata together with the data.
This must be imported to R so that the appropriate harmonization tasks
can be carried out with the appropriate R types.

### 2. Harmonization of concepts

After importing data with some descriptive metadata such as numerical
coding and labelling, we need to create a map of the information that is
in our R session to prepare a harmonization plan. We must find
information related to sufficiently similar concepts that can be
harmonized to be successfully joined into a single variable, and
eventually a table of similar variables must be joined.

We create a map of the measured concepts that needs to be harmonized,
for example, a binary sex variable with missing cases and a four-level
categorical variable on gender identification that has *other* and
*declined* options. See the vignette [Working With Survey
Metadata](https://retroharmonize.dataobservatory.eu/articles/survey_harmonization.html)
how mapping the metadata of the surveys can help getting started with
this first step.

We use a [crosswalk
table](https://retroharmonize.dataobservatory.eu/articles/crosswalk.html)
or a *crosswalk scheme* for all the variable name, value label and type
conversion tasks that we plan to do.

### 3. Harmonization of variable names

Make sure that `survey_1$sex` and `survey_2$gender` can be concatenated
to a gender vector or `survey_joined$gender`. See more in the [Working
With A Crosswalk
Table](https://retroharmonize.dataobservatory.eu/articles/crosswalk.html).

### 4. Harmonization of variable numerical codes and labels

For example, *Female=0* in `survey_1$sex` and *female=2* in
`survey_2$gender` becomes consistently female=0. Missing and *declined*
values are consistently handled.

### 5. Consistent types

To use R’s statistical functions with the concatenated version of
`survey_1$sex` and `survey_2$gender` they must have the same R type. In
the vast majority of the cases either *numeric* or *factor*, and in data
visualization applications sometimes *character*. See more in the
[Harmonize Value
Labels](https://retroharmonize.dataobservatory.eu/articles/harmonize_labels.html)
vignette.

### 6. Reproducibility & Documentation

To review statistical results and model results derived from the
concatenated variable (or the joined data frame), they must remain
comparable with `survey_1$sex` and `survey_2$gender`. It is also
necessary to have a new, unique row ID for each observation. If you want
to make your work available outside R, in a different software, the
joined, longitudional data frame must be exported in a consistent
manner.

## Use Cases

We also provide three extensive case studies illustrating how the
`retroharmonize` package can be used for ex-post harmonization of data
from cross-national surveys:

- [Afrobarometer](https://retroharmonize.dataobservatory.eu/articles/afrobarometer.html)
- [Arab
  Barometer](https://retroharmonize.dataobservatory.eu/articles/arabbarometer.html)
- [Eurobarometer](https://retroharmonize.dataobservatory.eu/articles/eurobarometer.html)

The creators of `retroharmonize` are not affiliated with either
Afrobarometer, Arab Barometer, Eurobarometer, or the organizations that
designs, produces or archives their surveys.

We create a large, harmonized dataset for extensive testing of our
packages capabilities. The replication data of this special use case can
be found on

- [Harmonized Cultural Access & Participation
  Dataset](https://retroharmonize.dataobservatory.eu/articles/cap.html)

You can find this harmonized dataset on Zenodo in the [Digital Music
Observatory](https://zenodo.org/communities/music_observatory/) and the
[Cultural Creative Sectors Industries Data
Observatory](https://zenodo.org/communities/ccsi/) repositories.

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.5781672.svg)](https://doi.org/10.5281/zenodo.5781672)

We are building experimental APIs data in the form of automated
observatories, which are running retroharmonize regularly and improving
known statistical data sources. See also the [Green Deal Data
Observatory](https://greendeal.dataobservatory.eu/) and the [Economy
Data Observatory](https://economy.dataobservatory.eu/).

## Working with SPSS files

Survey data is often available in SPSS’s custom labelled format.
Unfortunately, joining data with different labelling is not possible.
When you do not need to preserve the history of complex harmonization
problems, codebook, etc, then you do not necessary need to look under
the hoods of our S3 classes. The new `labelled_spss_survey()` class is
an inherited extension of [haven’s labelled_spss
class](https://haven.tidyverse.org/reference/labelled_spss.html). It not
only preserves variable and value labels and the user-defined missing
range, but also gives an identifier, for example, the filename or the
wave number, to the vector. Additionally, it enables the preservation—
as metadata attributes—the original variable names, labels, and value
codes and labels, from the source data. This way, the harmonized data
also contain the pre-harmonization record. The vignette [Working With
The labelled_spss_survey
Class](https://retroharmonize.dataobservatory.eu/articles/labelled_spss_survey.html)
provides more information about the `labelled_spss_survey()` class.

In [Harmonize Value
Labels](https://retroharmonize.dataobservatory.eu/articles/harmonize_labels.html)
we discuss the characteristics of the `labelled_spss_survey()` class and
demonstrates the problems that using this class solves.

## Citations and related work

### Citing the data sources

Our package has been tested on three harmonized survey’s microdata.
Because [retroharmonize](https://retroharmonize.dataobservatory.eu/) is
not affiliated with any of these data sources, to replicate our
tutorials or work with the data, you have download the data files from
these sources, and you have to cite those sources in your work.

**Afrobarometer** data: Cite
[Afrobarometer](https://www.afrobarometer.org/data/) **Arab Barometer**
data: cite [Arab
Barometer](https://www.arabbarometer.org/survey-data/data-downloads/).
**Eurobarometer** data: The
[Eurobarometer](https://commission.europa.eu/resources/statistics/public-opinion-survey_en)
data
[Eurobarometer](https://commission.europa.eu/resources/statistics/public-opinion-survey_en)
raw data and related documentation (questionnaires, codebooks, etc.) are
made available by *GESIS*, *ICPSR* and through the *Social Science Data
Archive* networks. You should cite your source, in our examples, we rely
on the [GESIS](https://search.gesis.org/) data files.

### Citing the retroharmonize R package

For main developer and contributors, see the
[package](https://retroharmonize.dataobservatory.eu/) homepage.

This work can be freely used, modified and distributed under the GPL-3
license:

``` r
citation("retroharmonize")
#> To cite package 'retroharmonize' in publications use:
#> 
#>   Antal D (2026). _retroharmonize: Ex Post Survey Data Harmonization_.
#>   R package version 0.2.6,
#>   <https://retroharmonize.dataobservatory.eu/>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Manual{,
#>     title = {retroharmonize: Ex Post Survey Data Harmonization},
#>     author = {Daniel Antal},
#>     year = {2026},
#>     note = {R package version 0.2.6},
#>     url = {https://retroharmonize.dataobservatory.eu/},
#>   }
```

### Contact

For contact information, see the
[package](https://retroharmonize.dataobservatory.eu/) homepage.

### Code of Conduct

Please note that the `retroharmonize` project is released with a
[Contributor Code of
Conduct](https://www.contributor-covenant.org/version/2/0/code_of_conduct/).
By contributing to this project, you agree to abide by its terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(retroharmonize)

examples_dir <- system.file("examples", package = "retroharmonize")
my_rds_files <- dir(examples_dir)[grepl(
  ".rds",
  dir(examples_dir)
)]

## ----memory-------------------------------------------------------------------
example_surveys <- read_surveys(file.path(examples_dir, my_rds_files))
documented_surveys <- document_surveys(survey_list = example_surveys)

attr(documented_surveys, "original_list")
documented_surveys

## ----files--------------------------------------------------------------------
document_surveys(survey_paths = file.path(examples_dir, my_rds_files))

## ----knitrsetup, include = FALSE----------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE-----------------------------------------------------
library(retroharmonize)
library(dplyr)

## ----surveyfiles--------------------------------------------------------------
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
survey_files

## ----surveypaths--------------------------------------------------------------
survey_paths <- file.path(examples_dir, survey_files)

## ----examplemetadata----------------------------------------------------------
example_metadata <- metadata_create(survey_paths = survey_paths)

## ----subsetmetadata-----------------------------------------------------------
subset_example_metadata <- example_metadata %>%
  filter(grepl("^unique_identifier_in|trust|country_code", var_label_orig)) %>%
  filter(grepl(
    "^unique_identifier_in|european_parliament|country_code",
    var_label_orig
  )) %>%
  filter(var_name_orig != "uniqid")

subset_example_metadata

## ----ct1----------------------------------------------------------------------
ct <- crosswalk_table_create(subset_example_metadata)
ct

## ----varnames-----------------------------------------------------------------
ct %>%
  mutate(var_name_target = case_when(
    var_name_orig == "rowid" ~ .data$var_name_orig,
    var_name_orig == "isocntry" ~ "geo",
    TRUE ~ "trust_ep"
  )) %>%
  distinct(across(all_of(c("filename", "var_name_orig", "var_name_target"))))

## ----vallabels----------------------------------------------------------------
ct %>%
  mutate(val_numeric_target = case_when(
    val_numeric_orig == 1 ~ .data$val_numeric_orig,
    val_numeric_orig == 2 ~ 0,
    TRUE ~ 99999
  )) %>%
  mutate(val_label_target = case_when(
    val_numeric_orig == 1 ~ "trust",
    val_numeric_orig == 2 ~ "distrust",
    TRUE ~ "declined"
  )) %>%
  distinct(across(all_of(c(
    "filename",
    "val_numeric_orig", "val_numeric_target",
    "val_label_orig", "val_label_target"
  ))))

## ----nalabels-----------------------------------------------------------------
ct %>%
  mutate(na_numeric_target = case_when(
    na_numeric_orig == 3 ~ 99999,
    TRUE ~ NA_real_
  )) %>%
  mutate(na_label_target = case_when(
    na_numeric_target == 99999 ~ "declined",
    TRUE ~ NA_character_
  )) %>%
  distinct(across(all_of(c(
    "filename", "val_numeric_orig",
    "na_numeric_orig", "na_numeric_target",
    "na_label_orig", "na_label_target"
  ))))

## ----crosswalktarget----------------------------------------------------------
example_crosswalk_table <- ct %>%
  mutate(var_name_target = case_when(
    var_name_orig == "rowid" ~ .data$var_name_orig,
    var_name_orig == "isocntry" ~ "geo",
    TRUE ~ "trust_ep"
  )) %>%
  mutate(val_numeric_target = case_when(
    val_numeric_orig == 1 ~ .data$val_numeric_orig,
    val_numeric_orig == 2 ~ 0,
    TRUE ~ 99999
  )) %>%
  mutate(val_label_target = case_when(
    val_numeric_orig == 1 ~ "trust",
    val_numeric_orig == 2 ~ "distrust",
    TRUE ~ "declined"
  )) %>%
  mutate(na_numeric_target = case_when(
    na_numeric_orig == 3 ~ 99999,
    TRUE ~ NA_real_
  )) %>%
  mutate(na_label_target = case_when(
    na_numeric_target == 99999 ~ "declined",
    TRUE ~ NA_character_
  ))

example_crosswalk_table

## ----readtomemory-------------------------------------------------------------
example_surveys <- read_surveys(survey_paths, .f = "read_rds")

## ----subsetsave1--------------------------------------------------------------
subset_survey_list_1 <- subset_surveys(
  survey_list = example_surveys,
  subset_vars = c("rowid", "isocntry", "qa10_1", "qa14_1"),
  subset_name = "subset_example"
)

## ----subsetsave11-------------------------------------------------------------
vapply(subset_survey_list_1, function(x) attr(x, "id"), character(1))

## ----subsetsave13-------------------------------------------------------------
head(subset_survey_list_1[[1]])

## ----subsetsave14-------------------------------------------------------------
lapply(subset_survey_list_1, names)

## ----subsetsave21-------------------------------------------------------------
subset_survey_list_2 <- subset_surveys(
  crosswalk_table = example_crosswalk_table,
  survey_list = example_surveys,
  subset_name = "trust_ep"
)

## ----subsetsave22-------------------------------------------------------------
vapply(subset_survey_list_2, function(x) attr(x, "id"), character(1))

## ----subsetsave23-------------------------------------------------------------
head(subset_survey_list_2[[1]])

## ----subsetsave24-------------------------------------------------------------
lapply(subset_survey_list_2, names)

## ----subsetsave---------------------------------------------------------------
subset_surveys(
  survey_list = example_surveys,
  crosswalk_table = example_crosswalk_table,
  subset_name = "trust_ep",
  import_path = examples_dir,
  export_path = tempdir()
)

## ----reread-------------------------------------------------------------------
readRDS(file.path(tempdir(), "ZA5913_trust_ep.rds")) %>%
  head()

## ----echo = FALSE, include=FALSE----------------------------------------------
## https://github.com/tidyverse/rvest/blob/master/vignettes/selectorgadget.Rmd
requireNamespace("png", quietly = TRUE)
embed_png <- function(path, dpi = NULL) {
  meta <- attr(png::readPNG(path, native = TRUE, info = TRUE), "info")
  if (!is.null(dpi)) meta$dpi <- rep(dpi, 2)
  knitr::asis_output(paste0(
    "<img src='", path, "'",
    " width=", round(meta$dim[1] / (meta$dpi[1] / 96)),
    " height=", round(meta$dim[2] / (meta$dpi[2] / 96)),
    " />"
  ))
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(retroharmonize)
library(dplyr)
library(tidyr)
load(file = system.file(
  "afrob", "afrob_vignette.rda",
  package = "retroharmonize"
))

## ----setup, eval=FALSE--------------------------------------------------------
# library(retroharmonize)
# library(dplyr)
# library(tidyr)

## ----import, eval=FALSE-------------------------------------------------------
# ### use here your own directory
# ab <- dir(afrobarometer_dir, pattern = "sav$")
# afrobarometer_rounds <- file.path(afrobarometer_dir, ab)
# 
# ab_waves <- read_surveys(afrobarometer_rounds, .f = "read_spss")

## ----set-id, eval=FALSE-------------------------------------------------------
# attr(ab_waves[[1]], "id") <- "Afrobarometer_R5"
# attr(ab_waves[[2]], "id") <- "Afrobarometer_R6"
# attr(ab_waves[[3]], "id") <- "Afrobarometer_R7"

## ----document-ab, eval=FALSE--------------------------------------------------
# documented_ab_waves <- document_surveys(ab_waves)

## -----------------------------------------------------------------------------
print(documented_ab_waves)

## ----create-metadata, eval=FALSE----------------------------------------------
# ab_metadata <- metadata_create(survey_list = ab_waves)

## ----selection, eval=FALSE----------------------------------------------------
# library(dplyr)
# to_harmonize <- ab_metadata %>%
#   filter(var_name_orig %in%
#     c("rowid", "DATEINTR", "COUNTRY", "REGION", "withinwt") |
#     grepl("trust ", label_orig)) %>%
#   mutate(var_label = var_label_normalize(label_orig)) %>%
#   mutate(var_label = case_when(
#     grepl("^unique identifier", var_label) ~ "unique_id",
#     TRUE ~ var_label
#   )) %>%
#   mutate(var_name = val_label_normalize(var_label))

## -----------------------------------------------------------------------------
to_harmonize <- to_harmonize %>%
  filter(
    grepl("president|parliament|religious|traditional|unique_id|weight|country|date_of_int", var_name)
  )

## -----------------------------------------------------------------------------
head(to_harmonize %>%
  select(all_of(c("id", "var_name", "var_label"))), 10)

## ----merge, eval=FALSE--------------------------------------------------------
# merged_ab <- merge_surveys(
#   survey_list = ab_waves,
#   var_harmonization = to_harmonize
# )
# 
# # country will be a character variable, and doesn't need a label
# merged_ab <- lapply(merged_ab,
#   FUN = function(x) {
#     x %>%
#       mutate(country = as_character(country))
#   }
# )

## ----eval=FALSE---------------------------------------------------------------
# documenteded_merged_ab <- document_surveys(merged_ab)

## -----------------------------------------------------------------------------
print(documenteded_merged_ab)

## ----check, eval=FALSE--------------------------------------------------------
# R6 <- pull_survey(merged_ab, id = "Afrobarometer_R6")

## ----pulled-attributes--------------------------------------------------------
attributes(R6$trust_president[1:20])

## ----document-item------------------------------------------------------------
document_survey_item(R6$trust_president)

## -----------------------------------------------------------------------------
collect_na_labels(to_harmonize)

## -----------------------------------------------------------------------------
collect_val_labels(to_harmonize %>%
  filter(grepl("trust", var_name)))

## ----specify------------------------------------------------------------------
harmonize_ab_trust <- function(x) {
  label_list <- list(
    from = c(
      "^not", "^just", "^somewhat",
      "^a", "^don", "^ref", "^miss", "^not", "^inap"
    ),
    to = c(
      "not_at_all", "little", "somewhat",
      "a_lot", "do_not_know", "declined", "inap", "inap",
      "inap"
    ),
    numeric_values = c(0, 1, 2, 3, 99997, 99998, 99999, 99999, 99999)
  )

  harmonize_values(
    x,
    harmonize_labels = label_list,
    na_values = c(
      "do_not_know" = 99997,
      "declined" = 99998,
      "inap" = 99999
    )
  )
}

## ----harmonize, eval=FALSE----------------------------------------------------
# harmonized_ab_waves <- harmonize_surveys(
#   survey_list = merged_ab,
#   .f = harmonize_ab_trust
# )

## ----eval=FALSE---------------------------------------------------------------
# h_ab_structure <- attributes(harmonized_ab_waves)

## -----------------------------------------------------------------------------
h_ab_structure$row.names <- NULL # We have over 100K row names
h_ab_structure

## ----year, eval=FALSE---------------------------------------------------------
# harmonized_ab_waves <- harmonized_ab_waves %>%
#   mutate(year = as.integer(substr(as.character(
#     date_of_interview
#   ), 1, 4)))

## ----eval=FALSE---------------------------------------------------------------
# harmonized_ab_waves <- harmonized_ab_waves %>%
#   filter(country %in% c(
#     "Niger", "Nigeria", "Algeria",
#     "South Africa", "Madagascar"
#   ))

## ----numeric------------------------------------------------------------------
harmonized_ab_waves %>%
  mutate_at(
    vars(starts_with("trust")),
    ~ as_numeric(.) * within_country_weighting_factor
  ) %>%
  select(-all_of("within_country_weighting_factor")) %>%
  group_by(country, year) %>%
  summarize_if(is.numeric, mean, na.rm = TRUE)

## ----factor-------------------------------------------------------------------
library(tidyr) ## tidyr::pivot_longer()
harmonized_ab_waves %>%
  select(-all_of("within_country_weighting_factor")) %>%
  mutate_if(is.labelled_spss_survey, as_factor) %>%
  pivot_longer(starts_with("trust"),
    names_to  = "institution",
    values_to = "category"
  ) %>%
  mutate(institution = gsub("^trust_", "", institution)) %>%
  group_by(country, year, institution, category) %>%
  summarize(n = n())

## ----out.width='95%', echo=FALSE----------------------------------------------
knitr::include_graphics("ab_plot1.png")

## ----setupvignette, include = FALSE-------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE-----------------------------------------------------
library(retroharmonize)
library(statcodelists)
library(dplyr)
library(knitr)

## ----createcodebook-----------------------------------------------------------
set.seed(12)
my_codebook <- create_codebook(
  survey = read_rds(
    system.file("examples", "ZA7576.rds",
      package = "retroharmonize"
    )
  )
)

sample_n(my_codebook, 12) %>%
  select(.data$filename,
    # Rename variables to DDI Codebook names
    Name = .data$var_name_orig,
    Label = .data$var_label_orig,
    .data$val_code_orig, .data$val_label_orig
  ) %>%
  kable()

## ----statcodelists------------------------------------------------------------
library(statcodelists)
CL_SEX

## ----sexexample, message=FALSE------------------------------------------------
data.frame(
  Value = c(1, 2),
  Label = c("Male", "Female")
)

## ----marriagestatusexample----------------------------------------------------
data.frame(
  Value = c(1, 2, 11),
  Label = c(
    "(Re-)Married: without children",
    "(Re-)Married: children this marriage",
    "Divorced/Separated: without children"
  ),
  Normalized = c(
    "Married or Remarried without children",
    "Married or Remarried with children this marriage",
    "Divorced or Separated without children"
  )
) %>% kable()

#' Document survey item provenance
#'
#' @description
#' Document the current and historical coding, labels, missing values,
#' and survey provenance of a harmonized survey variable.
#'
#' @param x A `labelled_spss_survey` vector originating from a single
#'   survey or concatenated from multiple surveys.
#'
#' @return
#' A named list containing:
#' \itemize{
#'   \item current and historical value coding,
#'   \item variable labels,
#'   \item valid and missing value definitions,
#'   \item original variable names,
#'   \item survey identifiers.
#' }
#'
#' @family documentation functions
#'
#' @examples
#' var1 <- labelled::labelled_spss(
#'   x = c(1, 0, 1, 1, 0, 8, 9),
#'   labels = c(
#'     "TRUST" = 1,
#'     "NOT TRUST" = 0,
#'     "DON'T KNOW" = 8,
#'     "INAP. HERE" = 9
#'   ),
#'   na_values = c(8, 9)
#' )
#'
#' var2 <- labelled::labelled_spss(
#'   x = c(2, 2, 8, 9, 1, 1),
#'   labels = c(
#'     "Tend to trust" = 1,
#'     "Tend not to trust" = 2,
#'     "DK" = 8,
#'     "Inap" = 9
#'   ),
#'   na_values = c(8, 9)
#' )
#'
#' harmonization <- list(
#'   from = c(
#'     "^tend\\sto|^trust",
#'     "^tend\\snot|not\\strust",
#'     "^dk|^don",
#'     "^inap"
#'   ),
#'   to = c(
#'     "trust",
#'     "not_trust",
#'     "do_not_know",
#'     "inap"
#'   ),
#'   numeric_values = c(1, 0, 99997, 99999)
#' )
#'
#' missing_values <- c(
#'   "do_not_know" = 99997,
#'   "inap" = 99999
#' )
#'
#' h1 <- harmonize_values(
#'   x = var1,
#'   harmonize_label = "Do you trust the European Union?",
#'   harmonize_labels = harmonization,
#'   na_values = missing_values,
#'   id = "survey1"
#' )
#'
#' h2 <- harmonize_values(
#'   x = var2,
#'   harmonize_label = "Do you trust the European Union?",
#'   harmonize_labels = harmonization,
#'   na_values = missing_values,
#'   id = "survey2"
#' )
#'
#' h3 <- concatenate(h1, h2)
#'
#' document_survey_item(h3)
#'
#' @export

document_survey_item <- function(x) {
  attribute_names <- names(attributes(x))
  original_x_name <- deparse(substitute(x))

  orig_names <- attribute_names[grepl("name$", attribute_names)]
  attr_label <- attribute_names[grepl("label$", attribute_names)]
  labels <- attribute_names[grepl("labels$", attribute_names)]

  attr_na_range <- attribute_names[grepl("na_range$", attribute_names)]
  coding <- as_tibble(sapply(labels, function(l) attr(x, l)))
  coding <- rlang::set_names(coding, gsub("labels", "values", names(coding)))
  labelling <- as_tibble(sapply(labels, function(l) names(attr(x, l))))
  tbl_length <- nrow(coding)

  list(
    code_table = bind_cols(coding, labelling) %>%
      mutate(missing = ifelse(values %in% attr(x, "na_values"),
        TRUE, FALSE
      )),
    history_var_name = c(
      c("name" = original_x_name),
      vapply(orig_names, function(l) attr(x, l), character(1))
    ),
    history_var_label = c(
      vapply(attr_label, function(l) attr(x, l), character(1))
    ),
    history_na_range = c(
      vapply(attr_na_range, function(l) attr(x, l), character(1))
    ),
    history_id = history_survey_id <- attr(x, "id")
  )
}

test_that("document_survey_item preserves variable name history", {
  var1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  var2 <- labelled::labelled_spss(
    x = c(2, 2, 8, 9, 1, 1),
    labels = c(
      "Tend to trust" = 1,
      "Tend not to trust" = 2,
      "DK" = 8,
      "Inap" = 9
    ),
    na_values = c(8, 9)
  )

  harmonization <- list(
    from = c(
      "^tend\\sto|^trust",
      "^tend\\snot|not\\strust",
      "^dk|^don",
      "^inap"
    ),
    to = c(
      "trust",
      "not_trust",
      "do_not_know",
      "inap"
    ),
    numeric_values = c(1, 0, 99997, 99999)
  )

  missing_values <- c(
    "do_not_know" = 99997,
    "inap" = 99999
  )

  h1 <- harmonize_values(
    x = var1,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = harmonization,
    na_values = missing_values,
    id = "survey1"
  )

  h2 <- harmonize_values(
    x = var2,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = harmonization,
    na_values = missing_values,
    id = "survey2"
  )

  h3 <- concatenate(h1, h2)

  documented <- document_survey_item(h3)

  expect_equal(
    documented$history_var_name,
    c(
      "name" = "h3",
      "survey1_name" = "var1",
      "survey2_name" = "var2"
    )
  )
})

#' Normalize value and variable labels
#'
#' \code{label_normalize} removes special characters, whitespace,
#' and other typical typing errors.
#'
#' \code{var_label_normalize} and \code{val_label_normalize} removes possible
#' chunks from question identifiers.
#'
#' The functions \code{var_label_normalize} and
#' \code{val_label_normalize} may
#' be differently implemented for various survey series.
#'
#' @param x A character vector of labels to be normalized.
#' @importFrom snakecase to_snake_case
#' @examples
#' label_normalize(
#'   c(
#'     "Don't know", " TRUST", "DO NOT  TRUST",
#'     "inap in Q.3", "Not 100%", "TRUST < 50%",
#'     "TRUST >=90%", "Verify & Check", "TRUST 99%+"
#'   )
#' )
#'
#' var_label_normalize(
#'   c(
#'     "Q1_Do you trust the national government?",
#'     " Do you trust the European Commission"
#'   )
#' )
#'
#' val_label_normalize(
#'   c(
#'     "Q1_Do you trust the national government?",
#'     " Do you trust the European Commission"
#'   )
#' )
#' @family variable label harmonization functions
#' @return Returns a suggested, normalized label without special characters. The
#' \code{var_label_normalize} and \code{val_label_normalize} returns them in
#' \code{snake_case} for programmatic use.
#' @export

label_normalize <- function(x) {
  ## unit tests for this function are in
  ## tests/testthat/test-label_normalize.R
  ## please add test after correcting for unexpected results

  y <- gsub("\\s+", " ", x)
  y <- gsub("on\\'t", "o not", y) # don't|Don't
  y <- gsub("ON\\'T", "O NOT", y)
  y <- gsub("an\\'t", "an not", y) # can't|Can't
  y <- gsub("AN\\'T", "AN NOT", y)
  y <- gsub("\\'ll", "will", y) # you'll, he'll (can be confused with shall)
  y <- gsub("\\'LL", "WILL", y)
  y <- gsub("\\&", " and ", y)
  y <- gsub("\\+", " plus ", y)
  y <- gsub("\\%", " pct ", y)
  y <- gsub(">=", " ge ", y)
  y <- gsub("<=", " le ", y)
  y <- gsub("<", " lt ", y)
  y <- gsub(">", "_gt_", y)
  y <- gsub("\\.|-|\\:|\\;|\\/|\\(|\\)|\\!|\\?", " ", y)
  y <- gsub(tolower("15_plus"), "gt 15", y)
  y <- gsub("__|___|___|\\s_|_\\s", "_", y)
  y <- gsub("\\s+", " ", y)
  y <- gsub("^_", "", y)
  y <- gsub("_$", "", y)
  trimws(tolower(y), which = "both")
}

#' @rdname label_normalize
#' @importFrom snakecase to_snake_case
#' @family harmonization functions
#' @export
var_label_normalize <- function(x) {
  y <- tolower(as.character(x))
  y <- gsub("^q[abcde]\\d{1,2}_\\d{1,2}", "", y) # remove QA117_1
  y <- gsub("^q[abcde]\\d{1,2}", "", y) # remove QA1, QB25 etc
  y <- gsub("^d\\d{1,2}", "", y) # removed d26_ etc
  y <- gsub("^c\\d{1,2}", "", y) # removed c26_ etc
  y <- gsub("^p\\d+{1,}_", "", y) # remove p6_ like starts
  y <- gsub("^p\\d+{1,}\\s", "", y) # remove p6  like starts
  y <- gsub("^q\\d+{1,}_", "", y) # remove q1_ like starts
  y <- gsub("^q\\d+{1,}[abcdefghijkl]", "", y) # remove q1  like starts
  y <- gsub("^\\d{1,2}_", "", y) # remove leading number 1_

  y <- label_normalize(y)
  y <- gsub(" +", " ", y)
  y <- trimws(tolower(as.character(y)), which = "both")
  snakecase::to_snake_case(y)
}

#' @rdname label_normalize
#' @importFrom snakecase to_snake_case
#' @family harmonization functions
#' @export
val_label_normalize <- function(x) {
  y <- trimws(tolower(as.character(x)), which = "both")
  y <- label_normalize(x)
  y <- gsub("^q[abcde]\\d{1,2}_\\d{1,2}", "", y) # remove QA117_1
  y <- gsub("^q[abcde]\\d{1,2}", "", y) # remove QA1, QB25 etc
  y <- gsub("^d\\d{1,2}", "", y) # removed d26_ etc
  y <- gsub("^c\\d{1,2}", "", y) # removed c26_ etc
  y <- gsub("^p\\d+{1,}_", "", y) # remove p6_ like starts
  y <- gsub("^p\\d+{1,}\\s", "", y) # remove p6  like starts
  y <- gsub("^q\\d+{1,}_", "", y) # remove q1_ like starts
  y <- gsub("^q\\d+{1,}\\s", "", y) # remove q1  like starts
  y <- gsub("^\\d{1,2}_", "", y) # remove leading number 1_

  y <- trimws(tolower(as.character(y)), which = "both")
  y <- gsub(" +", " ", y)
  snakecase::to_snake_case(y)
}

ln <- label_normalize(
  x = c(
    "Don't know", " TRUST", "DO NOT  TRUST",
    "inap in Q.3", "Not 100%", "TRUST < 50%",
    "TRUST >=90%", "Verify & Check", "TRUST 99%+"
  )
)

vrn <- var_label_normalize(
  c(
    "Q1_Do you trust the national government?",
    " Do you trust the European Commission"
  )
)
vrn

vln <- val_label_normalize(
  c(
    "Q1_Do you trust the national government?",
    " Do you trust the European Commission"
  )
)
vln

c(
  "do_you_trust_the_national_government",
  "do_you_trust_the_european_commission"
)

test_that("normalization works", {
  expect_equal(vrn, c(
    "do_you_trust_the_national_government",
    "do_you_trust_the_european_commission"
  ))
  expect_equal(vln, c(
    "do_you_trust_the_national_government",
    "do_you_trust_the_european_commission"
  ))
  expect_equal(
    ln,
    c(
      "do not know", "trust", "do not trust",
      "inap in q 3", "not 100 pct", "trust lt 50 pct",
      "trust ge 90 pct", "verify and check",
      "trust 99 pct plus"
    )
  )
})

test_that("merge_surveys example runs without error", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  survey_files <- dir(examples_dir, pattern = "\\.rds$", full.names = TRUE)

  example_surveys <- read_surveys(
    survey_files,
    save_to_rds = FALSE
  )

  metadata <- metadata_create(survey_list = example_surveys)

  to_harmonize <- metadata %>%
    dplyr::filter(
      var_name_orig %in% c("rowid", "w1") |
        grepl("^trust", var_label_orig)
    ) %>%
    dplyr::mutate(
      var_label = var_label_normalize(var_label_orig),
      var_name_target = val_label_normalize(var_label),
      var_name_target = ifelse(
        .data$var_name_orig %in% c("rowid", "w1", "wex"),
        .data$var_name_orig,
        .data$var_name_target
      )
    )

  expect_no_error(
    merge_surveys(
      survey_list = example_surveys,
      var_harmonization = to_harmonize
    )
  )
})

test_that("merge_surveys returns a list of surveys", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  rds_files <- dir(examples_dir, pattern = "\\.rds$")
  surveys <- read_surveys(file.path(examples_dir, rds_files))

  metadata <- metadata_create(surveys)

  to_harmonize <- metadata %>%
    filter(
      var_name_orig %in% c("rowid", "w1") |
        grepl("^trust", var_label_orig)
    ) %>%
    mutate(
      var_label = var_label_normalize(var_label_orig),
      var_name_target = val_label_normalize(var_label_orig),
      var_name_target = ifelse(
        var_name_orig %in% c("rowid", "w1", "wex"),
        var_name_orig,
        var_name_target
      )
    )

  merged <- merge_surveys(
    survey_list = surveys,
    var_harmonization = to_harmonize
  )

  expect_type(merged, "list")
  expect_length(merged, length(surveys))
})


test_that("harmonize_survey_values returns labelled SPSS variables", {
  harmonize_eb_trust <- function(x) {
    label_list <- list(
      from = c(
        "^tend\\snot", "^cannot", "^tend\\sto", "^can\\srely",
        "^dk", "^inap", "na"
      ),
      to = c(
        "not_trust", "not_trust", "trust", "trust",
        "do_not_know", "inap", "inap"
      ),
      numeric_values = c(0, 0, 1, 1, 99997, 99999, 99999)
    )

    harmonize_values(
      x,
      harmonize_labels = label_list,
      na_values = c(
        "do_not_know" = 99997,
        "declined" = 99998,
        "inap" = 99999
      )
    )
  }

  examples_dir <- system.file("examples", package = "retroharmonize")
  rds_files <- dir(examples_dir, pattern = "\\.rds$")
  surveys <- read_surveys(file.path(examples_dir, rds_files))

  metadata <- metadata_create(surveys)

  to_harmonize <- metadata %>%
    dplyr::filter(
      var_name_orig %in% c("rowid", "w1") |
        grepl("^trust", var_label_orig)
    ) %>%
    dplyr::mutate(
      var_label = var_label_normalize(var_label_orig),
      var_name_target = val_label_normalize(var_label_orig),
      var_name_target = ifelse(
        var_name_orig %in% c("rowid", "w1", "wex"),
        var_name_orig,
        var_name_target
      )
    )

  merged <- merge_surveys(
    survey_list = surveys,
    var_harmonization = to_harmonize
  )

  harmonized <- suppressWarnings(
    harmonize_survey_values(
      survey_list = merged,
      .f = harmonize_eb_trust,
      status_message = FALSE
    )
  )

  expect_true(
    is.labelled_spss_survey(
      harmonized$trust_in_institutions_army
    )
  )
})


test_that("harmonized trust variables have expected numeric range", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  rds_files <- dir(examples_dir, pattern = "\\.rds$")
  surveys <- read_surveys(file.path(examples_dir, rds_files))

  metadata <- metadata_create(surveys)

  to_harmonize <- metadata %>%
    dplyr::filter(
      var_name_orig %in% c("rowid", "w1") |
        grepl("^trust", var_label_orig)
    ) %>%
    dplyr::mutate(
      var_label = var_label_normalize(var_label_orig),
      var_name_target = val_label_normalize(var_label_orig),
      var_name_target = ifelse(
        var_name_orig %in% c("rowid", "w1", "wex"),
        var_name_orig,
        var_name_target
      )
    )

  merged <- merge_surveys(
    survey_list = surveys,
    var_harmonization = to_harmonize
  )

  harmonize_eb_trust <- function(x) {
    label_list <- list(
      from = c(
        "^tend\\snot", "^cannot", "^tend\\sto",
        "^can\\srely", "^dk", "^inap", "na"
      ),
      to = c(
        "not_trust", "not_trust", "trust", "trust",
        "do_not_know", "inap", "inap"
      ),
      numeric_values = c(
        0, 0, 1, 1,
        99997, 99999, 99999
      )
    )

    harmonize_values(x,
      harmonize_labels = label_list,
      na_values = c(
        "do_not_know" = 99997,
        "declined" = 99998,
        "inap" = 99999
      )
    )
  }

  harmonized <- suppressWarnings(
    harmonize_survey_values(
      survey_list = merged,
      .f = function(x) harmonize_eb_trust(x),
      status_message = FALSE
    )
  )

  expect_equal(
    range(
      as_numeric(
        harmonized$trust_in_institutions_european_union
      )[as_numeric(
        harmonized$trust_in_institutions_european_union
      ) < 99997],
      na.rm = TRUE
    ),
    c(0, 1)
  )
})

#' retroharmonize: Retrospective harmonization of survey data files
#'
#' The goal of \code{retroharmonize} is to facilitate retrospective (ex-post)
#' harmonization of data, particularly survey data, in a reproducible manner.
#' The package provides tools for organizing the metadata, standardizing the
#' coding of variables, variable names and value labels, including missing
#' values, and for documenting all transformations, with the help of
#' comprehensive S3 classes.
#'
#' @section import functions:
#' Read data stored in formats with rich metadata, such as SPSS (.sav) files,
#' and make them usable in a programmatic context.\cr
#' \code{\link{read_spss}}: read an SPSS file and record metadata for reproducibility\cr
#' \code{\link{read_rds}}: read an rds file and record metadata for reproducibility\cr
#' \code{\link{read_surveys}}: programmatically read a list of surveys\cr
#' \code{\link{pull_survey}}: pull a single survey from a survey list.\cr
#'
#' @section subsetting functions:
#' \code{\link{subset_surveys}}: remove variables from surveys that cannot be harmonized.
#'
#' @section variable name harmonization functions:
#' \code{\link{harmonize_survey_variables}}: Create a list of surveys with harmonized variable names.\cr
#' @section variable label harmonization functions:
#' Create consistent coding and labelling.\cr
#' \code{\link{harmonize_values}}: Harmonize the label list across surveys.\cr
#' \code{\link{harmonize_survey_values}}: Create a list of surveys with harmonized value labels.\cr
#' \code{\link{na_range_to_values}}: Make the \code{na_range} attributes,
#' as imported from SPSS, consistent with the \code{na_values} attributes.\cr
#' \code{\link{label_normalize}} removes special characters, whitespace,
#' and other typical typing errors and helps the uniformization of labels
#' and variable names.\cr
#'
#' @section survey harmonization functions:
#' \code{\link{merge_surveys}}: Create a list of surveys with harmonized names and variable labels.\cr
#' \code{\link{crosswalk_surveys}}: Create a list of surveys with harmonized variable names, harmonized
#' value labels and harmonize R classes.\cr
#' \code{\link{crosswalk}}: Create a joined data frame of surveys with harmonized variable names, harmonized
#' value labels and harmonize R classes.\cr
#' @section metadata functions:
#' \code{\link{metadata_create}}: Createa metadata dataa from one or more \code{\link{survey}}.\cr
#' \code{\link{metadata_survey_create}}: Create a joined metadata data frame from one survey.\cr
#' \code{\link{create_codebook}} and \code{\link{codebook_waves_create}}
#' \code{\link{crosswalk_table_create}}: Create an initial crosswalk table from a metadata data frame.\cr
#'
#' @section documentation functions:
#' Make the workflow reproducible by recording the harmonization process.
#' \code{\link{document_survey_item}}: Returns a list of the current and historic coding,
#' labelling of the valid range and missing values or range, the history of the variable names
#' and the history of the survey IDs.
#' \code{\link{document_surveys}}: Document the key attributes surveys in a survey list.
#'
#' @section type conversion functions:
#' Consistently treat labels and SPSS-style user-defined missing
#' values in the R language.
#' \code{\link{survey}} helps constructing a valid survey data frame, and
#' \code{\link{labelled_spss_survey}} helps creating a vector for a
#' questionnaire item.
#' \code{\link{as_numeric}}: convert to numeric values.\cr
#' \code{\link{as_factor}}: convert to labels to factor levels.\cr
#' \code{\link{as_character}}: convert to labels to characters.\cr
#' \code{\link{as_labelled_spss_survey}}: convert labelled and labelled_spss
#' vectors to labelled_spss_survey vectors.\cr
#' @docType package
#' @importFrom stringr str_extract
#' @import vctrs
#' @import rlang
#' @name retroharmonize
"_PACKAGE"


utils::globalVariables(c(
  "class_orig", "valid_labels", "val_code_orig", "val_label_orig",
  "na_labels", "entry", "id", "var_name_target", "var_name_orig",
  "filename", "label_type", "n", "n_na_labels",
  "n_labels", "n_valid_labels", "values"
))

#' Harmonize values and labels of labelled vectors
#'
#' Create a harmonized labelled vector with standardized value labels,
#' numeric coding, and missing value definitions.
#'
#' @description
#' `harmonize_values()` converts heterogeneous labelled survey vectors
#' into a harmonized representation suitable for cross-survey integration.
#'
#' The function:
#'
#' - harmonizes value labels using regex-based matching;
#' - assigns harmonized numeric codes;
#' - preserves original coding metadata;
#' - standardizes user-defined missing values;
#' - preserves SPSS-style labelled metadata;
#' - and records provenance attributes.
#'
#' @details
#' Harmonization is performed using a harmonization table supplied via
#' `harmonize_labels`.
#'
#' The harmonization table must contain:
#'
#' - `from`: regex patterns matching original labels;
#' - `to`: harmonized labels;
#' - `numeric_values`: harmonized numeric codes.
#'
#' Original labels and numeric codes are preserved in attributes
#' attached to the returned vector.
#'
#' If no harmonization table is supplied, the function still attempts
#' to normalize common missing value labels such as:
#'
#' - `"inap"`
#' - `"declined"`
#' - `"do_not_know"`
#'
#' @param x A labelled vector, typically of class
#'   `"haven_labelled"` or `"haven_labelled_spss"`.
#'
#' @param harmonize_label Optional harmonized variable label.
#'   Defaults to the original variable label.
#'
#' @param harmonize_labels A list describing harmonization rules.
#'   Must contain the elements:
#'
#'   - `from`
#'   - `to`
#'   - `numeric_values`
#'
#' @param na_values Named numeric vector defining harmonized
#'   missing value codes.
#'
#' @param na_range Optional SPSS-style missing value range.
#'   Usually left `NULL`.
#'
#' @param id Survey identifier.
#'   Defaults to `"survey_id"`.
#'
#' @param name_orig Optional original variable name.
#'   Defaults to the object name supplied to `x`.
#'
#' @param remove Optional regex pattern removed from original labels
#'   before harmonization.
#'
#' @param perl Logical. Use Perl-compatible regular expressions?
#'   Defaults to `FALSE`.
#'
#' @return
#' A harmonized `haven_labelled_spss` vector.
#'
#' The returned vector preserves:
#'
#' - harmonized value labels;
#' - harmonized numeric coding;
#' - SPSS missing value metadata;
#' - original coding metadata;
#' - survey provenance metadata.
#'
#' @family harmonization functions
#'
#' @examples
#' var1 <- labelled::labelled_spss(
#'   x = c(1, 0, 1, 1, 0, 8, 9),
#'   labels = c(
#'     "TRUST" = 1,
#'     "NOT TRUST" = 0,
#'     "DON'T KNOW" = 8,
#'     "INAP. HERE" = 9
#'   ),
#'   na_values = c(8, 9)
#' )
#'
#' harmonize_values(
#'   var1,
#'   harmonize_labels = list(
#'     from = c(
#'       "^tend\\sto|^trust",
#'       "^tend\\snot|not\\strust",
#'       "^dk|^don",
#'       "^inap"
#'     ),
#'     to = c(
#'       "trust",
#'       "not_trust",
#'       "do_not_know",
#'       "inap"
#'     ),
#'     numeric_values = c(
#'       1,
#'       0,
#'       99997,
#'       99999
#'     )
#'   ),
#'   na_values = c(
#'     "do_not_know" = 99997,
#'     "inap" = 99999
#'   ),
#'   id = "survey_id"
#' )
#'
#' @seealso
#' [harmonize_var_names()]
#'
#' @importFrom assertthat assert_that
#' @importFrom dplyr arrange distinct distinct_all filter if_else
#' @importFrom dplyr left_join mutate select
#' @importFrom haven labelled_spss
#' @importFrom labelled labelled na_range na_values
#' @importFrom labelled to_character val_labels var_label
#' @importFrom rlang .data set_names
#' @importFrom tibble as_tibble tibble
#' @importFrom tidyselect all_of
#'
#' @export

harmonize_values <- function(
  x,
  harmonize_label = NULL,
  harmonize_labels = NULL,
  na_values = c(
    "do_not_know" = 99997,
    "declined" = 99998,
    "inap" = 99999
  ),
  na_range = NULL,
  id = "survey_id",
  name_orig = NULL,
  remove = NULL,
  perl = FALSE
) {
  if (is.null(na_values)) input_na_values <- NULL

  if (is.null(na_values)[1] | is.na(na_values)[1]) {
    if ("na_values" %in% names(harmonize_labels)) {
      input_na_values <- na_values
      harmonize_labels <- harmonize_labels[c("from", "to", "numeric_values")]
    }
  } else {
    # what if there are no na_values given?
    input_na_values <- na_values
    harmonize_labels <- harmonize_labels[c("from", "to", "numeric_values")]
  }

  validate_label_list <- function(label_list) {
    assert_that(
      all(c("from", "to", "numeric_values") %in% names(label_list)),
      msg = "The harmonize_labels list must have 'from', 'to', and 'numeric_values' vectors."
    )

    ll_lengths <- vapply(label_list[c("from", "to", "numeric_values")], length, numeric(1))

    assert_that(
      length(unique(ll_lengths)) == 1,
      msg = paste0(
        "The 'from', 'to', and 'numeric_values' vectors must be of equal length, currently it is: ",
        as.character(paste(ll_lengths, collapse = ", "))
      )
    )

    ll <- as_tibble(label_list[c("from", "to", "numeric_values")])

    for (l in unique(label_list$to)) {
      matched_numeric_value <- ll %>%
        filter(.data$to == l) %>%
        distinct(.data$to, .data$numeric_values) %>%
        pull(.data$numeric_values)

      assert_that(length(matched_numeric_value) == 1,
        msg = paste0(
          "in harmonized_list ",
          l, " is matched with multiple numeric values: <",
          paste(matched_numeric_value, collapse = ","), ">"
        )
      )
    }
  }

  if (!is.null(harmonize_labels)) {
    validate_label_list(label_list = harmonize_labels)
  }

  if (is.null(id)) {
    # if not otherwise stated, inherit the ID of x, if present
    if (!is.null(attr(x, "id"))) {
      id <- attr(x, "id")
    } else {
      id <- "unknown"
    }
  }

  ## Get the original object name for recording it as metadata
  original_x_name <- deparse(substitute(x))

  ## Harmonize SPSS variable labels, if they exist
  if (is.null(harmonize_label)) harmonize_label <- labelled::var_label(x)
  if (is.null(harmonize_label)) harmonize_label <- original_x_name

  if (!is.null(harmonize_labels) & validate_harmonize_labels(harmonize_labels)) { ## see assertions.R
    harmonize_labels <- tibble::as_tibble(harmonize_labels) %>%
      select(all_of(c("from", "to", "numeric_values")))
  }

  original_values <- tibble::tibble(
    x = vctrs::vec_data(x)
  )

  original_values$orig_labels <- if_else(
    condition = original_values$x %in% labelled::val_labels(x),
    true = as_character(x),
    false = NA_character_
  )

  if (!is.null(remove)) {
    harmonize_labels$from <- gsub(remove, "", harmonize_labels$from)
    original_values$orig_labels <- gsub(remove, "", original_values$orig_labels)
  }

  if (is.na_range_to_values(x)) {
    x <- na_range_to_values(x)
  }

  if (!is.null(harmonize_labels)) {
    condition_string <- tolower(gsub("\\.|\\(|\\)", "", harmonize_labels$from))
    codebook_string <- tolower(gsub("\\.|\\(|\\)", "", original_values$orig_labels))

    original_values$orig_labels <- if_else(
      # label == "" if not in the harmonization list
      condition = grepl(paste(condition_string, collapse = "|"),
        codebook_string,
        perl = perl
      ),
      true = codebook_string,
      false = ""
    )

    code_table <- dplyr::distinct_all(original_values)
    str <- code_table$orig_labels # string of original values
    code_table$new_labels <- NA_character_
    code_table$new_values <- NA_real_

    for (o in seq_along(harmonize_labels$from)) {
      from_string <- tolower(gsub("\\.|\\(|\\)", "", harmonize_labels$from[o]))

      code_table$new_labels[which(grepl(from_string, str))] <- harmonize_labels$to[o]
      code_table$new_values[which(grepl(from_string, str))] <- harmonize_labels$numeric_values[o]

      # code_table$new_labels [which ( tolower(harmonize_labels$from[o]) == str) ] <- harmonize_labels$to[o]
      # code_table$new_values [which ( tolower(harmonize_labels$from[o]) == str) ] <- harmonize_labels$numeric_values[o]
    }

    old_code <- function() {
      ## Can be removed after extensive testing -------
      for (r in seq_along(harmonize_labels$to)) {
        ## harmonize the strings to new labelling by regex
        str[which(grepl(tolower(harmonize_labels$from[r]), str, perl = perl))] <- harmonize_labels$to[r]
      }

      add_new_values <- harmonize_labels %>%
        as_tibble() %>%
        dplyr::select(tidyselect::all_of(
          c("to", "numeric_values")
        )) %>%
        rlang::set_names(c("new_labels", "new_values"))

      code_table <- dplyr::left_join(
        code_table,
        add_new_values,
        by = "new_labels"
      ) %>%
        filter(!is.na(.data$new_values))
    }

    code_table$original_values <- NULL
  } else {
    # no harmonization is given -----------------------
    code_table <- get_labelled_attributes(x) ## see below main function
    na_labels <- names(na_values)

    if (length(na_labels) > 0) {
      # if there is no valid label harmonization, still check for potential missings
      potential_na_values <- sapply(na_labels, function(x) paste0("^", x, "|", x))
      na_regex <- sapply(potential_na_values, function(s) grepl(s, val_label_normalize(code_table$new_labels), perl = perl))
      for (c in seq_along(na_labels)) {
        code_table$new_labels[which(na_regex[, c])] <- na_labels[c]
      }
    }
  }

  code_table <- code_table %>%
    distinct_all() %>%
    dplyr::arrange(.data$new_values)

  new_value_table <- original_values %>%
    dplyr::left_join(
      code_table,
      by = c("x", "orig_labels")
    ) %>%
    dplyr::mutate(new_values = if_else(
      condition = is.na(.data$new_labels),
      true = x,
      false = .data$new_values
    )) %>% # invalid labels should be treated elsewhere
    dplyr::arrange(.data$new_values)


  original_values$x
  ## define new missing values, not with range
  ## A message should be given, these are just suspected to be missing
  new_na_values <- new_value_table$new_values[which(new_value_table$new_values >= 99900)]

  if (!is.null(na_values) & length(na_values) > 0) {
    # The user gets a message if the state missing range (described with labels) is not empty.
    potential_na_values <- original_values$x[which(original_values$x %in% as.numeric(na_values))]
    if (length(potential_na_values) > 0) {
      unique_potential_na_values <- paste(unique(potential_na_values), collapse = ", ")
      warning_message <- glue::glue("Variable {original_x_name} has values {unique_potential_na_values} that you mark as missing.")
      # message ( warning_message)
    }
  }


  new_na_values <- input_na_values

  # define new value - label pairs
  new_labelling <- new_value_table %>%
    dplyr::distinct(.data$new_values, .data$new_labels)
  new_labels <- new_labelling$new_values
  names(new_labels) <- new_labelling$new_labels

  new_labelling <- new_labelling %>%
    mutate(new_labels = ifelse(is.na(.data$new_labels),
      ifelse(.data$new_values %in% input_na_values,
        names(input_na_values), .data$new_labels
      ),
      .data$new_labels
    ))

  # define original labelling
  original_labelling <- new_value_table %>%
    dplyr::distinct(.data$new_values, .data$orig_labels)

  original_labels <- original_labelling$new_values
  names(original_labels) <- original_labelling$orig_labels

  # define original numeric code
  original_numerics <- new_value_table %>%
    dplyr::distinct(.data$new_values, .data$x)
  original_numeric_values <- original_numerics$new_values
  names(original_numeric_values) <- original_numerics$x

  # create new numerics
  new_numerics <- tibble::tibble(x = original_values$x) %>%
    dplyr::left_join(original_numerics, by = "x")

  # ifelse ( is.null(labelled::na_values(x)),
  #         na_values,
  #         c())

  na_value_labels <- na_values

  return_value <- labelled_spss_survey(
    x = new_numerics$new_values,
    labels = labelled::val_labels(x),
    label = harmonize_label,
    na_values = labelled::na_values(x),
    na_range = labelled::na_range(x),
    id = id,
    name_orig = original_x_name
  )

  add_to_value_range <- (!harmonize_labels$to %in% new_labelling$new_labels)
  further_valid_labels <- harmonize_labels$numeric_values[add_to_value_range]
  names(further_valid_labels) <- harmonize_labels$to[add_to_value_range]

  new_valid_range_labels <- sort(
    c(
      new_labels,
      further_valid_labels,
      input_na_values[which(!input_na_values %in% new_labelling$new_labels)]
    )
  )

  new_valid_range_labels_2 <- new_valid_range_labels[unique(names(new_valid_range_labels))]
  new_valid_range_labels_3 <- new_valid_range_labels_2[!is.na(new_valid_range_labels_2)]
  new_valid_range_labels_3

  attr(return_value, "labels") <- new_valid_range_labels_3
  attr(return_value, "na_values") <- unique(new_na_values)
  attr(return_value, paste0(attr(return_value, "id"), "_values")) <- original_numeric_values

  assertthat::assert_that(
    inherits(return_value, "haven_labelled_spss")
  )

  return_value
}

#' @importFrom dplyr distinct_all
#' @keywords internal
get_labelled_attributes <- function(x) {
  unlabelled_x <- sapply(attributes(x), function(i) {
    attributes(i) <- NULL
    x
  })[, 1]

  code_table <- tibble::tibble(
    x = unlabelled_x,
    new_values = unlabelled_x
  )

  code_table$orig_labels <- as_character(x)
  code_table$new_labels <- as_character(x)

  dplyr::distinct_at(code_table, dplyr::vars(
    all_of(c("x", "new_values", "orig_labels", "new_labels"))
  ))
}

#' Merge and harmonize surveys
#'
#' Harmonize variable names, labels, and identifiers across multiple
#' surveys using a metadata crosswalk table.
#'
#' @description
#' `merge_surveys()` applies a harmonization specification to a list
#' of survey objects and returns harmonized survey datasets with
#' aligned variable names and metadata.
#'
#' @details
#' Prior to version 0.2.0 this function was called `merge_waves()`,
#' reflecting terminology commonly used in Eurobarometer surveys.
#'
#' The harmonization table supplied in `var_harmonization`
#' typically originates from [metadata_create()] and contains
#' mappings between original and harmonized variable names.
#'
#' @param survey_list A list of survey objects.
#'
#' @param var_harmonization A metadata table describing the
#'   harmonization rules. The table must contain at least:
#'
#'   - `filename`
#'   - `var_name_orig`
#'   - `var_name_target`
#'   - `var_label`
#'
#' @return
#' A list of harmonized survey objects with standardized variable names
#' and variable labels.
#'
#' @family survey harmonization functions
#'
#' @seealso
#' [metadata_create()]
#'
#' @examples
#' \donttest{
#'
#' examples_dir <- system.file(
#'   "examples",
#'   package = "retroharmonize"
#' )
#'
#' survey_files <- dir(
#'   examples_dir,
#'   pattern = "\\.rds$",
#'   full.names = TRUE
#' )
#'
#' example_surveys <- read_surveys(
#'   survey_files
#' )
#'
#' metadata <- metadata_create(
#'   survey_list = example_surveys
#' )
#'
#' to_harmonize <- metadata %>%
#'   dplyr::filter(
#'     var_name_orig %in% c("rowid", "w1") |
#'       grepl("^trust", var_label_orig)
#'   ) %>%
#'   dplyr::mutate(
#'     var_label = var_label_normalize(var_label_orig),
#'     var_name_target = val_label_normalize(var_label),
#'     var_name_target = ifelse(
#'       .data$var_name_orig %in%
#'         c("rowid", "w1", "wex"),
#'       .data$var_name_orig,
#'       .data$var_name_target
#'     )
#'   )
#'
#' merged_surveys <- merge_surveys(
#'   survey_list = example_surveys,
#'   var_harmonization = to_harmonize
#' )
#'
#' merged_surveys[[1]]
#' }
#'
#' @importFrom dplyr across any_of filter mutate select
#' @importFrom haven is.labelled
#' @importFrom rlang .data set_names
#' @importFrom tidyselect all_of
#'
#' @export

merge_surveys <- function(survey_list, var_harmonization) {
  validate_survey_list(survey_list)

  if (any(!c("filename", "var_name_orig", "var_name_target", "var_label") %in%
    names(var_harmonization))) {
    stop(
      "var_harmonization must contain ",
      paste(c("filename", "var_name_orig", "var_name_target", "var_label"),
        collapse = ", "
      ),
      "."
    )
  }

  fn_merge <- function(dat) {
    select_vars <- var_harmonization %>%
      filter(.data$filename == attr(dat, "filename"))

    if (!"rowid" %in% select_vars$var_name_orig) {
      warning("rowid is not selected from ", attr(dat, "filename"))
    }

    tmp <- dat %>%
      select(all_of(c(select_vars$var_name_orig))) %>%
      rlang::set_names(nm = select_vars$var_name_target)

    labelled_vars <- names(tmp)[vapply(tmp, haven::is.labelled, logical(1))]

    if (length(labelled_vars) > 0) {
      fn_relabel <- function(x) as_labelled_spss_survey(x, id = attr(tmp, "id"))
      tmp <- tmp %>%
        mutate(across(any_of(labelled_vars), fn_relabel))
    }

    if (!is.null(select_vars$var_label)) {
      labelled_items <- vapply(tmp, is.labelled_spss_survey, logical(1))
      labelled_items <- names(labelled_items)[labelled_items]

      labelling <- select_vars %>%
        select(all_of(c("var_name_target", "var_label")))

      fn_relabelling <- function(x) {
        labelling$var_label[which(labelling$var_name_target == x)]
      }

      for (x in labelled_items) {
        attr(tmp[[x]], "label") <- fn_relabelling(x)
      }
    }

    tmp
  }

  lapply(survey_list, fn_merge)
}

#' Deprecated wrapper for `merge_surveys()`
#'
#' `merge_waves()` has been renamed to [merge_surveys()]
#' for more general survey harmonization workflows.
#'
#' @inheritParams merge_surveys
#' @param waves Deprecated alias for `survey_list`.
#'
#' @return
#' A list of harmonized survey objects.
#'
#' @seealso
#' [merge_surveys()]
#'
#' @family survey harmonization functions
#'
#' @export

merge_waves <- function(waves, var_harmonization) {
  merge_surveys(
    survey_list = waves,
    var_harmonization = var_harmonization
  )
}

#' Crosswalk and harmonize surveys
#'
#' Harmonize one or more surveys using a crosswalk table that defines how
#' variable names, value labels, numeric codes, and variable classes should
#' be aligned across surveys.
#'
#' A crosswalk table can be created with [crosswalk_table_create()] or supplied
#' manually as a data frame. At a minimum, the table must contain columns
#' `id`, `var_name_orig`, and `var_name_target`. Additional columns enable
#' harmonization of value labels, numeric codes, missing values, and variable
#' classes.
#'
#' @param crosswalk_table A crosswalk table created with
#'   [crosswalk_table_create()] or a data frame containing at least the columns
#'   `id`, `var_name_orig`, and `var_name_target`.
#'
#'   If the columns `val_label_orig` and `val_label_target` are present,
#'   value labels are harmonized. If `val_numeric_orig` and
#'   `val_numeric_target` are present, numeric codes are harmonized.
#'   If `class_target` is present, variables are coerced to the specified
#'   target class (`"factor"`, `"numeric"`, or `"character"`) using
#'   [as_factor()], [as_numeric()], or [as_character()].
#'
#' @param survey_list A list of survey objects to be harmonized.
#'
#' @param survey_paths Optional character vector of file paths to surveys.
#'   Used when surveys must be read from disk before harmonization.
#'
#' @param import_path Optional base directory used to resolve `survey_paths`.
#'   This is primarily intended for workflows where surveys are stored outside
#'   the current working directory.
#'
#' @param na_values Optional named vector defining numeric codes to be treated
#'   as missing values. Names correspond to missing-value labels.
#'
#' @return
#' `crosswalk_surveys()` returns a list of harmonized survey data frames.
#' `crosswalk()` returns either a single data frame (if only one survey is
#' harmonized) or a merged data frame combining all harmonized surveys.
#'
#' @seealso
#' [crosswalk_table_create()] to create a crosswalk table,
#' [harmonize_survey_variables()] for lower-level variable harmonization.
#'
#' @family harmonization functions
#'
#' @examples
#' \dontrun{
#' examples_dir <- system.file("examples", package = "retroharmonize")
#' survey_files <- dir(examples_dir, pattern = "\\.rds$")
#'
#' surveys <- read_surveys(
#'   file.path(examples_dir, survey_files),
#'   save_to_rds = FALSE
#' )
#'
#' metadata <- metadata_create(survey_list = surveys)
#'
#' crosswalk_table <- crosswalk_table_create(metadata)
#'
#' harmonized <- crosswalk_surveys(
#'   crosswalk_table = crosswalk_table,
#'   survey_list = surveys
#' )
#' }
#'
#' @export
crosswalk_surveys <- function(crosswalk_table,
                              survey_list = NULL,
                              survey_paths = NULL,
                              import_path = NULL,
                              na_values = NULL) {
  # ------------------------------------------------------------------
  # 0. Validate inputs
  # ------------------------------------------------------------------
  is.crosswalk_table(crosswalk_table)

  if (is.null(survey_list)) {
    stop("`survey_list` must be provided.", call. = FALSE)
  }

  # ------------------------------------------------------------------
  # 1. Keep only surveys referenced in the crosswalk table
  # ------------------------------------------------------------------
  survey_ids <- vapply(
    survey_list,
    function(x) attr(x, "id"),
    character(1)
  )

  keep <- survey_ids %in% unique(crosswalk_table$id)
  survey_list <- survey_list[keep]

  if (length(survey_list) == 0) {
    stop("No surveys match IDs found in the crosswalk table.",
      call. = FALSE
    )
  }

  # ------------------------------------------------------------------
  # 2. Harmonize variable names (structural harmonization)
  # ------------------------------------------------------------------
  harmonized_surveys <- harmonize_survey_variables(
    survey_list = survey_list,
    survey_paths = survey_paths,
    import_path = import_path,
    crosswalk_table = crosswalk_table
  )

  # ------------------------------------------------------------------
  # 3. Helper: relabel values within a single survey
  # ------------------------------------------------------------------
  relabel_survey <- function(survey, selection) {
    selection <- selection %>%
      dplyr::filter(!is.na(val_label_orig))

    vars <- unique(selection$var_name_target)
    if (length(vars) == 0) {
      return(survey)
    }

    for (var in vars) {
      mapping <- selection %>%
        dplyr::filter(var_name_target == var)

      if (!is.numeric(mapping$val_numeric_target)) {
        stop("`val_numeric_target` must be numeric.", call. = FALSE)
      }

      # Determine NA values (crosswalk overrides user input)
      local_na_values <- NULL
      if (all(c("na_numeric_target", "na_label_target") %in% names(mapping))) {
        idx <- !is.na(mapping$na_numeric_target)
        if (any(idx)) {
          local_na_values <- mapping$na_numeric_target[idx]
          names(local_na_values) <- mapping$na_label_target[idx]
        }
      } else {
        local_na_values <- na_values
      }

      harmonize_one <- function(x) {
        harmonize_values(
          x = x,
          harmonize_labels = list(
            from = paste0("^", mapping$val_label_orig, "$"),
            to = mapping$val_label_target,
            numeric_values = mapping$val_numeric_target
          ),
          na_values = local_na_values
        )
      }

      survey <- survey %>%
        dplyr::mutate(dplyr::across(dplyr::any_of(var), harmonize_one))
    }

    survey
  }

  # ------------------------------------------------------------------
  # 4. Harmonize a single survey (labels + classes)
  # ------------------------------------------------------------------
  harmonize_one_survey <- function(survey) {
    survey_id <- attr(survey, "id")
    if (length(survey_id) == 0) {
      stop("Survey has no `id` attribute.", call. = FALSE)
    }

    survey <- survey %>%
      dplyr::mutate(id = survey_id) %>%
      dplyr::relocate(id, .before = dplyr::everything())

    selection <- crosswalk_table %>%
      dplyr::filter(id == survey_id)

    if (all(c(
      "val_label_orig", "val_label_target",
      "val_numeric_target"
    ) %in%
      names(selection))) {
      survey <- relabel_survey(survey, selection)
    }

    if (!"class_target" %in% names(selection)) {
      return(survey)
    }

    survey %>%
      dplyr::mutate(
        dplyr::across(
          selection$var_name_target[selection$class_target == "factor"],
          as_factor
        ),
        dplyr::across(
          selection$var_name_target[selection$class_target == "character"],
          as_character
        ),
        dplyr::across(
          selection$var_name_target[selection$class_target == "numeric"],
          as_numeric
        )
      )
  }

  # ------------------------------------------------------------------
  # 5. Apply safely to all surveys
  # ------------------------------------------------------------------
  results <- purrr::map(
    harmonized_surveys,
    purrr::safely(harmonize_one_survey)
  )

  errors <- purrr::map_chr(
    results,
    ~ if (is.null(.x$error)) {
      ""
    } else {
      .x$error$message
    }
  )

  if (any(errors != "")) {
    error_table <- tibble::tibble(
      id = vapply(
        harmonized_surveys,
        function(x) attr(x, "id"), character(1)
      ),
      error = errors
    )

    message("The following surveys could not be harmonized:")
    print(error_table)
    stop("crosswalk_surveys() failed.", call. = FALSE)
  }

  # ------------------------------------------------------------------
  # 6. Return harmonized surveys
  # ------------------------------------------------------------------
  purrr::map(results, "result")
}

#' @title Validate a crosswalk table
#' @rdname crosswalk_table_create
#' @param ctable A table to validate if it is a crosswalk table.
#' @importFrom dplyr tally group_by across filter distinct
#' @family metadata functions
#' @export

is.crosswalk_table <- function(ctable) {
  assert_that(inherits(ctable, "data.frame"),
    msg = "The cross table should be a data frame object (data.frame, tibble, or similar.)"
  )

  assert_that(
    all(c("id", "var_name_orig", "var_name_target") %in% names(ctable)),
    msg = "The crosstable must have at least an 'id' and a 'var_name_orig', 'var_name_target' columns."
  )

  duplicates <- ctable %>%
    dplyr::select(all_of(c("id", "var_name_orig", "var_name_target"))) %>%
    dplyr::distinct(id, var_name_orig, var_name_target, .keep_all = TRUE) %>%
    dplyr::group_by(across(c("var_name_target", "id"))) %>%
    dplyr::tally() %>%
    dplyr::filter(n > 1) %>%
    dplyr::select(var_name_target) %>%
    unlist()

  error_msg <- paste(unique(duplicates), collapse = ", ")
  if (length(duplicates) == 0) error_msg <- ""

  assert_that(
    length(duplicates) == 0,
    msg = glue(
      "The crosstable '{deparse(substitute(ctable))}' has the following non-unique target variables: {error_msg}."
    )
  )
}

#' @title Create a crosswalk table
#'
#' @description Create a crosswalk table with the source variable names
#' and variable labels.
#'
#' @details The table contains a \code{var_name_target} and
#' \code{val_label_target} column, but
#' these values need to be set by further manual or
#' reproducible harmonization steps.
#'
#' @param metadata A metadata table created by
#' [metadata_create()].
#' @return A tibble with raw crosswalk table. It contains all
#' harmonization tasks, but the
#' target values need to be set by further manipulations.
#' @importFrom glue glue
#' @importFrom assertthat assert_that
#' @importFrom dplyr full_join
#' @importFrom purrr reduce
#' @family harmonization functions
#' @export

crosswalk_table_create <- function(metadata) {
  assert_that(inherits(metadata, "data.frame"),
    msg = "Parameter 'metadata' must be a data frame object."
  )

  assert_that(nrow(metadata) >= 1,
    msg = "The 'metadata' data frame must have at least one row."
  )

  compulsory_columns <- c("filename", "id", "var_name_orig", "labels")

  missing_columns <- setdiff(compulsory_columns, names(metadata))

  assert_that(
    length(missing_columns) == 0,
    msg = glue("The 'metadata' data frame has missing columns: {paste(missing_columns, collapse = ", "}")
  )

  fn_labels <- function(x) {
    if (is.na(x$labels)) {
      # The variable is not labelled
      label_length <- 1
      tibble(
        id = rep(unique(x$id), label_length),
        filename = rep(unique(x$filename), label_length),
        var_name_orig = rep(x$var_name_orig, label_length),
        var_name_target = rep(x$var_name_orig, label_length),
        val_numeric_orig = NA_real_,
        val_numeric_target = NA_real_,
        val_label_orig = NA_character_,
        val_label_target = NA_character_,
        class_orig = rep(unique(x$class_orig), label_length),
        class_target = rep(unique(x$class_orig), label_length)
      )
    } else {
      val_labels <- names(unlist(x$labels))
      label_length <- length(unlist(x$labels))

      tmp <- tibble(
        id = rep(unique(x$id), label_length),
        filename = rep(unique(x$filename), label_length),
        var_name_orig = rep(x$var_name_orig, label_length),
        var_name_target = rep(x$var_name_orig, label_length),
        val_numeric_orig = as.numeric(unlist(x$labels)),
        val_numeric_target = as.numeric(unlist(x$labels)),
        val_label_orig = as.character(vapply(x$labels, names, character(label_length))),
        val_label_target = as.character(vapply(x$labels, names, character(label_length)))
      )

      if ("na_labels" %in% names(x)) {
        na_labels <- names(unlist(x$na_labels))

        tmp$na_label_orig <- ifelse(tmp$val_label_orig %in% na_labels,
          tmp$val_label_orig, NA_character_
        )
        tmp$na_label_target <- tmp$na_label_orig

        tmp$na_numeric_orig <- ifelse(tmp$val_label_orig %in% na_labels,
          tmp$val_numeric_orig, NA_real_
        )
        tmp$na_numeric_target <- tmp$na_numeric_orig
      } else {
        tmp$na_label_orig <- NA_character_
        tmp$na_label_target <- NA_character_
        tmp$na_numeric_orig <- NA_character_
        tmp$na_numeric_target <- NA_real_
      }

      if ("var_label_orig" %in% names(x)) {
        tmp$var_label_orig <- x$var_label_orig
        tmp$var_label_target <- tmp$var_label_orig
      } else {
        tmp$var_label_orig <- NA_character_
        tmp$var_label_target <- NA_character_
      }
      if ("class_orig" %in% names(x)) {
        tmp <- tmp %>%
          left_join(x %>% select(all_of(c("var_name_orig", "class_orig"))),
            by = "var_name_orig"
          )

        tmp$class_target <- case_when(
          tmp$class_orig %in% c("numeric", "character") ~ tmp$class_orig,
          TRUE ~ "factor"
        )
      } else {
        tmp$class_orig <- NA_character_
        tmp$class_target <- NA_character_
      }
      tmp
    }
  }

  if (nrow(metadata) == 1) {
    fn_labels(x = metadata[1, ])
  } else {
    ctable_list <- lapply(1:nrow(metadata), function(x) fn_labels(metadata[x, ]))
    ctable <- suppressMessages(purrr::reduce(ctable_list, full_join))
    ctable
  }
}

#' @rdname crosswalk_surveys
#' @importFrom purrr reduce
#' @importFrom dplyr full_join
#' @export
crosswalk <- function(survey_list,
                      crosswalk_table,
                      na_values = NULL) {
  crosswalked <- crosswalk_surveys(
    survey_list = survey_list,
    crosswalk_table = crosswalk_table,
    na_values = na_values
  )

  if (!is.null(crosswalked) & length(crosswalked) > 1) {
    suppressMessages(purrr::reduce(crosswalked, full_join))
  } else {
    crosswalked
  }
}

test_survey <- read_rds(
  file = system.file("examples", "ZA7576.rds",
    package = "retroharmonize"
  )
)

test_metadata <- metadata_create(test_survey)
test_metadata <- test_metadata[c(1, 7, 18), ]
ctable_2 <- crosswalk_table_create(test_metadata)
ctable_2$var_name_target <- ifelse(ctable_2$var_name_orig == "qa14_3",
  "trust_ecb",
  ctable_2$var_name_orig
)

subsetted <- subset_surveys(
  crosswalk_table = ctable_2,
  subset_name = "tested",
  survey_list = test_survey,
  import_path = NULL
)


test_that("multiplication works", {
  expect_true(is.survey(subsetted[[1]]))
})

examples_dir <- system.file("examples", package = "retroharmonize")
survey_list <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
example_surveys <- read_surveys(
  file.path(examples_dir, survey_list),
  export_path = NULL
)

documented_surveys <- metadata_create(example_surveys)
documented_surveys <- documented_surveys[
  documented_surveys$var_name_orig %in% c(
    "rowid", "isocntry", "w1", "qd3_4",
    "qd3_8", "qd7.4", "qd7.8", "qd6.4", "qd6.8"
  ),
]
crosswalk_table <- crosswalk_table_create(metadata = documented_surveys)

freedom_table <- crosswalk_table[
  which(crosswalk_table$var_name_target %in% c("rowid", "freedom")),
]

harmonized <- harmonize_survey_variables(
  crosswalk_table = freedom_table,
  subset_name = "freedom",
  survey_list = example_surveys
)


test_that("harmonize_survey_variables() subsets and renames as expected", {
  expect_true(is.list(harmonized))
})

examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
example_surveys <- read_surveys(
  file.path(examples_dir, survey_files),
  export_path = NULL
)

subset_survey_list_1 <- subset_surveys(
  survey_list = example_surveys,
  subset_vars = c("rowid", "isocntry", "qa10_1", "qa14_1"),
  subset_name = "subset_example"
)

test_that("saving and subsetting (not on CRAN)", {
  expect_equal(
    vapply(subset_survey_list_1, function(x) attr(x, "id"), character(1)),
    c("ZA5913", "ZA6863", "ZA7576")
  )
  expect_equal(
    vapply(subset_survey_list_1, function(x) attr(x, "filename"), character(1)),
    c("ZA5913_subset_example.rds", "ZA6863_subset_example.rds", "ZA7576_subset_example.rds")
  )
  expect_equal(vapply(subset_survey_list_1, ncol, numeric(1)), c(3, 3, 3))
})


example_metadata <- metadata_create(
  survey_paths = file.path(examples_dir, survey_files),
  .f = "read_rds"
)

example_ctable <- example_metadata %>%
  filter(.data$var_name_orig %in% c("rowid", "isocntry", "d60", "wex")) %>%
  crosswalk_table_create()

crosswalk_table <- example_ctable
survey_paths <- file.path(examples_dir, survey_files)

testing_subsetting <- function() {
  test_survey <- read_rds(
    file = system.file("examples", "ZA7576.rds",
      package = "retroharmonize"
    )
  )

  test_metadata <- metadata_create(test_survey)
  test_metadata <- test_metadata[c(1, 7, 18), ]
  ctable <- crosswalk_table_create(test_metadata)
  ctable$var_name_target <- ifelse(ctable$var_name_orig == "qa14_3",
    "trust_ecb",
    ctable$var_name_orig
  )

  temporary_directory <- tempdir()

  temporary_saving_location <- file.path(temporary_directory, "ZA7576.rds")

  saveRDS(test_survey, temporary_saving_location, version = 2)

  file_result <- subset_surveys(
    crosswalk_table = ctable,
    survey_list = test_survey,
    subset_name = "tested",
    survey_paths = NULL,
    import_path = NULL,
    export_path = tempdir()
  )


  file_result == "ZA7576_tested.rds" &
    file.exists(file.path(tempdir(), "ZA7576_tested.rds"))
}

res <- evaluate_promise(testing_subsetting())

test_that("saving and subsetting", {
  skip_on_cran()
  expect_true("Saving ZA7576_tested.rds\n" %in% evaluate_promise(testing_subsetting())$messages)
})

test_that("saving and subsetting (not on CRAN)", {
  skip_on_cran()
  expect_true(ncol(readRDS(file.path(tempdir(), "ZA7576_tested.rds"))) == 3)
  expect_error(save_surveys(
    crosswalk_table = ctable,
    subset_name = "tested",
    import_path = NULL,
    export_path = NULL
  ))
})

load_example_surveys <- function() {
  examples_dir <- system.file("examples", package = "retroharmonize")
  files <- dir(examples_dir, pattern = "\\.rds$")
  read_surveys(
    file.path(examples_dir, files),
    export_path = NULL
  )
}

make_base_crosswalk <- function() {
  surveys <- load_example_surveys()

  metadata <- metadata_create(surveys)
  metadata <- metadata[
    metadata$var_name_orig %in% c(
      "rowid", "isocntry", "w1",
      "qd3_4", "qd3_8",
      "qd7.4", "qd7.8",
      "qd6.4", "qd6.8"
    ),
  ]

  list(
    surveys = surveys,
    metadata = metadata,
    crosswalk = crosswalk_table_create(metadata)
  )
}

test_that("crosswalk_table_create() creates a valid crosswalk schema", {
  obj <- make_base_crosswalk()
  cw <- obj$crosswalk

  expect_s3_class(cw, "data.frame")

  expect_true(
    all(
      c(
        "id", "filename",
        "var_name_orig", "var_name_target",
        "val_numeric_orig", "val_numeric_target",
        "val_label_orig", "val_label_target",
        "na_label_orig", "na_label_target",
        "na_numeric_orig", "na_numeric_target",
        "var_label_orig", "var_label_target",
        "class_orig", "class_target"
      ) %in% names(cw)
    )
  )

  expect_gte(nrow(cw), nrow(obj$metadata))
})

test_that("is.crosswalk_table() validates uniqueness of target variables", {
  obj <- make_base_crosswalk()

  expect_silent(is.crosswalk_table(obj$crosswalk))

  bad <- obj$crosswalk
  bad$var_name_target[bad$id == bad$id[1]][1:2] <- "dup"

  expect_error(
    is.crosswalk_table(bad),
    "non-unique target variables"
  )
})

test_that("harmonize_survey_variables() subsets and renames variables", {
  obj <- make_base_crosswalk()

  freedom_table <- obj$crosswalk[
    obj$crosswalk$var_name_target %in% c("rowid", "freedom"),
  ]

  out <- harmonize_survey_variables(
    survey_list = obj$surveys,
    crosswalk_table = freedom_table,
    subset_name = "freedom"
  )

  expect_length(out, length(obj$surveys))

  for (s in out) {
    expect_true(all(names(s) %in% c("rowid", "freedom")))
  }
})

test_that("harmonize_survey_variables() writes files when export_path is set", {
  obj <- make_base_crosswalk()

  files <- harmonize_survey_variables(
    survey_list = obj$surveys,
    crosswalk_table = obj$crosswalk,
    export_path = tempdir()
  )

  expect_type(files, "character")
  expect_length(files, length(obj$surveys))

  expect_true(
    all(fs::file_exists(file.path(tempdir(), files)))
  )
})

test_that("crosswalk_surveys() returns a list of harmonized surveys", {
  obj <- make_base_crosswalk()

  out <- crosswalk_surveys(
    survey_list = obj$surveys,
    crosswalk_table = obj$crosswalk
  )

  expect_type(out, "list")
  expect_length(out, length(obj$surveys))

  expected_rows <- sum(vapply(obj$surveys, nrow, numeric(1)))
  actual_rows <- sum(vapply(out, nrow, numeric(1)))

  expect_equal(actual_rows, expected_rows)
})

test_that("crosswalk_surveys() harmonizes names and labels correctly", {
  obj <- make_base_crosswalk()
  cw <- obj$crosswalk

  freedom_vars <- obj$metadata$var_name_orig[
    grepl("freedom", obj$metadata$var_label_orig)
  ]

  solidarity_vars <- obj$metadata$var_name_orig[
    grepl("solidarity", obj$metadata$var_label_orig)
  ]

  cw$var_name_target <- ifelse(
    cw$var_name_orig %in% freedom_vars, "freedom",
    ifelse(
      cw$var_name_orig %in% solidarity_vars,
      "solidarity",
      cw$var_name_orig
    )
  )

  cw$val_label_target <- ifelse(
    grepl("Mentioned|freedom|Solidarity", cw$val_label_orig),
    "mentioned",
    ifelse(
      cw$val_label_orig == "Not mentioned",
      "not_mentioned",
      ifelse(grepl("Inap", cw$val_label_orig), "inap", NA)
    )
  )

  cw$val_numeric_target <- ifelse(
    cw$val_label_target == "inap",
    99999,
    cw$val_numeric_orig
  )

  out <- crosswalk_surveys(
    survey_list = obj$surveys,
    crosswalk_table = cw,
    na_values = c("inap" = 99999)
  )

  expect_true(
    all(names(out[[1]]) ==
      c("id", "rowid", "isocntry", "freedom", "solidarity", "w1"))
  )

  all_values <- unique(unlist(lapply(out, function(x) as_character(x$freedom))))

  expect_true(
    all(all_values %in% c("mentioned", "not_mentioned", "inap"))
  )
})

test_that("crosswalk() returns a combined data frame", {
  obj <- make_base_crosswalk()

  out <- crosswalk(
    survey_list = obj$surveys,
    crosswalk_table = obj$crosswalk
  )

  expect_s3_class(out, "data.frame")

  expected_rows <- sum(vapply(obj$surveys, nrow, numeric(1)))
  expect_equal(nrow(out), expected_rows)
})

#' Pipe operator
#'
#' See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
NULL

#' Labelled to labelled_spss_survey
#'
#' @param x A vector of class haven_labelled or haven_labelled_spss.
#' @param id The survey identifier.
#' @return A vector of labelled_spss_survey
#' @family type conversion functions
#' @export

as_labelled_spss_survey <- function(x, id) {
  assertthat::assert_that(
    haven::is.labelled(x),
    msg = paste0(
      deparse(substitute(x)),
      " should be a haven_labelled or a class that inherits its labels."
    )
  )

  labelled_spss_survey(
    x         = vctrs::vec_data(x),
    labels    = labelled::val_labels(x),
    label     = labelled::var_label(x),
    na_values = labelled::na_values(x),
    na_range  = labelled::na_range(x),
    name_orig = deparse(substitute(x)),
    id        = id
  )
}

test_that("as_labelled_spss_survey converts labelled_spss correctly", {
  x <- haven::labelled_spss(
    x = c(1, 2, 9),
    labels = c(yes = 1, no = 2, dk = 9),
    na_values = 9,
    label = "Question"
  )

  result <- as_labelled_spss_survey(x, id = "za001")

  expect_s3_class(
    result,
    "retroharmonize_labelled_spss_survey"
  )

  expect_equal(
    labelled::val_labels(result),
    c(yes = 1, no = 2, dk = 9)
  )

  expect_equal(
    labelled::na_values(result),
    9
  )

  expect_equal(
    attr(result, "label"),
    "Question"
  )

  expect_equal(
    attr(result, "id"),
    "za001"
  )
})

test_that("as_labelled_spss_survey converts haven_labelled correctly", {
  x <- labelled::labelled(
    c(1, 2, 1),
    labels = c(yes = 1, no = 2)
  )

  result <- as_labelled_spss_survey(x, id = "survey_a")

  expect_s3_class(
    result,
    "retroharmonize_labelled_spss_survey"
  )

  expect_equal(
    labelled::val_labels(result),
    c(yes = 1, no = 2)
  )

  expect_equal(
    attr(result, "id"),
    "survey_a"
  )
})

test_that("as_labelled_spss_survey stores original object name", {
  my_variable <- haven::labelled_spss(
    x = c(1, 2),
    labels = c(yes = 1, no = 2)
  )

  result <- as_labelled_spss_survey(
    my_variable,
    id = "s1"
  )

  expect_equal(
    attr(result, "s1_name"),
    "my_variable"
  )
})

test_that("as_labelled_spss_survey errors on non-labelled input", {
  expect_error(
    as_labelled_spss_survey(
      c(1, 2, 3),
      id = "bad"
    ),
    "should be a haven_labelled"
  )
})

#' Retrieve a survey from a survey list
#'
#' Extract a single `survey` object from a list of surveys using either
#' its survey identifier or source file name.
#'
#' @description
#' `pull_survey()` retrieves a survey object from a list created with
#' [read_surveys()].
#'
#' Surveys can be selected using:
#'
#' - the survey identifier stored in the `"id"` attribute, or
#' - the original source file name stored in the `"filename"` attribute.
#'
#' @param survey_list A list of `survey` objects.
#'
#' @param id Optional survey identifier.
#'
#' @param filename Optional source file name.
#'
#' @return
#' A single `survey` object.
#'
#' @details
#' Either `id` or `filename` must be supplied.
#'
#' The function throws an error if:
#'
#' - neither argument is provided;
#' - the requested survey cannot be found;
#' - or multiple surveys match the query.
#'
#' @family import functions
#'
#' @examples
#' examples_dir <- system.file(
#'   "examples",
#'   package = "retroharmonize"
#' )
#'
#' survey_files <- dir(
#'   examples_dir,
#'   pattern = "\\.rds$"
#' )
#'
#' example_surveys <- read_surveys(
#'   file.path(examples_dir, survey_files)
#' )
#'
#' pull_survey(
#'   example_surveys,
#'   id = "ZA5913"
#' )
#'
#' @seealso
#' [read_surveys()]
#'
#' @export

pull_survey <- function(survey_list, id = NULL, filename = NULL) {
  if (is.null(id) && is.null(filename)) {
    stop("Either the id or the filename must be given")
  }

  if (!is.null(id)) {
    ids <- vapply(survey_list, function(x) attr(x, "id"), character(1))
    selected_id <- which(ids == id)

    if (length(selected_id) > 1) {
      stop("The id='", id, "' is not unique.")
    }
    if (length(selected_id) == 0) {
      stop("The id='", id, "' is not present.")
    }
    return(survey_list[[selected_id]])
  }

  filenames <- vapply(survey_list, function(x) attr(x, "filename"), character(1))
  selected_file <- which(filenames == filename)
  if (length(selected_file) > 1) {
    stop("The filenames='", filenames, "' is not unique.")
  }
  if (length(selected_file) == 0) {
    stop("The filenames='", filenames, "' is not present.")
  }

  survey_list[[selected_file]]
}

test_that("pull_survey() retrieves surveys by id", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  result <- pull_survey(
    example_surveys,
    id = "ZA5913"
  )

  expect_true(
    inherits(result, "survey")
  )

  expect_equal(
    nrow(result),
    35
  )

  expect_equal(
    attr(result, "id"),
    "ZA5913"
  )
})

test_that("pull_survey() retrieves surveys by filename", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  result <- pull_survey(
    example_surveys,
    filename = "ZA5913.rds"
  )

  expect_true(
    inherits(result, "survey")
  )

  expect_equal(
    nrow(result),
    35
  )

  expect_equal(
    attr(result, "filename"),
    "ZA5913.rds"
  )
})

test_that("pull_survey() errors when neither id nor filename is supplied", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  expect_error(
    pull_survey(example_surveys),
    "Either the id or the filename must be given"
  )
})

test_that("pull_survey() errors when id is not found", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  expect_error(
    pull_survey(
      example_surveys,
      id = "missing_survey"
    ),
    "is not present"
  )
})

test_that("pull_survey() errors when filename is not found", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  expect_error(
    pull_survey(
      example_surveys,
      filename = "missing_file.rds"
    ),
    "is not present"
  )
})

#' Create metadata tables from survey datasets
#'
#' Create a variable-level metadata table from one or more survey
#' datasets. Metadata are extracted either from survey objects already
#' loaded into memory or directly from survey files.
#'
#' The resulting metadata table contains information about:
#'
#' \itemize{
#'   \item variable names and labels,
#'   \item storage classes,
#'   \item value labels,
#'   \item user-defined missing values,
#'   \item and missing value ranges.
#' }
#'
#' `metadata_create()` is a convenience wrapper around repeated
#' [metadata_survey_create()] calls.
#'
#' @param survey_paths Optional character vector containing paths to
#'   survey files.
#'
#' @param survey_list Optional list of survey objects of class
#'   [survey()].
#'
#' @param .f Import function used to read surveys from
#'   `survey_paths`. When `NULL`, the import function is inferred from
#'   the file extension.
#'
#' @return A data frame containing variable-level survey metadata.
#'
#' @examples
#' examples_dir <- system.file(
#'   "examples",
#'   package = "retroharmonize"
#' )
#'
#' my_rds_files <- dir(examples_dir)[grepl(
#'   "\\.rds$",
#'   dir(examples_dir)
#' )]
#'
#' example_surveys <- read_surveys(
#'   file.path(examples_dir, my_rds_files)
#' )
#'
#' metadata_create(example_surveys)
#'
#' @family metadata functions
#' @seealso [metadata_survey_create()], [create_variable_catalog()]
#' @export

metadata_create <- function(survey_list = NULL,
                            survey_paths = NULL,
                            .f = NULL) {
  if (!is.null(survey_list)) {
    validate_survey_list(survey_list)
    if (!"list" %in% class(survey_list)) {
      assert_that(is.survey(survey_list),
        msg = "metadata_create(survey_list, ...) is neither a list nor a survey."
      )
      survey_id <- attr(survey_list, "id")
      survey_list <- list(i = survey_list)
      names(survey_list)[1] <- survey_id
    }
    metadata_list <- lapply(survey_list, metadata_survey_create)
    do.call(rbind, metadata_list)
  } else if (is.null(survey_paths)) {
    stop("Error in metadata_surveys_create(): both 'survey_list' and 'survey_paths' are NULL.")
  } else {
    validate_survey_files(survey_paths)
    read_survey_create_metadata <- function(x, .f) {
      tmp <- read_survey(x, .f)
      message("Read: ", x)
      metadata_survey_create(tmp)
    }

    metadata_list <- lapply(
      X = survey_paths,
      FUN = function(x) read_survey_create_metadata(x, .f)
    )
    do.call(rbind, metadata_list)
  }
}

#' @rdname metadata_create
#' @details The form \code{metadata_waves_create} is deprecated.

metadata_waves_create <- function(survey_list) {
  .Deprecated(
    new = "metadata_surveys_create",
    msg = "metadata_waves_create() is deprecated, use create_surveys_metadata() instead",
    old = "merge_waves"
  )
  metadata_survey_create(survey_list)
}

#' Create variable-level metadata from a survey dataset
#'
#' Extract variable-level metadata from a survey dataset and return
#' the result as a nested data frame.
#'
#' The metadata table contains:
#'
#' \itemize{
#'   \item variable names and labels,
#'   \item imported storage classes,
#'   \item value labels,
#'   \item user-defined missing values,
#'   \item missing value ranges,
#'   \item and summary counts of labelled categories.
#' }
#'
#' For multiple surveys, use [metadata_create()], which applies
#' `metadata_survey_create()` across a list of surveys or survey files.
#'
#' @param survey A survey object of class [survey()].
#'
#'   Survey objects are typically created with:
#'
#'   \itemize{
#'     \item [read_rds()]
#'     \item [read_spss()]
#'     \item [read_dta()]
#'     \item [read_csv()]
#'     \item [read_survey()]
#'   }
#'
#'   Survey objects can also be created manually from a data frame
#'   with [survey()].
#'
#' @return A nested data frame containing:
#'
#' \describe{
#'   \item{filename}{Original survey file name.}
#'   \item{id}{Survey identifier.}
#'   \item{var_name_orig}{Original variable name.}
#'   \item{class_orig}{Imported storage class.}
#'   \item{var_label_orig}{Original variable label.}
#'   \item{labels}{List column of value labels.}
#'   \item{valid_labels}{List column of non-missing value labels.}
#'   \item{na_labels}{List column of user-defined missing labels.}
#'   \item{na_range}{List column containing user-defined missing ranges.}
#'   \item{n_labels}{Number of labelled categories.}
#'   \item{n_valid_labels}{Number of non-missing categories.}
#'   \item{n_na_labels}{Number of missing categories.}
#' }
#'
#' @examples
#' metadata_survey_create(
#'   survey = read_rds(
#'     system.file(
#'       "examples",
#'       "ZA7576.rds",
#'       package = "retroharmonize"
#'     )
#'   )
#' )
#'
#' @importFrom assertthat assert_that
#' @importFrom dplyr group_by left_join mutate select ungroup
#' @importFrom labelled na_range na_values val_labels var_label
#' @importFrom purrr map
#' @importFrom tibble tibble
#' @importFrom tidyr nest unnest
#'
#' @family metadata functions
#' @seealso [metadata_create()], [create_variable_catalog()]
#' @export

metadata_survey_create <- function(survey) {
  ## Assertions before running the function -----------------------------
  if ("list" %in% class(survey)) {
    assert_that(all(vapply(survey, is.survey, logical(1))),
      msg = "Parameter 'survey' is not of s3 class survey or a list of them. See ?is.survey."
    )
    metadata_df <- metadata_create(survey_list = survey)
    return(metadata_df)
  } else if (
    # Accidentally the file names were supplied.
    # This will validate if the surveys are indeed existing files.
    is.character(survey)) {
    warning("The parameter 'survey' is not a single survey but a character vector. Try to understand them as a file names. See ?metadata_create.")
    metadata_df <- metadata_create(survey_list = survey)
    return(metadata_df)
  } else {
    assert_that(is.survey(survey),
      msg = "Parameter 'survey' must be of s3 class survey. See ?is.survey."
    )
  }

  filename <- attr(survey, "filename")

  if (is.null(filename)) filename <- "unknown"

  id <- ifelse(is.null(attr(survey, "id")), attr(survey, "identifier"), attr(survey, "id"))
  if (is.null(id)) id <- "missing"

  if (ncol(survey) == 0) {
    # Special case when file could not be read and survey is empty
    return(metadata_initialize(
      filename = filename,
      id = paste0(filename, " could not be read.")
    ))
  }

  var_label_orig <- lapply(survey, labelled::var_label)

  class_orig <- vapply(survey, function(x) class(x)[1], character(1))

  metadata <- tibble(
    filename = filename,
    id = id,
    var_name_orig = names(survey),
    class_orig = class_orig,
    var_label_orig = ifelse(vapply(var_label_orig, is.null, logical(1)),
      "",
      unlist(var_label_orig)
    ) %>%
      as.character() %>%
      var_label_normalize()
  )

  fn_valid_range <- function(x) {
    labelled::val_labels(x)[!labelled::val_labels(x) %in% labelled::na_values(x)]
  }

  na_labels <- function(x) {
    # labels that refer to na_values
    labs <- labelled::val_labels(x)
    if (is.null(labs)) {
      return(NA_character_)
    }
    selected_labs <- labelled::na_values(x)
    labs[labs %in% selected_labs]
  }

  to_list_column <- function(.f = "na_values") {
    if (.f == "na_labels") {
      x <- sapply(
        survey,
        na_labels
      )
    } else if (.f == "na_range") {
      x <- sapply(
        survey,
        labelled::na_range
      )
    } else if (.f == "valid_range") {
      x <- sapply(
        survey,
        fn_valid_range
      )
    } else if (.f == "labels") {
      x <- sapply(
        survey,
        labelled::val_labels
      )
    } else {
      stop(
        "Unknown metadata field: ",
        .f
      )
    }

    x[vapply(x, is.null, logical(1))] <- NA_character_

    names(x) <- names(survey)

    x
  }

  range_df <- tibble::tibble(
    var_name_orig = names(survey),
    labels = rep(NA_character_, length(names(survey))),
    valid_labels = rep(NA_character_, length(names(survey))),
    na_labels = rep(NA_character_, length(names(survey))),
    na_range = rep(NA_character_, length(names(survey))),
    n_labels = rep(0, length(names(survey))),
    n_valid_labels = rep(0, length(names(survey))),
    n_na_labels = rep(0, length(names(survey)))
  )

  if (
    any(vapply(
      lapply(survey, class),
      function(x) any(grepl("labelled", x)),
      logical(1)
    ))
  ) {
    range_df <- tibble::tibble(
      var_name_orig = names(survey),
      labels = to_list_column(.f = "labels"),
      valid_labels = to_list_column(.f = "valid_range"),
      na_labels = to_list_column(.f = "na_labels"),
      na_range = to_list_column(.f = "na_range")
    )
    label_length <- function(x) {
      ifelse(is.na(x[[1]])[1] | length(x[[1]]) == 0,
        0, length(x[[1]])
      )
    }

    range_df$n_labels <- vapply(
      1:nrow(range_df),
      function(x) label_length(range_df$labels[x]),
      numeric(1)
    )
    range_df$n_valid_labels <- vapply(
      1:nrow(range_df),
      function(x) label_length(range_df$valid_labels[x]),
      numeric(1)
    )
    range_df$n_na_labels <- vapply(
      1:nrow(range_df),
      function(x) label_length(range_df$na_labels[x]),
      numeric(1)
    )
  } else {
    ## Special case when there are no labelled variables present
    return(
      metadata %>%
        left_join(range_df,
          by = "var_name_orig"
        ) %>%
        as.data.frame()
    )
  }

  return_df <- metadata %>%
    left_join(
      range_df %>%
        group_by(var_name_orig) %>%
        tidyr::nest(),
      by = "var_name_orig"
    ) %>%
    tidyr::unnest(cols = "data") %>%
    ungroup() %>%
    mutate(
      n_na_labels = as.numeric(n_na_labels),
      n_valid_labels = as.numeric(n_valid_labels),
      n_labels = as.numeric(n_labels)
    ) %>%
    as.data.frame()

  change_label_to_empty <- function() {
    "none" <- NA_real_
  }
  ## Avoid the accidental creation of empty CHARACTER lists, because they do not bind with
  ## numeric lists.

  return_df$label_type <- vapply(return_df$labels, function(x) class(x)[1], character(1))
  return_dflabels <- ifelse(return_df$label_type == "character" & return_df$n_labels == 0,
    yes = change_label_to_empty(),
    no = return_df$labels
  )
  return_df$valid_labels <- ifelse(return_df$label_type == "character" & return_df$n_labels == 0,
    yes = change_label_to_empty(),
    no = return_df$valid_labels
  )
  return_df$na_labels <- ifelse(return_df$label_type == "character" & return_df$n_labels == 0,
    yes = change_label_to_empty(),
    no = return_df$na_labels
  )

  return_df %>%
    select(-label_type)
}

# -----------------------------------------------------------------------
#' @title Initialize a metadata data frame
#'
#' @importFrom tibble tibble
#' @param filename A file name
#' @param id An id.
#' @return A nested data frame with metadata and the range of
#' labels, na_values and the na_range itself.
#' @keywords internal
metadata_initialize <- function(filename, id) {
  tibble(
    filename = filename,
    id = id,
    class_orig = NA_character_,
    var_name_orig = NA_character_,
    var_label_orig = NA_character_,
    labels = NA_character_,
    valid_labels = list("none" = NA_real_),
    na_labels = list("none" = NA_real_),
    na_range = list("none" = NA_real_),
    n_labels = 0,
    n_valid_labels = 0,
    n_na_labels = 0
  )
}

test_that("harmonize_var_names() returns a list", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$var_name_suggested <-
    metadata$var_name_orig

  result <- harmonize_var_names(
    survey_list = example_surveys,
    metadata = metadata
  )

  expect_true(is.list(result))
  expect_equal(length(result), length(example_surveys))
})

test_that("harmonize_var_names() preserves survey dimensions", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$var_name_suggested <-
    metadata$var_name_orig

  result <- harmonize_var_names(
    survey_list = example_surveys,
    metadata = metadata
  )

  expect_equal(
    nrow(result[[1]]),
    nrow(example_surveys[[1]])
  )

  expect_equal(
    ncol(result[[1]]),
    ncol(example_surveys[[1]])
  )
})

test_that("harmonize_var_names() renames variables", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$var_name_suggested <-
    metadata$var_name_orig

  metadata$var_name_suggested[
    metadata$var_name_orig == "rowid"
  ] <- "uniqid"

  result <- harmonize_var_names(
    survey_list = example_surveys,
    metadata = metadata
  )

  expect_true(
    "uniqid" %in% names(result[[1]])
  )

  expect_false(
    "rowid" %in% names(result[[1]])
  )
})

test_that("harmonize_var_names() subsets variables", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata <- metadata %>%
    filter(
      var_name_orig %in% c(
        "rowid",
        "w1"
      )
    ) %>%
    mutate(
      var_name_suggested = var_name_orig
    )

  result <- harmonize_var_names(
    survey_list = example_surveys,
    metadata = metadata
  )

  expect_equal(
    names(result[[1]]),
    c("rowid", "w1")
  )
})

test_that("harmonize_var_names() errors on missing metadata columns", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$wrong_name <-
    metadata$var_name_orig

  expect_error(
    harmonize_var_names(
      survey_list = example_surveys,
      metadata = metadata,
      new = "var_name_suggested"
    )
  )
})

test_that("harmonize_var_names() warns when metadata is missing for a survey", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$var_name_suggested <-
    metadata$var_name_orig

  metadata <- metadata[
    metadata$filename != attr(example_surveys[[1]], "filename"),
  ]

  expect_warning(
    harmonize_var_names(
      survey_list = example_surveys,
      metadata = metadata
    )
  )
})

test_that("harmonize_var_names() preserves survey class", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  survey_files <- dir(
    examples_dir,
    pattern = "\\.rds$"
  )

  example_surveys <- read_surveys(
    file.path(examples_dir, survey_files)
  )

  metadata <- metadata_create(
    example_surveys
  )

  metadata$var_name_suggested <-
    metadata$var_name_orig

  result <- harmonize_var_names(
    survey_list = example_surveys,
    metadata = metadata
  )

  expect_true(
    inherits(result[[1]], "survey")
  )
})

test_that("Only surveys are accepted", {
  expect_error(
    metadata_survey_create(
      data.frame(
        a = 1:2,
        b = c("b", "C")
      )
    )
  )
})

test_that("Correct values are returned", {
  test_survey <- read_rds(
    file = system.file(
      "examples",
      "ZA7576.rds",
      package = "retroharmonize"
    )
  )

  example_metadata <- metadata_survey_create(
    survey = test_survey
  )

  q_labels <- length(
    labelled::val_labels(test_survey$qd6.12)
  )

  q_na <- length(
    labelled::na_values(test_survey$qd6.12)
  )

  test_value <- example_metadata[
    which(example_metadata$var_name_orig == "qd6.12"),
  ]

  test_value2 <- example_metadata[
    which(example_metadata$var_name_orig == "qg8"),
  ]

  expect_equal(ncol(example_metadata), 12)

  expect_equal(
    example_metadata$var_name_orig[1],
    "rowid"
  )

  expect_equal(
    unique(example_metadata$filename),
    "ZA7576.rds"
  )

  expect_equal(
    as.character(unlist(example_metadata$na_labels[2])),
    NA_character_
  )

  expect_equal(
    example_metadata$var_label_orig[1],
    "unique_identifier_in_za_7576"
  )

  expect_equal(
    length(test_value$na_labels),
    q_na
  )

  expect_equal(
    test_value$n_valid_labels,
    q_labels - q_na
  )

  expect_equal(
    c(
      length(test_value2$labels[[1]]),
      length(test_value2$valid_labels[[1]]),
      length(test_value2$na_labels[[1]])
    ),
    c(8, 6, 2)
  )
})

test_that("Correct values are returned from multiple surveys", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  my_rds_files <- dir(examples_dir)[grepl(
    "\\.rds$",
    dir(examples_dir)
  )]

  example_surveys <- read_surveys(
    file.path(examples_dir, my_rds_files)
  )

  metadata_multiple_surveys <- metadata_create(
    example_surveys
  )

  expect_equal(
    metadata_multiple_surveys$var_name_orig[1],
    "rowid"
  )
})


test_that("metadata_survey_create works without labelled variables", {
  plain_df <- data.frame(
    rowid = 1:3,
    age = c(20, 30, 40)
  )

  plain_survey <- survey(
    plain_df,
    id = "plain"
  )

  result <- metadata_survey_create(
    plain_survey
  )

  expect_true(
    is.data.frame(result)
  )

  expect_equal(
    nrow(result),
    2
  )
})


test_that("metadata_survey_create handles empty surveys", {
  empty_survey <- survey(
    data.frame(),
    id = "empty"
  )

  result <- metadata_survey_create(
    empty_survey
  )

  expect_true(
    is.data.frame(result)
  )

  expect_equal(
    nrow(result),
    1
  )
})


test_that("metadata_survey_create handles missing filename", {
  test_survey <- read_rds(
    system.file(
      "examples",
      "ZA7576.rds",
      package = "retroharmonize"
    )
  )

  attr(test_survey, "filename") <- NULL

  result <- metadata_survey_create(
    test_survey
  )

  expect_equal(
    unique(result$filename),
    "unknown"
  )
})

#' @keywords  internal
validate_survey_list <- function(survey_list) {
  assert_that(!is.null(survey_list),
    msg = "The parameter 'survey_list' is NULL."
  )

  assert_that("list" %in% class(survey_list) | is.survey(survey_list),
    msg = "The parameter 'survey_list' is not a list or a single survey."
  )

  if ("list" %in% class(survey_list)) {
    are_these_surveys <- vapply(survey_list, is.survey, logical(1))
    not_surveys <- paste(which(are_these_surveys == FALSE), collapse = ", ")
    assert_that(all(are_these_surveys),
      msg = glue::glue("{not_surveys} in 'survey_list' are not surveys.")
    )

    n_survey <- length(survey_list)

    filenames <- vapply(
      survey_list,
      function(x) {
        attr(x, "filename")
      },
      character(1)
    )

    ids <- vapply(survey_list, function(x) attr(x, "id"), character(1))

    duplicate_ids <- ids[duplicated(ids)]

    missing_ids <- vapply(
      survey_list, function(x) {
        is.null(attr(x, "id"))
      },
      logical(1)
    )

    missing_filenames <- vapply(
      survey_list,
      function(x) is.null(attr(x, "filename")),
      logical(1)
    )

    assert_that(!all(missing_ids),
      msg = paste0(paste(which(missing_ids), " have no IDs"))
    )

    assert_that(!all(missing_filenames),
      msg = paste0(paste(which(missing_ids), " have no filenames"))
    )

    ids <- tryCatch(
      {
        vapply(survey_list, function(x) attr(x, "id"), character(1))
      },
      error = function(cond) {
        message("Some IDs are not character(1L) single characters.")
      },
      finally = {}
    )

    filenames <- tryCatch(
      {
        vapply(survey_list, function(x) attr(x, "filename"), character(1))
      },
      error = function(cond) {
        message("Some filenames are not character(1L) single characters.")
      },
      finally = {}
    )

    duplicate_ids <- ids[duplicated(ids)]
    duplicate_filenames <- filenames[duplicated(filenames)]

    assert_that(length(duplicate_ids) == 0,
      msg = paste0(
        paste(duplicate_ids, collapse = ", "),
        " are not unique."
      )
    )

    assert_that(length(duplicate_filenames) == 0,
      msg = paste0(
        paste(duplicate_filenames, collapse = ", "),
        " are not unique."
      )
    )
  }
}

#' @importFrom fs file_exists
#' @keywords internal
validate_survey_files <- function(survey_files) {
  existing_survey_files <- vapply(
    survey_files,
    fs::file_exists,
    logical(1)
  )
  not_existing_survey_files <- existing_survey_files[
    existing_survey_files == FALSE
  ]

  if (length(not_existing_survey_files) > 0) {
    error_msg <- paste0(
      "the following files were not found :",
      paste(names(not_existing_survey_files),
        collapse = ", "
      )
    )
    stop("Error in validate_survey_files() -  ", error_msg)
  }

  TRUE
}

#' @title Validate harmonize_labels parameter
#' Check if "from", "to", and "numeric_values" are of equal lengths.
#' @importFrom dplyr select
#' @importFrom assertthat assert_that
#' @keywords  internal
validate_harmonize_labels <- function(harmonize_labels) {
  if (inherits(harmonize_labels, "list") | inherits(harmonize_labels, "data.frame")) {
    assertthat::assert_that(
      all(c("from", "numeric_values", "to") %in% names(harmonize_labels)),
      msg = "The harmonize_values must contain <from>, <to> and <numeric_values> vectors."
    )

    assertthat::assert_that(
      inherits(harmonize_labels$numeric_values, "numeric"),
      msg = "The harmonize_values must a numeric <numeric_values> vector."
    )

    assertthat::assert_that(
      is.numeric(harmonize_labels$numeric_values) |
        is.null(harmonize_labels$numeric_values),
      msg = "The harmonize_values must have a numeric <numeric_values> with non-NULL or non-NA values."
    )

    assertthat::assert_that(
      inherits(harmonize_labels$from, "character"),
      msg = "The harmonize_values must a character <from> vector."
    )

    assertthat::assert_that(
      inherits(harmonize_labels$to, "character"),
      msg = "The harmonize_values must a character <to> vector."
    )

    list_length <- as.numeric(vapply(c("from", "numeric_values", "to"), function(x) length(harmonize_labels[[x]]), numeric(1)))

    assertthat::assert_that(
      all(vapply(
        list_length,
        function(x) list_length[[1]] == x,
        logical(1)
      )),
      msg = "<harmonize_label> must have <from>, <to>, <numeric_values> of equal lengths."
    )
  } else {
    stop("<harmonize label> must have <from>, <to>, <numeric_values> of equal lengths as list or data.frame.")
  }
}

#' Coercion methods for labelled survey vectors
#'
#' Convert labelled SPSS-style survey vectors to common R data types.
#' These helpers provide consistent coercion behavior for
#' `"retroharmonize_labelled_spss_survey"` objects while respecting
#' labelled missing values.
#'
#' @param x A labelled survey vector created with
#'   [labelled_spss_survey()].
#'
#' @return
#' * `as_numeric()` returns a numeric vector with labelled missing
#'   values converted to `NA`.
#' * `as_character()` returns a character vector based on the
#'   factor representation of `x`.
#' * `as_factor()` returns a factor with levels derived from value
#'   labels.
#'
#' @seealso
#' [labelled_spss_survey()],
#' [haven::as_factor()]
#'
#' @family type conversion functions
#'
#' @name labelled_spss_survey_coercion
NULL


#' @rdname labelled_spss_survey_coercion
#' @export
as_numeric <- function(x) {
  vec_convert_na(x)
}


#' @rdname labelled_spss_survey_coercion
#' @export
as_character <- function(x) {
  as.character(as_factor(x))
}


#' @rdname labelled_spss_survey_coercion
#' @param levels Character string indicating how factor levels
#'   should be constructed. Currently retained for compatibility.
#' @param ordered Logical; whether the resulting factor should be ordered.
#'   Currently ignored.
#' @export
as_factor <- function(x, levels = "default", ordered = FALSE) {
  tmp <- haven::as_factor(x)
  attr(tmp, "levels") <- levels(tmp)
  class(tmp) <- "factor"
  tmp
}

## Scenarios with different labels ----------------------------
test_that("subsetting workds", {
  v1 <- labelled_spss_survey(
    x = c(1, 0, 1, 9),
    labels = c(
      "yes" = 1,
      "no" = 0,
      "inap" = 9
    ),
    na_values = 9,
    label = "My survey item",
    id = "test"
  )
  expect_equal(as_numeric(v1[3:4]), c(1, NA))
})


test_that("type conversion is correct", {
  my_x <- 1:10
  x1 <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )
  x2 <- labelled_spss_survey(my_x,
    labels = c(Good = 1, Bad = 8),
    na_range = c(9, Inf),
    label = "Quality Rating",
    id = "survey2"
  )
  expect_equal(as_numeric(x1), c(1:8, NA, NA))
  expect_equal(attr(x2, "survey2_labels"), c(Good = 1, Bad = 8))
  expect_equal(attr(x1, "survey1_na_values"), c(9, 10))
})

test_that("NA values are correct", {
  x1 <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )
  expect_equal(sum(is.na(x1)), 2)
  expect_equal(levels(as_factor(x1)), c("Good", 2:7, "Bad", 9:10))
  expect_equal(retroharmonize::as_character(x = x1), c("Good", 2:7, "Bad", 9:10))
})

test_that("errors work", {
  x1 <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )

  expect_error(sum(as_factor(x1)))
})

test_that("attributes are present", {
  my_x <- 1:10
  x1 <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )
  x2 <- labelled_spss_survey(my_x,
    labels = c(Good = 1, Bad = 8),
    na_range = c(9, Inf),
    label = "Quality Rating",
    id = "survey2"
  )
  expect_equal(attr(x1, "id"), "survey1")
  expect_equal(attr(x2, "label"), "Quality Rating")
  expect_equal(attr(x2, "survey2_name"), "my_x")
  expect_equal(attr(x2, "na_range"), c(9, Inf))
  expect_equal(attr(x2, "survey2_na_range"), c(9, Inf))
})

test_that("na_range_to_values harmonizes range-only missing values", {
  var1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      TRUST = 1,
      `NOT TRUST` = 0,
      `DON'T KNOW` = 8,
      `INAP. HERE` = 9
    ),
    na_range = c(8, 12)
  )

  res <- na_range_to_values(var1)

  expect_equal(labelled::na_values(res), c(8, 9))
  expect_equal(as_numeric(res), c(1, 0, 1, 1, 0, NA, NA))
})


test_that("explicit na_values outside range trigger warning and adjustment", {
  var2 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 7, 9),
    labels = c(
      TRUST = 1,
      `NOT TRUST` = 0,
      `DON'T KNOW` = 7,
      `INAP. HERE` = 9
    ),
    na_range = c(8, 12),
    na_values = c(7, 9)
  )

  expect_warning(
    res <- na_range_to_values(var2),
    "Inconsistent missing ranges"
  )

  expect_equal(labelled::na_values(res), c(7, 9))
  expect_equal(labelled::na_range(res), c(7, 12))
})

test_that("non-SPSS labelled vectors are returned unchanged", {
  var3 <- labelled::labelled(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(TRUST = 1, `NOT TRUST` = 0)
  )

  res <- na_range_to_values(var3)

  expect_identical(res, var3)
  expect_null(labelled::na_values(res))
})

test_that("harmonize_values casts labelled_spss correctly", {
  var_1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  h1 <- harmonize_values(
    x = var_1,
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "declined"    = 99998,
      "inap"        = 99999
    ),
    id = "survey_id",
    harmonize_label = "Do you trust the European Union?"
  )

  expect_s3_class(h1, "haven_labelled_spss")
  expect_true(is.numeric(h1))
  expect_true(is.double(h1))
})


test_that("harmonize_values stores metadata attributes", {
  var_1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  h1 <- harmonize_values(
    x = var_1,
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "declined"    = 99998,
      "inap"        = 99999
    ),
    id = "survey_id",
    harmonize_label = "Do you trust the European Union?"
  )

  expect_equal(attr(h1, "label"), "Do you trust the European Union?")
  expect_equal(attr(h1, "id"), "survey_id")

  expect_equal(
    attr(h1, "survey_id_labels"),
    c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    )
  )

  expect_equal(
    attr(h1, "survey_id_values"),
    c("0" = 0, "1" = 1, "8" = 99997, "9" = 99999)
  )
})


test_that("harmonize_values recodes values correctly", {
  var_1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  h1 <- harmonize_values(
    x = var_1,
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "declined"    = 99998,
      "inap"        = 99999
    )
  )

  expect_equal(
    as.vector(h1),
    c(1, 0, 1, 1, 0, 99997, 99999)
  )
})


test_that("harmonize_values recasting helpers work", {
  var_1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  h1 <- harmonize_values(
    x = var_1,
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "declined"    = 99998,
      "inap"        = 99999
    )
  )

  expect_equal(as_numeric(h1), c(1, 0, 1, 1, 0, NA, NA))
  expect_equal(
    levels(as_factor(h1)),
    c("not_trust", "trust", "do_not_know", "declined", "inap")
  )
  expect_equal(
    as_character(h1),
    c(
      "trust", "not_trust", "trust", "trust",
      "not_trust", "do_not_know", "inap"
    )
  )
})


test_that("harmonize_values handles invalid input gracefully", {
  var_1 <- labelled::labelled_spss(
    x = c(1, 0, 1),
    labels = c("TRUST" = 1, "NOT TRUST" = 0),
    na_values = 8
  )

  expect_error(
    harmonize_values(
      var_1,
      harmonize_labels = list(wrong_from = c("a"), wrong_to = c("b"))
    )
  )

  expect_error(
    harmonize_values(var_1, harmonize_labels = c("a", "b"))
  )
})


test_that("harmonize_values works with overlapping labels", {
  mc_var <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "Mentioned" = 1,
      "Not mentioned" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )

  h_mc <- harmonize_values(
    x = mc_var,
    harmonize_labels = list(
      from = c("^mentioned", "^not mentioned", "^dk|^don", "^inap"),
      to = c("mentioned", "not_mentioned", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    )
  )

  expect_setequal(
    unique(as_character(h_mc)),
    c("mentioned", "not_mentioned", "do_not_know", "inap")
  )
})

test_that("correct values are returned", {
  var1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )
  var2 <- labelled::labelled_spss(
    x = c(2, 2, 8, 9, 1, 1),
    labels = c(
      "Tend to trust" = 1,
      "Tend not to trust" = 2,
      "DK" = 8,
      "Inap" = 9
    ),
    na_values = c(8, 9)
  )
  h1 <- harmonize_values(
    x = var1,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "inap" = 99999
    ),
    id = "survey1",
  )
  h2 <- harmonize_values(
    x = var2,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "inap" = 99999
    ),
    id = "survey2"
  )
  h3 <- concatenate(x = h1, y = h2)
  expect_true(inherits(h3, "retroharmonize_labelled_spss_survey"))
  expect_equal(as_numeric(h3), c(
    1, 0, 1, 1, 0, NA, NA,
    0, 0, NA, NA, 1, 1
  ))
  expect_equal(as_character(h3), c(
    "trust", "not_trust", "trust", "trust", "not_trust", "do_not_know", "inap",
    "not_trust", "not_trust", "do_not_know", "inap", "trust", "trust"
  ))
  expect_equal(levels(as_factor(h3)), c("not_trust", "trust", "do_not_know", "inap"))
})

test_that("correct values are returned", {
  var1 <- labelled::labelled_spss(
    x = c(1, 0, 1, 1, 0, 8, 9),
    labels = c(
      "TRUST" = 1,
      "NOT TRUST" = 0,
      "DON'T KNOW" = 8,
      "INAP. HERE" = 9
    ),
    na_values = c(8, 9)
  )
  var2 <- labelled::labelled_spss(
    x = c(2, 2, 8, 9, 1, 1),
    labels = c(
      "Tend to trust" = 1,
      "Tend not to trust" = 2,
      "DK" = 8,
      "Inap" = 9
    ),
    na_values = c(8, 9)
  )
  h1 <- harmonize_values(
    x = var1,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "inap" = 99999
    ),
    id = "survey1",
  )
  h2 <- harmonize_values(
    x = var2,
    harmonize_label = "Do you trust the European Union?",
    harmonize_labels = list(
      from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
      to = c("trust", "not_trust", "do_not_know", "inap"),
      numeric_values = c(1, 0, 99997, 99999)
    ),
    na_values = c(
      "do_not_know" = 99997,
      "inap" = 99999
    ),
    id = "survey2"
  )
  h3 <- concatenate(x = h1, y = h2)
  a <- tibble::tibble(
    rowid = paste0("survey1", 1:length(h1)),
    hvar = h1,
    w = runif(n = length(h1), 0, 1)
  )
  b <- tibble::tibble(
    rowid = paste0("survey2", 1:length(h2)),
    hvar = h2,
    w = runif(n = length(h2), 0, 1)
  )
  c <- dplyr::bind_rows(a, b)
  d <- vctrs::vec_rbind(a, b)
  x <- a$hvar
  y <- b$hvar
  expect_equal(attr(d$hvar, "survey2_labels"), c(
    "Tend to trust" = 1,
    "Tend not to trust" = 2,
    "DK" = 8,
    "Inap" = 9
  ))
  expect_equal(attr(c$hvar, "survey2_labels"), c(
    "Tend to trust" = 1,
    "Tend not to trust" = 2,
    "DK" = 8,
    "Inap" = 9
  ))
  expect_equal(attr(c$hvar, "survey1_labels"), c(
    "TRUST" = 1,
    "NOT TRUST" = 0,
    "DON'T KNOW" = 8,
    "INAP. HERE" = 9
  ))
})

#' Read a survey dataset from a CSV file
#'
#' Import a survey dataset stored in comma-separated value (`.csv`)
#' format and convert it into a survey-compatible tibble with
#' reproducibility metadata retained as attributes.
#'
#' The imported object is returned as a tibble with additional survey
#' metadata such as identifiers, DOI references, and optional dataset
#' bibliographic metadata.
#'
#' @param file Path to a `.csv` file.
#'
#' @param dataset_bibentry Optional bibliographic metadata created
#'   with [dataset::dublincore()] or [dataset::datacite()].
#'
#' @param id Optional dataset identifier. When omitted, the file name
#'   without extension is used.
#'
#' @param doi Optional dataset DOI identifier.
#'
#' @param ... Additional arguments passed to
#'   [utils::read.csv()].
#'
#' @return A tibble-like survey object with metadata attributes
#'   retained for reproducible workflows.
#'
#' @examples
#' # Create a temporary CSV file:
#' path <- system.file(
#'   "examples",
#'   "ZA7576.rds",
#'   package = "retroharmonize"
#' )
#'
#' read_survey <- read_rds(path)
#'
#' test_csv_file <- tempfile(fileext = ".csv")
#'
#' write.csv(
#'   x = read_survey,
#'   file = test_csv_file,
#'   row.names = FALSE
#' )
#'
#' # Read the CSV file:
#' re_read <- read_csv(
#'   file = test_csv_file,
#'   id = "ZA7576",
#'   doi = "test_doi"
#' )
#'
#' @importFrom dataset datacite dataset_df defined dublincore is.dataset_df
#' @importFrom dplyr any_of select
#' @importFrom fs is_file path_ext_remove path_file
#' @importFrom labelled to_labelled var_label `var_label<-`
#' @importFrom purrr safely
#' @importFrom tibble as_tibble rowid_to_column
#' @importFrom utils object.size read.csv
#'
#' @family import functions
#' @export

read_csv <- function(file,
                     id = NULL,
                     doi = NULL,
                     dataset_bibentry = NULL,
                     ...) {
  # filename <- fs::path_file(file)

  filename <- file
  source_file_info <- valid_file_info(file)

  if (is.null(id)) {
    id <- fs::path_ext_remove(filename)
  }

  safely_readcsv <- purrr::safely(read.csv)

  tmp <- safely_readcsv(file = file)

  if (!is.null(tmp$error)) {
    warning(tmp$error, "\nReturning an empty survey.")
    return(
      survey(data.frame(),
        id = "Could not read file",
        filename = filename,
        doi = doi
      )
    )
  } else {
    tmp <- tmp$result
  }

  tmp <- tmp %>% dplyr::select(-any_of("X"))

  chr_vars <- vapply(
    1:ncol(tmp),
    function(x) inherits(tmp[, x], "character"),
    logical(1)
  )

  chr_unique_n <- vapply(
    which(chr_vars),
    function(x) length(unique(tmp[x, ])),
    integer(1)
  )

  to_fct_vars <- which(chr_vars)[which(chr_unique_n > 1)]


  for (i in to_fct_vars) {
    tmp[, i] <- dataset::defined(as.factor(tmp[, i]))
  }

  tmp_df <- dataset_df(tmp,
    identifier = doi,
    dataset_bibentry = dataset_bibentry
  )


  if (is.null(doi)) {
    if ("doi" %in% names(tmp)) {
      doi <- tmp$doi[1]
    } else {
      doi <- ""
    }
  }

  if (!"rowid" %in% names(tmp)) {
    stop("CSV file does not contain a 'rowid' column.")
  }

  rowid_chr <- as.character(tmp$rowid)

  tmp_df$rowid <- paste0(
    id, "_",
    gsub(id, "", rowid_chr, fixed = TRUE)
  )

  var_label(tmp_df$rowid) <- paste0("Unique identifier in ", id)

  return_survey <- survey_df(
    x = tmp,
    dataset_bibentry = dataset_bibentry,
    identifier = id,
    filename = filename
  )

  object_size <- as.numeric(object.size(as_tibble(tmp)))
  attr(return_survey, "object_size") <- object_size
  attr(return_survey, "source_file_size") <- source_file_info$size

  if (dataset::dataset_title(return_survey) == "Untitled Dataset") {
    dataset::dataset_title(return_survey,
      overwrite = TRUE
    ) <- "Untitled Survey"
  }

  ## For backward compatibility
  attr(return_survey, "id") <- id
  attr(return_survey, "doi") <- doi


  return_survey
}

test_that("attributes work", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  test_read <- read_rds(file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )
  expect_equal(attr(test_read, "id"), "ZA7576")
  expect_equal(attr(test_read, "filename"), "ZA7576.rds")
  expect_equal(attr(test_read, "doi"), "test_doi")
})

read_write_test <- function() {
  examples_dir <- system.file("examples", package = "retroharmonize")
  test_read <- read_rds(file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )
  temp_save_location <- tempdir()
  write.csv(test_read, file.path(temp_save_location, "test.csv"), row.names = F)
  reread <- read_csv(file = file.path(temp_save_location, "test.csv"))
  is.survey()
}

test_that("file size attributes are recorded", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds")
  )

  expect_true(!is.null(attr(test_read, "object_size")))
  expect_true(!is.null(attr(test_read, "source_file_size")))

  expect_true(attr(test_read, "object_size") > 0)
  expect_true(attr(test_read, "source_file_size") > 0)
})


test_that("id defaults to filename without extension", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds")
  )

  expect_equal(
    attr(test_read, "id"),
    "ZA7576"
  )
})


test_that("failed reads return empty survey", {
  bad_file <- tempfile(fileext = ".rds")

  writeLines("not an rds file", bad_file)

  expect_warning(
    result <- read_rds(bad_file)
  )

  expect_true(is.survey(result))
  expect_equal(ncol(result), 0)
})


test_that("rowid is normalized with survey id", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds"),
    id = "TESTID"
  )

  expect_true(
    all(grepl("^TESTID_", test_read$rowid))
  )
})

test_that("rowid label is created", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576"
  )

  expect_equal(
    labelled::var_label(test_read$rowid),
    "Unique identifier in ZA7576"
  )
})

#' @importFrom assertthat assert_that
#' @importFrom fs is_file
#' @keywords  internal
valid_file_info <- function(file) {
  assertthat::assert_that(
    fs::is_file(file),
    msg = paste0("file='", file, "' is not a file. ")
  )

  file.info(file)
}

#' Read a Stata `.dta` survey file
#'
#' Import a survey dataset stored in Stata `.dta` format and convert it
#' into a `survey` object with harmonized metadata and labelled variables.
#'
#' This function wraps [haven::read_dta()] and adds:
#'
#' - error handling,
#' - survey metadata creation,
#' - `rowid` normalization,
#' - preservation of variable labels,
#' - conversion of labelled variables,
#' - and provenance metadata.
#'
#' @param file Path to a Stata `.dta` file.
#' @param id Optional survey identifier. Defaults to the file name
#'   without extension.
#' @param doi Optional DOI identifier for the survey.
#' @param .name_repair Strategy for repairing invalid or duplicated
#'   column names. Passed to [haven::read_dta()].
#'
#' @return A `survey` object inheriting from `data.frame` and `tbl_df`.
#'
#' @details
#' Variable labels are preserved using the `"label"` attribute.
#'
#' Labelled variables are converted to harmonized labelled survey vectors
#' where possible. Variables that inherit from `haven_labelled` but do not
#' contain valid label definitions are converted back to standard vectors.
#'
#' If the file cannot be read, the function returns an empty `survey`
#' object and emits a warning.
#'
#' @family import functions
#'
#' @examples
#' \donttest{
#' path <- system.file(
#'   "examples",
#'   "iris.dta",
#'   package = "haven"
#' )
#'
#' survey_object <- read_dta(path)
#'
#' attr(survey_object, "id")
#' attr(survey_object, "filename")
#' }
#'
#' @importFrom assertthat assert_that
#' @importFrom dplyr bind_cols mutate_all select
#' @importFrom fs path_ext_remove path_file
#' @importFrom haven read_dta
#' @importFrom labelled var_label var_label<-
#' @importFrom purrr safely
#' @importFrom tibble as_tibble rowid_to_column
#' @importFrom tidyselect all_of everything
#' @importFrom utils object.size
#'
#' @export

read_dta <- function(file,
                     id = NULL,
                     doi = NULL,
                     .name_repair = "unique") {
  source_file_info <- valid_file_info(file)

  safely_read_haven_dta <- purrr::safely(.f = haven::read_dta)

  tmp <- safely_read_haven_dta(
    file = file,
    .name_repair = .name_repair
  )

  if (!is.null(tmp$error)) {
    warning(tmp$error, "\nReturning an empty survey.")
    return(
      survey(data.frame(), id = "Could not read file", filename = fs::path_file(file), doi = doi)
    )
  } else {
    tmp <- tmp$result
  }

  tmp <- tmp %>%
    tibble::rowid_to_column(var = "rowid")

  all_vars <- names(tmp)

  assertthat::assert_that(length(all_vars) > 0,
    msg = "The STATA file has no names."
  )

  filename <- fs::path_file(file)

  if (is.null(id)) {
    id <- fs::path_ext_remove(filename)
  }

  if (is.null(doi) && "doi" %in% names(tmp)) {
    doi <- tmp$doi[1]
  }

  tmp$rowid <- paste0(id, "_", tmp$rowid)


  label_orig <- lapply(tmp, labelled::var_label)

  labelled::var_label(
    tmp$rowid
  ) <- paste0("Unique identifier in ", id)

  converted <- tmp[!vapply(
    tmp,
    function(x) is.null(attr(x, "labels")),
    logical(1)
  )]

  converted <- converted[
    vapply(
      converted,
      function(x) length(attr(x, "labels")) > 0,
      logical(1)
    )
  ]

  converted <- converted %>%
    mutate_all(as_labelled_spss_survey, id)

  not_converted <- tmp %>%
    select(-all_of(names(converted)))

  convert_fake_labelled <- function(x) {
    # Fake labelled cases do not have labels, and they confuse the labelling functions
    # They should be immediately imported as non-labelled vectors

    if (!inherits(x, "haven_labelled")) {
      return(x)
    }

    if (inherits(x, "double") & length(attr(x, "labels")) == 0) {
      as_numeric(x)
    } else if (inherits(x, "character") & length(attr(x, "labels")) == 0) {
      as_character(x)
    } else {
      x
    }
  }

  not_converted <- not_converted %>%
    mutate(
      across(
        everything(),
        convert_fake_labelled
      )
    )

  if (ncol(converted) == 0) {
    return_df <- not_converted
  } else if (ncol(not_converted) == 0) {
    return_df <- converted
  } else {
    return_df <- bind_cols(not_converted, converted)
  }

  return_df <- return_df %>%
    select(all_of(all_vars))


  labelling_orig <- names(label_orig)
  labelling_orig[as.numeric(which(vapply(label_orig, is.null, logical(1))))] <- ""

  original_labels <- lapply(
    label_orig,
    function(x) {
      if (is.null(x)) "" else x
    }
  )

  for (i in seq_along(return_df)) {
    # Only labelled classes will have a label
    labelled::var_label(return_df[[i]]) <- original_labels[[i]]
  }

  return_survey <- survey(return_df,
    id = id,
    filename = filename,
    doi = doi
  )

  object_size <- as.numeric(object.size(as_tibble(return_df)))
  attr(return_survey, "object_size") <- object_size
  attr(return_survey, "source_file_size") <- source_file_info$size

  return_survey
}

test_that("invalid dta file returns empty survey", {
  bad_file <- tempfile(fileext = ".dta")

  writeLines("not a dta file", bad_file)

  expect_warning(
    result <- read_dta(bad_file)
  )

  expect_true(is.survey(result))
  expect_equal(ncol(result), 0)
})

test_that("read_dta imports a stata file", {
  tmp_file <- tempfile(fileext = ".dta")

  test_df <- tibble::tibble(
    id = 1:3,
    country = c("HU", "DE", "FR")
  )

  haven::write_dta(
    test_df,
    tmp_file
  )

  result <- read_dta(tmp_file)

  expect_true(is.survey(result))
  expect_equal(nrow(result), 3)
})

test_that("rowid is created and normalized", {
  tmp_file <- tempfile(fileext = ".dta")

  test_df <- tibble::tibble(
    x = 1:3
  )

  haven::write_dta(
    test_df,
    tmp_file
  )

  result <- read_dta(
    tmp_file,
    id = "TEST"
  )

  expect_true("rowid" %in% names(result))

  expect_true(
    all(grepl("^TEST_", result$rowid))
  )
})


test_that("variable labels are preserved", {
  tmp_file <- tempfile(fileext = ".dta")

  test_df <- tibble::tibble(
    age = c(20, 30, 40)
  )

  labelled::var_label(test_df$age) <- "Age of respondent"

  haven::write_dta(
    test_df,
    tmp_file
  )

  result <- read_dta(tmp_file)

  expect_equal(
    labelled::var_label(result$age),
    "Age of respondent"
  )
})


test_that("provenance attributes are recorded", {
  tmp_file <- tempfile(fileext = ".dta")

  haven::write_dta(
    tibble::tibble(x = 1:3),
    tmp_file
  )

  result <- read_dta(tmp_file)

  expect_true(!is.null(attr(result, "object_size")))
  expect_true(!is.null(attr(result, "source_file_size")))
})

#' Read a survey from a CSV file
#'
#' Import a survey stored in a CSV file and return it as a survey object
#' with attached dataset- and survey-level metadata.
#'
#' The CSV file is read using [utils::read.csv()]. Character variables
#' with more than one unique value are automatically converted to
#' labelled factors. A unique row identifier is added and labelled.
#' @param survey_list A list containing surveys of class survey.
#' @param subset_name Character string appended to filenames of
#'   subsetted surveys. Defaults to `"subset"`.
#' @param crosswalk_table A crosswalk table created with
#'   [crosswalk_table_create()].
#' @param survey_paths Optional character vector of file paths to surveys.
#'
#' @param import_path Optional base directory used to resolve `survey_paths`.
#'
#' @param export_path Optional directory where subsetted surveys are
#' exported to
#'
#' @details
#' If the file cannot be read, an empty survey object is returned with
#' a warning.
#'
#' If a column named `"X"` is present (commonly created by
#' `write.csv()`), it is removed automatically.
#'
#' @return
#' An object of class `"survey"`, which is a data frame with attached
#' survey- and dataset-level metadata.
#'
#' @seealso
#' [read_rds()] for importing surveys from RDS files,
#' [survey_df()] for constructing survey objects manually.
#'
#' @family import functions
#'
#' @importFrom dplyr left_join filter select distinct_all
#' @importFrom tibble tibble
#' @importFrom rlang set_names
#'
#' @examples
#' # Create a temporary CSV file from an example survey
#' path <- system.file("examples", "ZA7576.rds",
#'   package = "retroharmonize"
#' )
#' survey <- read_rds(path)
#'
#' tmp <- tempfile(fileext = ".csv")
#' write.csv(survey, tmp, row.names = FALSE)
#'
#' # Read the CSV file back as a survey
#' re_read <- read_csv(
#'   file = tmp,
#'   id = "ZA7576",
#'   doi = "10.0000/example"
#' )
#'
#' @export


harmonize_survey_variables <- function(crosswalk_table,
                                       subset_name = "subset",
                                       survey_list = NULL,
                                       survey_paths = NULL,
                                       import_path = NULL,
                                       export_path = NULL) {
  # This is a wrapper for subset_save_survey with strict validation
  # of new variable names.

  # validates structure and unambiguous naming
  is.crosswalk_table(crosswalk_table)

  subsetted_surveys <- subset_surveys(
    crosswalk_table = crosswalk_table,
    subset_name = subset_name,
    survey_list = survey_list,
    import_path = import_path,
    export_path = export_path
  )

  rename_survey_to_memory <- function(x) {
    if (is.survey(x)) {
      this_survey <- x
    } else if (is.character(x)) {
      this_survey <- read_rds(file.path(export_path, x))
    } else {
      stop(" rename_survey_file(x) x must be a filename or a survey.")
    }

    survey_id <- gsub(paste0("_", subset_name), "", attr(this_survey, "id"))

    new_names <- tibble(var_name_orig = names(this_survey)) %>%
      left_join(
        crosswalk_table %>%
          filter(id == survey_id) %>%
          select(var_name_orig, var_name_target) %>%
          distinct_all(),
        by = "var_name_orig",
      ) %>%
      mutate(var_name_target = ifelse(var_name_orig == "rowid",
        yes = "rowid",
        no = var_name_target
      )) %>%
      select(var_name_target) %>%
      unlist() %>%
      as.character()

    rlang::set_names(this_survey, nm = new_names)
  }

  rename_survey_to_file <- function(x) {
    if (is.survey(x)) {
      this_survey <- x
    } else if (is.character(x)) {
      # this_survey <- read_rds(file.path(export_path, x))
      this_survey <- readRDS(file.path(export_path, x))
    } else {
      stop(" rename_survey_file(x) x must be a filename or a survey.")
    }

    survey_id <- gsub(paste0("_", subset_name), "", attr(this_survey, "id"))


    new_names <- tibble(var_name_orig = names(this_survey)) %>%
      left_join(
        crosswalk_table %>%
          filter(id == survey_id) %>%
          select(var_name_orig, var_name_target) %>%
          distinct_all(),
        by = "var_name_orig",
      ) %>%
      mutate(var_name_target = ifelse(var_name_orig == "rowid",
        yes = "rowid",
        no = var_name_target
      )) %>%
      select(var_name_target) %>%
      unlist() %>%
      as.character()

    this_survey <- rlang::set_names(this_survey, nm = new_names)
    saveRDS(this_survey, file = file.path(export_path, x), version = 2)
    x
  }

  if (is.null(export_path)) {
    # Results to memory  ---------------------------------
    lapply(subsetted_surveys, rename_survey_to_memory)
  } else {
    # Results to file  -----------------------------------

    vapply(subsetted_surveys, rename_survey_to_file, character(1))
  }
}

## Internal functions not to be exported ----------------------
#' @keywords  internal
cat_line <- function(...) {
  # identical to haven:::cat_line
  cat(paste0(..., "\n", collapse = ""))
}

#' @keywords  internal
is.labelled_spss <- function(x) inherits(x, "haven_labelled_spss")

#' @keywords  internal
is.labelled <- function(x) inherits(x, "haven_labelled")

#' Convert to haven_labelled_spss
#' @param x A vector
#' @param na_labels A named vector of missing values, defaults to
#' \code{c( "inap" = "inap")} for character vectors and
#' \code{c( 99999 = "inap")} for numeric vectors.
#' @return A haven_labelled_spss vector
#' @importFrom labelled val_labels
#' @keywords internal
convert_to_labelled_spss <- function(x, na_labels = NULL) {
  if (is.null(na_labels) && is.numeric(unclass(x))) {
    na_labels <- structure(99999, names = "inap")
  } else if (is.null(na_labels) && is.character(unclass(x))) {
    na_labels <- structure("inap", names = "inap")
  }

  stopifnot(!any(unclass(na_labels) %in% x))


  labelled_spss(x, c(labelled::val_labels(x), na_labels),
    na_values = unclass(na_labels)
  )
}


#' @keywords  internal
remove_na_range <- function(x) {
  assert_that(is.labelled_spss_survey(x))
  if (!is.null(attr(x, "na_range"))) {
    min_na_range <- min(attr(x, "na_range"))
    max_na_range <- max(attr(x, "na_range"))

    if (all(
      !max_na_range %in% range(x),
      !min_na_range %in% range(x)
    )) {
      attr(x, "na_range") <- NULL
    }
  }
  x
}

test_that("is.labelled_spss identifies haven_labelled_spss vectors", {
  x <- labelled::labelled_spss(
    x = c(1, 2, 9),
    labels = c(
      yes = 1,
      no = 2,
      inap = 9
    ),
    na_values = 9
  )

  expect_true(is.labelled_spss(x))
  expect_false(is.labelled_spss(1:3))
})

test_that("is.labelled identifies haven_labelled vectors", {
  x <- labelled::labelled(
    x = c(1, 2),
    labels = c(
      yes = 1,
      no = 2
    )
  )

  expect_true(is.labelled(x))
  expect_false(is.labelled("a"))
})

test_that("convert_to_labelled_spss adds default numeric missing labels", {
  x <- c(1, 2, 3)

  out <- convert_to_labelled_spss(x)

  expect_true(is.labelled_spss(out))

  expect_equal(
    labelled::na_values(out),
    c(inap = 99999)
  )

  expect_equal(
    labelled::val_labels(out),
    c(inap = 99999)
  )
})

test_that("convert_to_labelled_spss adds default character missing labels", {
  x <- c("a", "b")

  out <- convert_to_labelled_spss(x)

  expect_true(is.labelled_spss(out))

  expect_equal(
    labelled::na_values(out),
    c(inap = "inap")
  )

  expect_equal(
    labelled::val_labels(out),
    c(inap = "inap")
  )
})

test_that("convert_to_labelled_spss preserves existing labels", {
  x <- labelled::labelled(
    x = c(1, 2),
    labels = c(
      yes = 1,
      no = 2
    )
  )

  out <- convert_to_labelled_spss(x)

  expect_equal(
    labelled::val_labels(out),
    c(
      yes = 1,
      no = 2,
      inap = 99999
    )
  )
})

test_that("convert_to_labelled_spss errors on conflicting missing values", {
  x <- c(1, 2, 99999)

  expect_error(
    convert_to_labelled_spss(x)
  )
})

test_that("remove_na_range removes unused na_range", {
  x <- labelled_spss_survey(
    x = c(1, 2, 3),
    labels = c(
      yes = 1,
      no = 2
    ),
    na_range = c(8, 9),
    id = "survey1"
  )

  out <- remove_na_range(x)

  expect_null(attr(out, "na_range"))
})

test_that("remove_na_range preserves active na_range", {
  x <- labelled_spss_survey(
    x = c(1, 2, 8),
    labels = c(
      yes = 1,
      no = 2,
      missing = 8
    ),
    na_range = c(8, 9),
    id = "survey1"
  )

  out <- remove_na_range(x)

  expect_equal(
    attr(out, "na_range"),
    c(8, 9)
  )
})

test_that("concatenate() preserves labelled class", {
  v1 <- haven::labelled_spss(
    x = c(1, 2),
    labels = c(YES = 1, NO = 2),
    na_values = 9,
    label = "Example"
  )

  v2 <- haven::labelled_spss(
    x = c(2, 1),
    labels = c(YES = 1, NO = 2),
    na_values = 9,
    label = "Example"
  )

  result <- concatenate(v1, v2)

  expect_true(
    haven::is.labelled(result)
  )

  expect_true(
    inherits(result, "haven_labelled_spss")
  )
})


test_that("concatenate() preserves labels", {
  v1 <- haven::labelled_spss(
    x = c(1, 2),
    labels = c(YES = 1, NO = 2),
    na_values = 9,
    label = "Example"
  )

  v2 <- haven::labelled_spss(
    x = c(2, 1),
    labels = c(YES = 1, NO = 2),
    na_values = 9,
    label = "Example"
  )

  result <- concatenate(v1, v2)

  expect_equal(
    labelled::val_labels(result),
    c(YES = 1, NO = 2)
  )

  expect_equal(
    labelled::var_label(result),
    "Example"
  )
})


test_that("concatenate() concatenates values", {
  v1 <- haven::labelled_spss(
    x = c(1, 2),
    labels = c(YES = 1, NO = 2),
    na_values = 9
  )

  v2 <- haven::labelled_spss(
    x = c(2, 1),
    labels = c(YES = 1, NO = 2),
    na_values = 9
  )

  result <- concatenate(v1, v2)

  expect_equal(
    as.vector(result),
    c(1, 2, 2, 1)
  )
})


test_that("concatenate() errors on incompatible labels", {
  v1 <- haven::labelled_spss(
    x = c(1, 2),
    labels = c(YES = 1, NO = 2),
    na_values = 9
  )

  v2 <- haven::labelled_spss(
    x = c(1, 2),
    labels = c("TRUE" = 1, "FALSE" = 2),
    na_values = 9
  )

  expect_warning(
    concatenate(v1, v2)
  )
})

#' Create a survey codebook
#'
#' Expand survey metadata into a long-format codebook of value labels.
#'
#' `create_codebook()` takes survey-level metadata and returns a
#' tidy data frame describing all labelled variables and their
#' associated value labels. Each row corresponds to a single
#' value label, classified as either a valid value or a missing value.
#'
#' Unlabelled numeric and character variables are excluded.
#'
#' @param metadata A metadata table created by [metadata_create()].
#'   If supplied, `survey` must be `NULL`.
#'
#' @param survey A survey object of class `"survey"`.
#'   If supplied, metadata is generated internally using
#'   [metadata_create()].
#'
#' @return
#' A data frame with one row per value label, including:
#' \itemize{
#'   \item survey identifiers (`id`, `filename`)
#'   \item original variable names and labels
#'   \item value codes and value labels
#'   \item label type (`"valid"` or `"missing"`)
#'   \item summary counts of labels
#' }
#'
#' Additional user-defined metadata columns present in the input
#' metadata are preserved.
#'
#' @details
#' For multiple survey waves, use [codebook_surveys_create()].
#'
#' If both `metadata` and `survey` are provided, `survey` takes precedence.
#'
#' @seealso
#' [metadata_create()],
#' [codebook_surveys_create()]
#'
#' @importFrom tidyr unnest_longer
#' @importFrom dplyr mutate filter arrange left_join bind_rows select
#' @importFrom dplyr any_of
#' @importFrom assertthat assert_that
#' @importFrom rlang set_names
#'
#' @family metadata functions
#'
#' @examples
#' survey <- read_rds(
#'   system.file("examples", "ZA7576.rds", package = "retroharmonize")
#' )
#'
#' cb <- create_codebook(survey = survey)
#' head(cb)
#'
#' @export

create_codebook <- function(metadata = NULL,
                            survey = NULL) {
  assertthat::assert_that(
    !(is.null(metadata) & is.null(survey)),
    msg = "Either the parameter 'metadata' or the parameter 'survey' must be given. They are both NULL."
  )

  if (!is.null(metadata)) {
    if (inherits(metadata, "survey") & is.null(survey)) {
      survey <- metadata
      warning("You did not give a survey parameter, but the metadata parameter was of type survey. It was treated as survey.")
    }
  }

  if (!is.null(survey)) {
    assertthat::assert_that(inherits(survey, "survey"),
      msg = "Parameter survey must be of class survey."
    )
    metadata <- metadata_create(survey)
  }

  metadata_names <- c(
    "filename", "id", "var_name_orig", "class_orig", "var_label_orig",
    "labels", "valid_labels", "na_labels", "na_range",
    "n_labels", "n_valid_labels", "n_na_labels"
  )

  user_vars <- names(metadata)[!names(metadata) %in% metadata_names]

  assertthat::assert_that(
    all(metadata_names %in% names(metadata))
  )

  metadata$entry <- 1:nrow(metadata)

  if (length(user_vars) > 0) {
    user_data <- metadata %>% select(all_of(c("entry", user_vars)))
  }


  metadata$label_type <- vapply(
    metadata$labels,
    function(x) class(x)[1],
    character(1)
  )

  metadata$label_type <- ifelse(
    test = is.na(metadata$valid_labels) & is.na(metadata$na_labels),
    yes  = "not_labelled",
    no   = metadata$label_type
  )


  char_labels <- vapply(
    metadata$valid_labels,
    function(x) class(x)[1],
    character(1)
  ) == "character"

  num_labels <- vapply(
    metadata$valid_labels,
    function(x) class(x)[1],
    character(1)
  ) == "numeric"

  metadata_labelled_numeric <- metadata[num_labels, ]
  n_labelled_numeric <- ifelse(!is.null(metadata_labelled_numeric),
    nrow(metadata_labelled_numeric),
    0
  )

  if (n_labelled_numeric > 0) {
    # These area cases when the labels are of class numeric
    valid_labelled_numeric <- metadata_labelled_numeric %>%
      dplyr::filter(grepl("labelled", class_orig)) %>%
      dplyr::select(all_of(c(
        "entry", "id", "filename", "var_name_orig",
        "var_label_orig", "valid_labels"
      ))) %>%
      tidyr::unnest_longer(valid_labels) %>%
      dplyr::select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "valid_labels", "valid_labels_id"
      ))) %>%
      rlang::set_names(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig", "val_code_orig",
        "val_label_orig"
      )) %>%
      dplyr::mutate(
        label_range = "valid",
        val_code_orig = as.character(val_code_orig),
        val_label_orig = as.character(val_label_orig)
      )

    na_labelled_numeric <- metadata[num_labels, ] %>%
      dplyr::filter(grepl("labelled", class_orig)) %>%
      dplyr::select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "na_labels"
      ))) %>%
      tidyr::unnest_longer(na_labels) %>%
      dplyr::select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "na_labels", "na_labels_id"
      ))) %>%
      rlang::set_names(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "val_code_orig", "val_label_orig"
      )) %>%
      dplyr::mutate(
        # This is the missing observation range
        label_range = "missing"
      ) %>%
      filter(!is.na(val_code_orig)) %>%
      mutate(
        val_code_orig = as.character(val_code_orig),
        val_label_orig = as.character(val_label_orig)
      )


    num_labels <- valid_labelled_numeric %>%
      dplyr::bind_rows(
        na_labelled_numeric
      ) %>%
      dplyr::arrange(entry, val_code_orig) %>%
      left_join(
        metadata %>% select(any_of(c(
          "entry", "id", "filename", "na_range",
          "n_labels", "n_valid_labels", "n_na_labels",
          user_vars
        ))),
        by = c("entry", "id", "filename")
      )
  }

  metadata_labelled_character <- metadata[char_labels, ]
  n_labelled_character <- ifelse(
    !is.null(metadata_labelled_character),
    nrow(metadata_labelled_character),
    0
  )

  if (n_labelled_character > 0) {
    # These area cases when the na_labels are of class character
    valid_labelled_character <- metadata_labelled_character %>%
      filter(grepl("labelled", class_orig)) %>%
      select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig", "valid_labels"
      ))) %>%
      unnest_longer(valid_labels) %>%
      select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "valid_labels", "valid_labels_id"
      ))) %>%
      rlang::set_names(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "val_code_orig", "val_label_orig"
      )) %>%
      mutate(
        # This is the valid observation range
        label_range = "valid"
      ) %>%
      mutate(val_code_orig = as.character(val_code_orig))


    na_labelled_character <- metadata[char_labels, ] %>%
      filter(grepl("labelled", class_orig)) %>%
      select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig", "na_labels"
      ))) %>%
      unnest_longer(na_labels) %>%
      select(all_of(c(
        "entry", "id", "filename",
        "var_name_orig", "var_label_orig",
        "na_labels", "na_labels_id"
      ))) %>%
      rlang::set_names(c(
        "entry", "id", "filename", "var_name_orig",
        "var_label_orig", "val_code_orig", "val_label_orig"
      )) %>%
      mutate(
        # This is the missing observation range
        label_range = "missing"
      ) %>%
      filter(!is.na(val_code_orig)) %>%
      mutate(val_code_orig = as.character(val_code_orig))


    char_labels <- valid_labelled_character %>%
      dplyr::bind_rows(
        na_labelled_character
      ) %>%
      dplyr::arrange(entry, val_code_orig) %>%
      left_join(
        metadata %>% select(any_of(c(
          "entry", "id", "filename", "na_range",
          "n_labels", "n_valid_labels", "n_na_labels",
          user_vars
        ))),
        by = c("entry", "id", "filename")
      )
  }

  if (n_labelled_character + n_labelled_numeric == 0) {
    # There are no labelled variables
    tibble(
      entry = vector("integer"),
      id = vector("character"),
      filename = vector("character"),
      var_name_orig = vector("character"),
      var_label_orig = vector("character"),
      val_code_orig = vector("character"),
      val_label_orig = vector("character"),
      label_range = vector("character"),
      na_range = vector("character"),
      n_labels = vector("numeric"),
      n_valid_labels = vector("numeric"),
      n_na_labels = vector("numeric")
    ) %>%
      left_join(user_data[0, ], by = "entry")
  } else if (n_labelled_character == 0) {
    num_labels %>%
      dplyr::arrange(entry)
  } else if (n_labelled_numeric == 0) {
    char_labels %>%
      dplyr::arrange(entry)
  } else {
    num_labels %>%
      bind_rows(char_labels) %>%
      dplyr::arrange(entry)
  }
}

#' @rdname create_codebook
#' @param waves A list of surveys.
#' @export
codebook_waves_create <- function(waves) {
  .Deprecated(
    new = "codebook_surveys_create",
    msg = "codebook_waves_create() is deprecated, use codebook_surveys_create() instead."
  )
}

#' @rdname create_codebook
#' @param survey_list A list containing surveys of class survey.
#' @family metadata functions
#' @examples
#' \donttest{
#' examples_dir <- system.file("examples", package = "retroharmonize")
#' survey_list <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
#'
#' example_surveys <- read_surveys(
#'   file.path(examples_dir, survey_list),
#'   save_to_rds = FALSE
#' )
#'
#' codebook_surveys_create(example_surveys)
#' }
#' @export

codebook_surveys_create <- function(survey_list) {
  assertthat::assert_that(inherits(survey_list, "list"),
    msg = "The parameter waves must be a list (of surveys.)"
  )

  assertthat::assert_that(
    all(unlist(lapply(
      survey_list,
      function(x) inherits(x, "survey")
    ))),
    msg = "Every elements of the wave list must be of type survey."
  )

  codebook_list <- lapply(
    survey_list,
    function(x) create_codebook(survey = x)
  )

  do.call(rbind, codebook_list)
}

test_that("create_codebook works with metadata input", {
  survey <- read_rds(
    system.file("examples", "ZA7576.rds", package = "retroharmonize")
  )

  metadata <- metadata_create(survey)
  metadata$user_var <- paste0(seq_len(nrow(metadata)), "_user")

  codebook <- create_codebook(metadata = metadata)

  expect_s3_class(codebook, "data.frame")
  expect_true("user_var" %in% names(codebook))
})


test_that("create_codebook works with survey input", {
  survey <- read_rds(
    system.file("examples", "ZA7576.rds", package = "retroharmonize")
  )

  codebook <- create_codebook(survey = survey)

  expect_s3_class(codebook, "data.frame")
  expect_true(all(
    c("entry", "val_code_orig", "val_label_orig", "label_range") %in%
      names(codebook)
  ))
})


test_that("codebook_waves_create emits deprecation warning", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  survey_files <- dir(examples_dir, pattern = "\\.rds$")
  surveys <- read_surveys(
    file.path(examples_dir, survey_files),
    export_path = NULL
  )

  expect_warning(
    codebook_waves_create(waves = surveys),
    class = "deprecatedWarning"
  )
})


test_that("codebook_surveys_create combines surveys correctly", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  survey_files <- dir(examples_dir, pattern = "\\.rds$")
  surveys <- read_surveys(
    file.path(examples_dir, survey_files),
    export_path = NULL
  )

  codebook <- codebook_surveys_create(survey_list = surveys)

  expect_s3_class(codebook, "data.frame")
  expect_true(all(survey_files %in% unique(codebook$filename)))
  expect_true(all(c("valid", "missing") %in% unique(codebook$label_range)))
})


test_that("create_codebook fails with invalid input", {
  expect_error(
    create_codebook(survey = data.frame()),
    regexp = "Parameter survey must be of class survey"
  )

  expect_error(
    create_codebook(metadata = NULL, survey = NULL),
    regexp = "Either the parameter 'metadata' or the parameter 'survey' must be given"
  )
})

#' Convert labelled missing values to NA
#'
#' Internal helper used by numeric summary methods to replace
#' SPSS-style missing values with `NA`.
#'
#' @param x A labelled survey vector.
#'
#' @return A numeric vector with missing values converted to `NA`.
#'
#' @keywords internal
#' @importFrom labelled na_values
#' @importFrom vctrs vec_data
vec_convert_na <- function(x) {
  ifelse(
    vctrs::vec_data(x) %in% labelled::na_values(x),
    NA_real_,
    vctrs::vec_data(x)
  )
}

#' @rdname labelled_spss_survey
#' @importFrom stats median
#' @inheritParams stats::median
#' @export
median.retroharmonize_labelled_spss_survey <- function(x,
                                                       na.rm = TRUE,
                                                       ...) {
  if (is.character(x)) {
    abort("Can't compute median of labelled_spss_survey<character>")
  }

  median(vec_convert_na(x), na.rm = na.rm, ...)
}


#' @rdname labelled_spss_survey
#' @importFrom stats quantile
#' @inheritParams stats::quantile
#' @export
quantile.retroharmonize_labelled_spss_survey <- function(x,
                                                         probs,
                                                         ...) {
  if (is.character(x)) {
    abort("Can't compute quantiles of labelled_spss_survey<character>")
  }

  quantile(vec_convert_na(x), probs, na.rm = TRUE, ...)
}


#' @rdname labelled_spss_survey
#' @importFrom stats weighted.mean
#' @inheritParams stats::weighted.mean
#' @export
weighted.mean.retroharmonize_labelled_spss_survey <- function(x,
                                                              w,
                                                              ...) {
  if (is.character(x)) {
    abort("Can't compute weighted mean of labelled_spss_survey<character>")
  }

  if (!is.numeric(w)) {
    abort("Weights must be numeric.")
  }

  weighted.mean(vec_convert_na(x), w, na.rm = TRUE, ...)
}


#' @rdname labelled_spss_survey
#' @inheritParams base::mean
#' @export
mean.retroharmonize_labelled_spss_survey <- function(x, ...) {
  if (is.character(x)) {
    abort("Can't compute mean of labelled_spss_survey<character>")
  }

  mean(vec_convert_na(x), ...)
}


#' @rdname labelled_spss_survey
#' @export
sum.retroharmonize_labelled_spss_survey <- function(x, ...) {
  if (is.character(x)) {
    abort("Can't compute sum of labelled_spss_survey<character>")
  }

  sum(vec_convert_na(x), ...)
}

#' @title Document survey lists
#'
#' @description Document the key attributes surveys in a survey list.
#'
#' @details The function has two alternative input parameters. If \code{survey_list} is the
#' input, it returns  the name of the original source data file, the number of rows and
#' columns, and the size of the object as stored in memory. In case \code{survey_paths}
#' contains the source data files, it will sequentially read those files, and add the file
#' size, the last access and the last modified time attributes.
#'
#' @param survey_list A list of \code{\link{survey}} objects.
#' @param survey_paths A vector of full file paths to the surveys to subset, defaults to
#' \code{NULL}.
#' @param .f A function to import the surveys with.
#' Defaults to \code{'read_rds'}. For SPSS files,
#' \code{read_spss} is recommended, which is a
#' well-parameterized version of \code{\link[haven]{read_spss}} that
#' saves some metadata, too. For STATA files use \code{read_dta}.
#' @return Returns a data frame with the key attributes of the surveys
#' in a survey list: the name of the data file, the number of rows and
#' columns, and the size of the object as stored in memory.
#' @importFrom tibble tibble
#' @importFrom here here
#' @family documentation functions
#' @examples
#' examples_dir <- system.file("examples", package = "retroharmonize")
#'
#' my_rds_files <- dir(examples_dir)[grepl(
#'   ".rds",
#'   dir(examples_dir)
#' )]
#'
#' example_surveys <- read_surveys(file.path(examples_dir, my_rds_files))
#'
#' documented <- document_surveys(example_surveys)
#'
#' attr(documented, "original_list")
#' documented
#'
#' document_surveys(survey_paths = file.path(examples_dir, my_rds_files))
#'
#' @export
#' @importFrom utils object.size

document_surveys <- function(survey_list = NULL,
                             survey_paths = NULL,
                             .f = NULL) {
  if (!is.null(survey_list)) {
    validate_survey_list(survey_list)
  } else if (!is.null(survey_paths)) {
    validate_survey_files(survey_paths)
  } else {
    stop(" in document_surveys() - both 'survey_list' and 'survey_path' is NULL.")
  }

  if (!is.null(survey_list)) {
    n_survey <- length(survey_list)

    return_df <- tibble(
      id = vapply(survey_list, function(x) attr(x, "id"), character(1)),
      filename = vapply(survey_list, function(x) attr(x, "filename"), character(1)),
      ncol = vapply(survey_list, ncol, integer(1)),
      nrow = vapply(survey_list, nrow, integer(1)),
      object_size = vapply(survey_list, utils::object.size, double(1))
    )

    attr(return_df, "original_list") <- deparse(substitute(survey_list))

    return_df
  } else {
    survey_files <- fs::path_file(survey_paths)

    tmp <- tibble(
      id = fs::path_ext_remove(survey_files),
      filename = survey_files,
      ncol = NA_real_,
      nrow = NA_real_,
      object_size = NA_real_,
      file_size = NA_real_,
      accessed = NA_character_,
      last_modified = NA_character_
    )

    for (i in seq_along(survey_files)) {
      file_info <- file.info(survey_paths[i])
      tmp$file_size[i] <- file_info$size
      tmp$accessed[i] <- as.character(file_info$atime)
      tmp$last_modified[i] <- as.character(file_info$mtime)
    }

    for (i in seq_along(survey_files)) {
      message(length(i), "/", i, " ", survey_files[i])
      tmp_survey <- read_survey(file_path = survey_paths[i])
      if (!is.null(tmp_survey)) {
        tmp$object_size[i] <- object.size(tmp_survey)
        tmp$ncol[i] <- ncol(tmp_survey)
        tmp$nrow[i] <- nrow(tmp_survey)
      } else {
        next
      }
    }

    return_df <- tmp
  }

  return_df
}

#' @rdname document_surveys
#' @details The earlier form \code{document_waves} is deprecated.
#' Currently called \code{\link{document_surveys}}.
#' @param waves  A list of \code{\link{survey}} objects.
#' @export

document_waves <- function(waves) {
  .Deprecated("document_surveys",
    msg = "document_waves() is deprecated. Use document_surveys() instead"
  )
  merge_surveys(survey_list = waves)
}

examples_dir <- system.file("examples", package = "retroharmonize")

my_rds_files <- dir(examples_dir)[grepl(
  ".rds",
  dir(examples_dir)
)]

example_surveys <- read_surveys(file.path(examples_dir, my_rds_files))


documented <- document_surveys(example_surveys)

test_that("correct attributes are returned", {
  expect_equal(documented$id, c("ZA5913", "ZA6863", "ZA7576"))
})

documented_files <- document_surveys(survey_paths = file.path(examples_dir, my_rds_files))

#' Labelled SPSS-style vectors with survey provenance
#'
#' Create a labelled vector compatible with
#' [haven::labelled_spss()] that carries additional
#' survey-level provenance metadata.
#'
#' The resulting object behaves like a
#' `haven_labelled_spss` vector, but stores:
#' \itemize{
#'   \item a survey identifier;
#'   \item the original variable name;
#'   \item the original value coding.
#' }
#'
#' @details
#' Several arithmetic and statistical summary methods operate on
#' the numeric representation of labelled survey vectors,
#' converting SPSS-style missing values to `NA` before computation.
#'
#' You can coerce `labelled_spss_survey` vectors to numeric,
#' character or factor representation.
#'
#' @param x A vector of values.
#' @param labels A named vector of value labels.
#' @param na_values A vector of values to be treated as missing.
#' @param na_range A numeric range defining missing values.
#' @param label A variable label.
#' @param id A character scalar identifying the survey.
#' @param name_orig Original variable name. Defaults to the name of `x`.
#'
#' @return
#' An object of class `"retroharmonize_labelled_spss_survey"`,
#' extending [haven::labelled_spss()].
#'
#' @seealso
#' [haven::labelled_spss()],
#' [as_factor()],
#' [as_numeric()],
#' [as_character()]
#'
#' @examples
#' x <- labelled_spss_survey(
#'   x = c(1, 2, 9),
#'   labels = c(Yes = 1, No = 2),
#'   na_values = 9,
#'   id = "survey_1"
#' )
#'
#' is.na(x)
#' as_factor(x)
#'
#' @export
labelled_spss_survey <- function(
  x = double(),
  labels = NULL,
  na_values = NULL,
  na_range = NULL,
  label = NULL,
  id = NULL,
  name_orig = NULL
) {
  x_vector <- vctrs::vec_data(x)
  na_values <- vec_cast_named(na_values, x_vector,
    x_arg = "na_values", to_arg = "x"
  )
  labelled <- labelled::labelled(x, labels = labels)

  if (is.null(name_orig)) {
    name_orig <- deparse(substitute(x))
  }

  if (is.null(id)) id <- name_orig

  new_labelled_spss_survey(
    vctrs::vec_data(labelled),
    labels = labels,
    label = label,
    na_values = na_values,
    na_range = na_range,
    id = id,
    name_orig = name_orig
  )
}

#' @keywords internal
new_labelled_spss_survey <- function(x,
                                     labels,
                                     na_values,
                                     na_range,
                                     label,
                                     id,
                                     name_orig) {
  if (!is.null(na_values) && !vctrs::vec_is(x, na_values)) {
    stop("`na_values` must be same type as `x`.",
      call. = FALSE
    )
  }

  if (!is.null(na_range)) {
    if (!is.numeric(x)) {
      stop("`na_range` is only applicable for labelled numeric vectors.",
        call. = FALSE
      )
    }
    if (!is.numeric(na_range) || length(na_range) != 2) {
      stop("`na_range` must be a numeric vector of length two.",
        call. = FALSE
      )
    }
  }

  if (is.null(label)) {
    # haven no longer allows NULL labels
    label <- ""
  }

  tmp <- haven::labelled_spss(vctrs::vec_data(x),
    labels = labels,
    label = label,
    na_values = na_values,
    na_range = na_range
  )

  original_coding <- sort(unique(x))
  names(original_coding) <- original_coding

  attr(tmp, "class") <- c(
    "retroharmonize_labelled_spss_survey",
    "haven_labelled_spss",
    "haven_labelled"
  )
  if (length(id) == 1) {
    attr(tmp, paste0(id, "_name")) <- name_orig
    attr(tmp, paste0(id, "_values")) <- original_coding
    attr(tmp, paste0(id, "_label")) <- label
    attr(tmp, paste0(id, "_labels")) <- attr(tmp, "labels")
    attr(tmp, paste0(id, "_na_values")) <- attr(tmp, "na_values")
    attr(tmp, paste0(id, "_na_range")) <- attr(tmp, "na_range")
  } else {
    id <- paste(id, collapse = ", ")
  }

  attr(tmp, "id") <- id

  tmp
}

## Subsetting -------------------------------------------------

#' @rdname labelled_spss_survey
#' @param i Index vector used for subsetting.
#' @export
`[.retroharmonize_labelled_spss_survey` <- function(x, i, ...) {
  preserve_structure <- attributes(x)
  x <- vctrs::vec_data(x)[i]
  attributes(x) <- preserve_structure
  x
}

## Displaying methods -----------------------------------------
#' @keywords internal
get_labeltext <- function(x, prefix = ": ") {
  label <- attr(x, "label", exact = TRUE)
  if (!is.null(label)) {
    paste0(prefix, label)
  }
}

#' @rdname labelled_spss_survey
#' @param object A labelled_spss_survey to summarize.
#' @export
summary.retroharmonize_labelled_spss_survey <- function(object, ...) {
  if (!is.null(attr(object, "label"))) {
    cat(attr(object, "label"))
  }
  cat("\nNumeric values without coding:\n")
  print(summary(vctrs::vec_data(object), ...))
  cat("Numeric representation:\n")
  print(summary(as_numeric(object)))
  cat("Factor representation:\n")
  summary(as_factor(object))
}

#' @rdname labelled_spss_survey
#' @param value Replacement values used when assigning names.
#' @export
"names<-.retroharmonize_labelled_spss_survey" <- function(x, value) {
  attr(x, "names") <- value
  x
}

## Missingness ------------------------------------------------------

#' @rdname labelled_spss_survey
#' @export
is.na.retroharmonize_labelled_spss_survey <- function(x) {
  miss <- NextMethod()
  val <- vctrs::vec_data(x)

  na_values <- attr(x, "na_values")
  if (!is.null(na_values)) {
    miss <- miss | val %in% na_values
  }

  na_range <- attr(x, "na_range")
  if (!is.null(na_range)) {
    miss <- miss | (val >= na_range[1] & val <= na_range[2])
  }

  miss
}

#' @rdname labelled_spss_survey
#' @exportS3Method
levels.retroharmonize_labelled_spss_survey <- function(x) {
  NULL
}

#' @rdname labelled_spss_survey
#' @importFrom haven format_tagged_na
#' @param digits Number of digits to use in string representation in
#' the format method.
#' @export
format.retroharmonize_labelled_spss_survey <- function(
  x,
  ...,
  digits = getOption("digits")
) {
  if (is.double(x)) {
    haven::format_tagged_na(x, digits = digits)
  } else {
    format(vctrs::vec_data(x), ...)
  }
}

# Type system -------------------------------------------------------------

#' @rdname labelled_spss_survey
#' @export
is.labelled_spss_survey <- function(x) {
  inherits(x, "retroharmonize_labelled_spss_survey")
}

test_that("arithmetic methods work", {
  x1 <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )
  expect_equal(
    sum(x1, na.rm = TRUE),
    sum(1:8)
  )
  expect_equal(
    mean(x1, na.rm = TRUE),
    mean(1:8)
  )
  expect_equal(
    median(x1),
    median(1:8)
  )
  expect_equal(
    quantile(x1, 0.25),
    quantile(1:8, 0.25)
  )
  expect_equal(
    weighted.mean(x1, w = c(2, rep(1, 9))),
    weighted.mean(1:8, w = c(2, rep(1, 7)))
  )
})

test_that("vec_ptype_full returns readable type", {
  x <- labelled_spss_survey(
    x = 1:3,
    labels = c(Yes = 1),
    id = "survey1"
  )

  out <- vctrs::vec_ptype_full(x)

  expect_type(out, "character")
  expect_length(out, 1)

  expect_match(
    out,
    "labelled_spss_survey"
  )
})


test_that("vec_ptype_abbr returns compact abbreviation", {
  x <- labelled_spss_survey(
    x = 1:3,
    labels = c(Yes = 1),
    id = "survey1"
  )

  out <- vec_ptype_abbr(x)

  expect_equal(out, "retroh_int")
})


test_that("format returns character vector", {
  x <- labelled_spss_survey(
    x = c(1, 2, 9),
    labels = c(Yes = 1, No = 2),
    na_values = 9,
    id = "survey1"
  )

  out <- format(x)

  expect_type(out, "character")
  expect_length(out, 3)
})


test_that("print method works", {
  x <- labelled_spss_survey(
    x = c(1, 2, 9),
    labels = c(Yes = 1, No = 2),
    na_values = 9,
    label = "Question text",
    id = "survey1"
  )

  expect_snapshot_output(
    print(x)
  )
})


test_that("pillar shaft works", {
  x <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )

  expect_snapshot_output(
    pillar::pillar_shaft(tibble::tibble(v1 = x))
  )
})


test_that("pillar display works", {
  x <- labelled_spss_survey(
    x = 1:10,
    labels = c(Good = 1, Bad = 8),
    na_values = c(9, 10),
    id = "survey1"
  )

  expect_snapshot_output(
    pillar::pillar(x)
  )
})

test_that("ptype2 with base numeric types returns double prototype", {
  x <- labelled_spss_survey(
    x = c(1, 0, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    id = "survey1"
  )

  expect_equal(vctrs::vec_ptype2(x, double()), double())
  expect_equal(vctrs::vec_ptype2(double(), x), double())

  expect_equal(vctrs::vec_ptype2(x, integer()), double())
  expect_equal(vctrs::vec_ptype2(integer(), x), double())
})

test_that("cast to base numeric types drops labelled_spss_survey class", {
  x <- labelled_spss_survey(
    x = c(1, 0, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    id = "survey1"
  )

  out_double <- vctrs::vec_cast(x, double())
  out_integer <- vctrs::vec_cast(x, integer())

  expect_equal(out_double, c(1, 0, 9))
  expect_type(out_double, "double")
  expect_false(inherits(out_double, "retroharmonize_labelled_spss_survey"))

  expect_equal(out_integer, c(1L, 0L, 9L))
  expect_type(out_integer, "integer")
  expect_false(inherits(out_integer, "retroharmonize_labelled_spss_survey"))
})


test_that("compatible labelled survey vectors have labelled survey prototype", {
  x <- labelled_spss_survey(
    x = c(1, 0, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    label = "Question",
    id = "survey1",
    name_orig = "q1"
  )

  y <- labelled_spss_survey(
    x = c(0, 1, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    label = "Question",
    id = "survey2",
    name_orig = "q1"
  )

  ptype <- vctrs::vec_ptype2(x, y)

  expect_s3_class(ptype, "retroharmonize_labelled_spss_survey")
  expect_equal(attr(ptype, "labels"), c(yes = 1, no = 0, inap = 9))
  expect_equal(attr(ptype, "na_values"), 9)
  expect_equal(attr(ptype, "id"), "multi-wave")
})


test_that("vec_c combines compatible labelled survey vectors", {
  x <- labelled_spss_survey(
    x = c(1, 0, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    label = "Question",
    id = "survey1",
    name_orig = "q1"
  )

  y <- labelled_spss_survey(
    x = c(0, 1, 9),
    labels = c(yes = 1, no = 0, inap = 9),
    na_values = 9,
    label = "Question",
    id = "survey2",
    name_orig = "q1"
  )

  out <- vctrs::vec_c(x, y)

  expect_s3_class(out, "retroharmonize_labelled_spss_survey")
  expect_equal(vctrs::vec_data(out), c(1, 0, 9, 0, 1, 9))
  expect_equal(attr(out, "labels"), c(yes = 1, no = 0, inap = 9))
  expect_equal(attr(out, "na_values"), 9)
})


test_that("vec_c errors when value labels differ", {
  x <- labelled_spss_survey(
    x = c(1, 0),
    labels = c(yes = 1, no = 0),
    id = "survey1"
  )

  y <- labelled_spss_survey(
    x = c(1, 0),
    labels = c(ja = 1, nee = 0),
    id = "survey2"
  )

  expect_error(vctrs::vec_c(x, y))
})


test_that("vec_c errors when labelled numeric values differ", {
  x <- labelled_spss_survey(
    x = c(1, 0),
    labels = c(yes = 1, no = 0),
    id = "survey1"
  )

  y <- labelled_spss_survey(
    x = c(1, 2),
    labels = c(yes = 1, no = 2),
    id = "survey2"
  )

  expect_error(vctrs::vec_c(x, y))
})


test_that("vec_c errors when na_values differ", {
  x <- labelled_spss_survey(
    x = c(1, 9),
    labels = c(yes = 1, inap = 9),
    na_values = 9,
    id = "survey1"
  )

  y <- labelled_spss_survey(
    x = c(1, 8),
    labels = c(yes = 1, inap = 9),
    na_values = 8,
    id = "survey2"
  )

  expect_error(vctrs::vec_c(x, y))
})


test_that("vec_c errors when na_range differs", {
  x <- labelled_spss_survey(
    x = c(1, 9),
    labels = c(yes = 1),
    na_range = c(8, 9),
    id = "survey1"
  )

  y <- labelled_spss_survey(
    x = c(1, 9),
    labels = c(yes = 1),
    na_range = c(9, 10),
    id = "survey2"
  )

  expect_error(vctrs::vec_c(x, y))
})


test_that("character cast only works for character labelled survey vectors", {
  x_chr <- labelled_spss_survey(
    x = c("a", "b"),
    labels = c(A = "a", B = "b"),
    id = "survey1"
  )

  x_num <- labelled_spss_survey(
    x = c(1, 2),
    labels = c(A = 1, B = 2),
    id = "survey1"
  )

  expect_equal(
    vctrs::vec_cast(x_chr, character()),
    c("a", "b")
  )

  expect_error(
    vctrs::vec_cast(x_num, character())
  )
})

#' @title Data Input
#'
#' @description See \code{utils::\link[utils:read.table]{read.csv}} for details.
#'
#' @name read.csv
#' @rdname read.csv
#' @keywords internal
#' @export
#' @importFrom utils read.csv
NULL

#' @title Harmonize values in surveys
#'
#' @description
#' Harmonize value codes and value labels across multiple surveys and
#' combine them into a single data frame.
#'
#' @details
#' The function first aligns the structure of all surveys by ensuring that
#' they contain the same set of variables. Missing variables are added and
#' filled with appropriate missing values depending on their type.
#'
#' Variables of class \code{"retroharmonize_labelled_spss_survey"} are then
#' harmonized by applying a user-supplied function \code{.f} to each variable
#' separately within each survey.
#'
#' The harmonization function \code{.f} must return a vector of the same length
#' as its input. If \code{.f} returns \code{NULL}, the original variable is kept
#' unchanged.
#'
#' Prior to version 0.2.0 this function was called \code{harmonize_waves}.
#'
#' @param survey_list
#' A list of surveys (data frames). In earlier versions this argument was called
#' \code{waves}.
#'
#' @param .f
#' A function applied to each labelled variable
#' (class \code{"retroharmonize_labelled_spss_survey"}). The function must not
#' change the length of the input vector.
#'
#' @param status_message
#' Logical. If \code{TRUE}, prints the identifier of each survey as it is
#' processed.
#'
#' @return
#' A data frame containing the row-wise combination of all surveys, with
#' harmonized labelled variables and preserved attributes describing the
#' original surveys.
#'
#' @family harmonization functions
#'
#' @export
#'
#' @importFrom dplyr select bind_cols mutate_all pull
#' @importFrom tidyselect all_of
#' @importFrom tibble as_tibble
#' @importFrom rlang set_names
#' @importFrom haven labelled_spss
#'
#' @examples
#' \donttest{
#' examples_dir <- system.file("examples", package = "retroharmonize")
#' survey_files <- dir(examples_dir, pattern = "\\.rds$", full.names = TRUE)
#'
#' surveys <- read_surveys(
#'   survey_files,
#'   export_path = NULL
#' )
#'
#' # Keep only supported variable types
#' surveys <- lapply(
#'   surveys,
#'   function(s) {
#'     s[, vapply(
#'       s,
#'       function(x) {
#'         inherits(x, c(
#'           "retroharmonize_labelled_spss_survey",
#'           "numeric",
#'           "character",
#'           "Date"
#'         ))
#'       },
#'       logical(1)
#'     )]
#'   }
#' )
#'
#' # Identity harmonization (no-op)
#' harmonized <- harmonize_survey_values(
#'   survey_list = surveys,
#'   .f = function(x) x,
#'   status_message = FALSE
#' )
#'
#' head(harmonized)
#' }
harmonize_survey_values <- function(
  survey_list,
  .f,
  status_message = FALSE
) {
  validate_survey_list(survey_list)

  all_names <- unique(unlist(lapply(survey_list, names)))

  ## classify variables robustly
  classes <- unlist(lapply(
    survey_list,
    function(x) {
      lapply(x, function(y) {
        if (inherits(y, "retroharmonize_labelled_spss_survey")) {
          "retroharmonize_labelled_spss_survey"
        } else if (inherits(y, c("numeric", "double", "integer"))) {
          "numeric"
        } else if (inherits(y, "character")) {
          "character"
        } else if (inherits(y, "Date")) {
          "Date"
        } else {
          "other"
        }
      })
    }
  ))

  retroharmonized <- unique(names(classes)[classes == "retroharmonize_labelled_spss_survey"])
  numerics <- unique(names(classes)[classes == "numeric"])
  characters <- unique(names(classes)[classes == "character"])
  dates <- unique(names(classes)[classes == "Date"])
  other_types <- names(classes)[classes == "other"]

  if (length(other_types) > 0) {
    stop(
      "Only labelled_spss_survey, numeric, character and Date types are allowed",
      call. = FALSE
    )
  }


  original_attributes <- document_surveys(survey_list)

  ## ---- extend surveys so all have same columns ----
  extend_survey <- function(dat) {
    to_add_rh <- setdiff(retroharmonized, names(dat))
    to_add_num <- setdiff(numerics, names(dat))
    to_add_chr <- setdiff(characters, names(dat))
    to_add_date <- setdiff(dates, names(dat))

    out <- dat

    if (length(to_add_num) > 0) {
      out <- dplyr::bind_cols(
        out,
        as.data.frame(matrix(NA_real_, nrow(dat), length(to_add_num)),
          stringsAsFactors = FALSE
        ) %>%
          rlang::set_names(to_add_num)
      )
    }

    if (length(to_add_chr) > 0) {
      out <- dplyr::bind_cols(
        out,
        as.data.frame(matrix(NA_character_, nrow(dat), length(to_add_chr)),
          stringsAsFactors = FALSE
        ) %>%
          rlang::set_names(to_add_chr)
      )
    }

    if (length(to_add_date) > 0) {
      out <- dplyr::bind_cols(
        out,
        as.data.frame(matrix(as.Date(NA), nrow(dat), length(to_add_date)),
          stringsAsFactors = FALSE
        ) %>%
          rlang::set_names(to_add_date)
      )
    }

    if (length(to_add_rh) > 0) {
      add_rh_df <- as.data.frame(
        matrix(99999, nrow(dat), length(to_add_rh)),
        stringsAsFactors = FALSE
      ) %>%
        rlang::set_names(to_add_rh)

      add_rh_df <- dplyr::mutate_all(add_rh_df, function(x) {
        haven::labelled_spss(
          x,
          labels = c(inap = 99999),
          na_values = c(
            "do_not_know" = 99997,
            "declined" = 99998,
            "inap" = 99999
          )
        )
      })

      for (i in seq_len(ncol(add_rh_df))) {
        attr(add_rh_df[, i], "label") <- to_add_rh[i]
      }

      add_rh_df <- tibble::as_tibble(
        lapply(add_rh_df, function(x) {
          as_labelled_spss_survey(x, attr(dat, "id"))
        })
      )

      out <- dplyr::bind_cols(out, add_rh_df)
    }

    dplyr::select(out, tidyselect::all_of(all_names))
  }

  extended <- lapply(survey_list, extend_survey)

  ## ---- harmonize labelled variables ----
  to_harmonize <- lapply(
    extended,
    function(x) dplyr::select(x, tidyselect::all_of(retroharmonized))
  )

  fn_harmonize <- function(dat) {
    harmonized <- lapply(dat, function(x) {
      out <- .f(x)

      if (is.null(out)) {
        return(x)
      }

      if (length(out) != length(x)) {
        stop(
          "Harmonization function must not change vector length",
          call. = FALSE
        )
      }

      out
    })

    tibble::as_tibble(harmonized)
  }

  harmonized <- lapply(to_harmonize, fn_harmonize)

  ## ---- bind everything together ----
  result <- harmonized[[1]]
  for (i in 2:length(harmonized)) {
    result <- vctrs::vec_rbind(result, harmonized[[i]])
  }

  attr(result, "id") <- paste("Surveys:", paste(original_attributes$id, collapse = "; "))
  attr(result, "filename") <- paste("Original files:", paste(original_attributes$filename, collapse = "; "))

  result
}


#' @rdname harmonize_survey_values
#' @details The earlier form \code{harmonize_waves} is deprecated.
#' The function is currently called \code{\link{harmonize_waves}}.
#' @param waves A list of surveys. Deprecated.
#' @export


harmonize_waves <- function(waves, .f, status_message = FALSE) {
  .Deprecated("harmonize_waves ",
    msg = "harmonize_waves() is deprecated, use harmonize_survey_values() instead",
    old = "harmonize_waves"
  )
  harmonize_survey_values(
    survey_list = waves,
    .f = .f,
    status_message = status_message
  )
}

example_survey_paths <- function() {
  examples_dir <- system.file("examples", package = "retroharmonize")
  dir(examples_dir, pattern = "\\.rds$", full.names = TRUE)
}

example_supported_vars <- function(s) {
  s[, vapply(
    s,
    function(x) {
      inherits(x, c(
        "retroharmonize_labelled_spss_survey",
        "numeric",
        "character",
        "Date"
      ))
    },
    logical(1)
  )]
}

test_that("harmonize_survey_values errors on unsupported variable types", {
  s <- read_rds(
    system.file("examples", "ZA7576.rds", package = "retroharmonize")
  )

  # Inject unsupported type: logical
  s$logical_flag <- s$rowid == s$rowid

  expect_error(
    harmonize_survey_values(
      survey_list = list(s),
      .f = identity,
      status_message = FALSE
    ),
    "Only labelled_spss_survey, numeric, character and Date types are allowed"
  )
})


test_that(
  "harmonize_survey_values works on example surveys after
  filtering supported types",
  {
    examples_dir <- system.file("examples", package = "retroharmonize")
    survey_files <- dir(examples_dir, pattern = "\\.rds$", full.names = TRUE)

    surveys <- read_surveys(
      survey_files,
      export_path = NULL
    )

    surveys <- lapply(
      surveys,
      function(s) {
        s[, vapply(
          s,
          function(x) {
            inherits(x, c(
              "retroharmonize_labelled_spss_survey",
              "numeric",
              "character",
              "Date"
            ))
          },
          logical(1)
        )]
      }
    )

    expect_no_error(
      harmonize_survey_values(
        survey_list = surveys,
        .f = function(x) x,
        status_message = FALSE
      )
    )
  }
)

#' Read a survey from an `.rds` file
#'
#' Import a serialized survey object stored in `.rds` format and return
#' it as a `survey` object with harmonized metadata attributes.
#'
#' This function restores survey objects previously saved with
#' [base::saveRDS()] or exported from the `retroharmonize`
#' workflow. The returned object retains survey metadata and gains
#' additional provenance attributes such as source file name and file size.
#'
#' @param file Path to an `.rds` file containing a survey object.
#' @param dataset_bibentry Optional bibliographic metadata created with
#'   [dataset::dublincore()] or [dataset::datacite()].
#' @param id Optional survey identifier. Defaults to the file name
#'   without extension.
#' @param doi Optional DOI identifier for the survey.
#'
#' @return A `survey` object inheriting from `data.frame` and `tbl_df`
#'   with survey metadata attributes.
#'
#' @details
#' If the file cannot be read, an empty `survey` object is returned
#' and a warning is emitted.
#'
#' The function:
#'
#' - restores the serialized object,
#' - validates source file information,
#' - normalizes `rowid`,
#' - records provenance metadata,
#' - and stores object and source file sizes as attributes.
#'
#' @family import functions
#'
#' @examples
#' path <- system.file(
#'   "examples",
#'   "ZA7576.rds",
#'   package = "retroharmonize"
#' )
#'
#' survey_object <- read_rds(path)
#'
#' attr(survey_object, "id")
#' attr(survey_object, "filename")
#' attr(survey_object, "doi")
#'
#' @importFrom dataset dataset_df
#' @importFrom fs path_ext_remove path_file
#' @importFrom labelled var_label<-
#' @importFrom purrr safely
#' @importFrom tibble as_tibble
#' @importFrom utils object.size
#'
#' @export

read_rds <- function(file,
                     dataset_bibentry = NULL,
                     id = NULL,
                     doi = NULL) {
  source_file_info <- valid_file_info(file)
  filename <- fs::path_file(file)

  if (is.null(id)) {
    id <- fs::path_ext_remove(filename)
  }

  safely_readRDS <- purrr::safely(readRDS)

  tmp <- safely_readRDS(file = file)

  if (!is.null(tmp$error)) {
    warning(tmp$error, "\nReturning an empty survey.")
    return(
      survey(data.frame(), id = "Could not read file", filename = filename, doi = doi)
    )
  } else {
    tmp <- tmp$result
  }

  # if ( ! "rowid" %in% names(tmp) ) {
  #  tmp <- tibble::rowid_to_column(tmp)
  # }

  tmp_df <- dataset_df(tmp,
    identifier = doi,
    dataset_bibentry = dataset_bibentry
  )

  if (is.null(doi)) {
    if ("doi" %in% colnames(tmp_df)) {
      doi <- tmp_df$doi[1]
    } else {
      doi <- ""
    }
  }

  tmp_df$rowid <- paste0(
    id,
    "_",
    gsub(paste0("^", id, "_?"), "", tmp_df$rowid)
  )

  labelled::var_label(
    tmp_df$rowid
  ) <- paste0("Unique identifier in ", id)

  return_survey <- survey(tmp_df, id = id, filename = filename, doi = doi)

  object_size <- as.numeric(object.size(as_tibble(tmp_df)))
  attr(return_survey, "object_size") <- object_size
  attr(return_survey, "source_file_size") <- source_file_info$size

  return_survey
}

test_that("harmonize_na_values returns input unchanged when there are no SPSS labelled variables", {
  df <- tibble::tibble(
    x = 1:3,
    y = c("a", "b", "c")
  )

  expect_identical(
    harmonize_na_values(df),
    df
  )
})


test_that("harmonize_na_values preserves survey class", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )

  tested <- harmonize_na_values(df = test_read)

  expect_s3_class(tested, "survey")
})


test_that("harmonize_na_values keeps row count and column names", {
  examples_dir <- system.file("examples", package = "retroharmonize")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )

  tested <- harmonize_na_values(df = test_read)

  expect_equal(nrow(tested), nrow(test_read))
  expect_equal(names(tested), names(test_read))
})


test_that("current dk behaviour is documented", {
  x <- haven::labelled_spss(
    x = c(1, 8, 9),
    labels = c(
      yes = 1,
      dk = 8,
      inap = 9
    ),
    na_values = c(8, 9)
  )

  df <- tibble::tibble(x = x)

  out <- harmonize_na_values(df)

  expect_equal(
    unname(as.vector(out$x)),
    c(1, 8, 99999)
  )
})

#' Harmonize variable names across surveys
#'
#' Harmonize variable names in a list of survey objects using
#' a metadata crosswalk table.
#'
#' @description
#' `harmonize_var_names()` renames variables across multiple
#' surveys to a shared harmonized naming scheme.
#'
#' The harmonization rules are defined in a metadata table,
#' typically created with [metadata_create()].
#'
#' @details
#' The function can also be used for survey subsetting workflows.
#' If `metadata` contains only a subset of variables for a survey,
#' only those variables are retained in the harmonized output.
#'
#' @param survey_list A list of survey objects, typically imported
#'   with [read_surveys()].
#'
#' @param metadata A metadata table containing harmonization rules.
#'   Typically created with [metadata_create()] and combined across
#'   surveys.
#'
#' @param old Name of the column in `metadata` containing
#'   the original variable names.
#'
#' @param new Name of the column in `metadata` containing
#'   the harmonized variable names.
#'
#' @param rowids Logical. Should original `rowid` variables
#'   be renamed to `"uniqid"`?
#'
#' @return
#' A list of surveys with harmonized variable names.
#'
#' @family harmonization functions
#'
#' @seealso
#' [metadata_create()],
#' [crosswalk()]
#'
#' @examples
#' examples_dir <- system.file(
#'   "examples",
#'   package = "retroharmonize"
#' )
#'
#' survey_files <- dir(
#'   examples_dir,
#'   pattern = "\\.rds$"
#' )
#'
#' example_surveys <- read_surveys(
#'   file.path(examples_dir, survey_files)
#' )
#'
#' metadata <- metadata_create(
#'   example_surveys
#' )
#'
#' metadata$var_name_suggested <-
#'   label_normalize(metadata$var_name)
#'
#' metadata$var_name_suggested[
#'   metadata$label_orig == "age_education"
#' ] <- "age_education"
#'
#' harmonized_surveys <- harmonize_var_names(
#'   survey_list = example_surveys,
#'   metadata = metadata
#' )
#'
#' harmonized_surveys[[1]]
#'
#' @importFrom assertthat assert_that
#' @importFrom dplyr inner_join left_join mutate select
#' @importFrom glue glue
#' @importFrom rlang set_names
#' @importFrom tidyselect all_of
#'
#' @export

harmonize_var_names <- function(survey_list,
                                metadata,
                                old = "var_name_orig",
                                new = "var_name_suggested",
                                rowids = TRUE) {
  assert_that(all(c(new, old, "filename") %in% names(metadata)),
    msg = glue::glue("'{old}', '{new}' and 'filename' must be column names in metadata.")
  )

  metadata <- metadata %>%
    select(all_of(c(old, new, "filename"))) %>%
    set_names(c("var_name_orig", "var_name_suggested", "filename"))


  rename_this_survey <- function(this_survey) {
    this_metadata <- metadata[attr(this_survey, "filename") == metadata$filename, ]

    if (!attr(this_survey, "filename") %in% metadata$filename) {
      warning(
        glue::glue(
          "The metadata of {attr(this_survey, 'filename')} cannot be found"
        )
      )
    }

    renaming <- data.frame(var_name_orig = names(this_survey)) %>%
      inner_join(
        this_metadata %>%
          select(all_of(c("var_name_orig", "var_name_suggested"))),
        by = "var_name_orig"
      )

    subset_this_survey <- this_survey %>%
      select(all_of(renaming$var_name_orig))

    rlang::set_names(subset_this_survey,
      nm = renaming$var_name_suggested
    )
  }

  lapply(survey_list, rename_this_survey)
}

#' Harmonize SPSS-style missing value ranges
#'
#' Ensure consistency between SPSS-style missing value ranges
#' (`na_range`) and explicit missing values (`na_values`)
#' for labelled survey vectors.
#'
#' When both attributes are present, this function:
#' \itemize{
#'   \item adjusts the missing range if it conflicts with existing missing values,
#'   \item derives missing values from the range when necessary,
#'   \item leaves non-SPSS-labelled vectors unchanged.
#' }
#'
#' This harmonization is important before joining, binding, or
#' summarizing survey data.
#'
#' @param x A labelled vector created with
#'   [haven::labelled_spss()] or
#'   `retroharmonize_labelled_spss_survey`.
#'
#' @return
#' The input vector with harmonized `na_values` and `na_range`
#' attributes. If no harmonization is needed, `x` is returned
#' unchanged.
#'
#' @seealso
#' [labelled::na_range()],
#' [labelled::na_values()],
#' [as_numeric()]
#'
#' @family variable label harmonization functions
#' @export
na_range_to_values <- function(x) {
  if (!inherits(x, "haven_labelled_spss")) {
    return(x)
  }
  if (is.null(labelled::na_range(x))) {
    return(x)
  }

  na_values <- numeric(0)

  if (!is.null(labelled::na_values(x))) {
    na_values <- labelled::na_values(x)
  }

  na_min <- labelled::na_range(x)[1]
  na_max <- labelled::na_range(x)[2]

  if (length(na_values) > 0) {
    if (min(na_values) < na_min) {
      warning("Inconsistent missing ranges: min(na_values) < min(na_range)")
      na_min <- min(na_values)
    }

    if (max(na_values) > na_max) {
      warning("Inconsistent missing ranges: max(na_values) > max(na_range)")
      na_max <- max(na_values)
    }
  }

  present_na <- unclass(x)[unclass(x) >= na_min & unclass(x) <= na_max]

  if (length(present_na) == 0) {
    present_na <- na_min:na_max
  }

  labelled::na_values(x) <- present_na
  labelled::na_range(x) <- c(na_min, na_max)

  x
}

#' Test whether missing values need harmonization
#'
#' Checks whether both `na_values` and `na_range` attributes
#' are present on a labelled vector.
#'
#' @param x A labelled vector.
#' @return Logical scalar.
#'
#' @keywords internal
#' @export
is.na_range_to_values <- function(x) {
  !is.null(labelled::na_values(x)) &&
    !is.null(labelled::na_range(x))
}

#' Create a survey object
#'
#' Construct a survey object from a data frame or tibble by attaching
#' survey-level metadata such as an identifier, source filename, and
#' basic dataset-level descriptive metadata.
#'
#' This function is primarily intended for use by import helpers such as
#' [read_rds()], [read_spss()], [read_dta()], and [read_csv()].
#' Most users will not need to call it directly.
#'
#' @param x A data frame or tibble containing the survey data.
#' @param title Optional title for the survey. Defaults to `"Untitled Survey"`.
#' @param creator A [utils::person()] object describing the dataset creator.
#'   Defaults to `person("Unknown", "Creator")`.
#' @param dataset_bibentry Optional dataset-level bibliographic metadata.
#'   If `NULL`, a minimal DataCite entry is created automatically using
#'   `title`, `creator`, and `dataset_subject`.
#' @param dataset_subject Dataset subject metadata.
#'   If `NULL`, defaults to the Library of Congress Subject Heading
#'   \href{https://id.loc.gov/authorities/subjects/sh85130875}{Surveys}.
#' @param identifier A character scalar identifying the survey.
#' @param filename A character scalar giving the source filename, or `NULL`
#'   if unknown.
#'
#' @param ... potentially further arguments for methods.
#'
#' @return
#' An object of class `"survey_df"`, which is a data frame with additional
#' survey-level metadata stored as attributes and dataset-level metadata
#' stored using the `dataset` package.
#'
#' @seealso
#' [read_survey()] for importing survey data from external files.
#'
#' @family importing functions
#'
#' @importFrom utils bibentry person
#' @importFrom dataset is.dataset_df dataset_df datacite subject_create
#' @importFrom dataset 'identifier<-'
#'
#' @examples
#' survey_df(
#'   x = data.frame(
#'     rowid = 1:6,
#'     observations = runif(6)
#'   ),
#'   identifier = "example",
#'   filename = "no_file"
#' )
#'
#' @export

survey_df <- function(x,
                      title = NULL,
                      creator = person("Unknown", "Creator"),
                      dataset_bibentry = NULL,
                      dataset_subject = NULL,
                      identifier,
                      filename) {
  if (is.null(dataset_subject)) {
    dataset_subject <- dataset::subject_create(
      term = "Surveys",
      subjectScheme = "Library of Congress Subject Headings (LCSH)",
      schemeURI = "https://id.loc.gov/authorities/subjects.html",
      valueURI = "http://id.loc.gov/authorities/subjects/sh85130875",
      classificationCode = NULL,
      prefix = ""
    )
  }

  Title <- if (is.null(title)) "Untitled Survey" else title

  Creator <- creator

  if (is.null(dataset_bibentry)) {
    dataset_bibentry <- dataset::datacite(
      Title = Title,
      Creator = Creator,
      Subject = dataset_subject
    )
  }

  if (!is.dataset_df(x)) {
    tmp <- dataset::dataset_df(x,
      dataset_bibentry = dataset_bibentry,
      identifier = identifier,
      filename = filename
    )
  } else {
    tmp <- x
    identifier(tmp) <- identifier
  }

  dataset::subject(tmp) <- dataset_subject
  attr(tmp, "filename") <- filename
  attr(tmp, "id") <- identifier
  class(tmp) <- c("survey_df", class(tmp))
  tmp
}

#' @rdname survey_df
#' @export
is.survey_df <- function(x) {
  inherits(x, "survey") | inherits(x, "survey_df")
}


#' @rdname survey_df
#' @export
print.survey_df <- function(x, ...) {
  NextMethod()
}

test_that("survey_df() constructs a survey from a data frame", {
  df <- data.frame(
    rowid = 1:6,
    observations = runif(6)
  )

  survey <- survey_df(
    x = df,
    identifier = "example",
    filename = "no_file"
  )

  expect_s3_class(survey, "survey_df")
  expect_true(is.data.frame(survey))
  expect_true(is.numeric(survey$observations))
  expect_equal(attr(survey, "id"), "example")
  expect_equal(attr(survey, "filename"), "no_file")
})

test_that("survey_df() creates default dataset metadata", {
  survey <- survey_df(
    x = data.frame(a = 1:3),
    identifier = "example",
    filename = "file.csv"
  )

  expect_true(dataset::is.dataset_df(survey))

  bib <- attr(survey, "dataset_bibentry")

  expect_equal(bib$Title, "Untitled Survey")

  survey_subject <- attr(survey, "subject")
  expect_equal(survey_subject[[1]]$term, "Surveys")
  expect_equal(
    survey_subject[[1]]$valueURI,
    "http://id.loc.gov/authorities/subjects/sh85130875"
  )
})

test_that("is.survey_df() identifies survey_df objects", {
  survey <- survey_df(
    x = data.frame(x = 1),
    identifier = "example",
    filename = "file.csv"
  )

  expect_true(is.survey_df(survey))
  expect_false(is.survey_df(data.frame(x = 1)))
})


test_that("print.survey_df() works without error", {
  survey <- survey_df(
    x = data.frame(x = 1),
    title = "Example Survey",
    identifier = "example",
    filename = "file.csv"
  )

  expect_invisible(print(survey))
})

#' @title Create a survey data frame
#'
#' @description Store the data of a survey in a tibble (data frame) with a unique
#' survey identifier, import filename, and optional document object identifier.
#'
#' @details Whilst you can create a survey object with this helper function, it is most likely that
#' you will receive it with an importing function, i.e.
#' \code{\link{read_rds}}, \code{\link{read_spss}} \code{\link{read_dta}}, \code{\link{read_csv}} or
#' their common wrapper \code{\link{read_survey}}.
#'
#' @param object A tibble or data frame that contains the survey data.
#' @param id A mandatory identifier for the survey.
#' @param filename The import file name.
#' @param doi Optional document object identifier (doi), can be omitted.
#' @return A tibble with \code{id},  \code{filename},  \code{doi}
#' metadata information.
#' @family importing functions
#' @examples
#' example_survey <- survey(
#'   object = data.frame(
#'     rowid = 1:6,
#'     observations = runif(6)
#'   ),
#'   id = "example",
#'   filename = "no_file"
#' )
#' @export

survey <- function(object = data.frame(),
                   id = "survey_id",
                   filename = NULL,
                   doi = NULL) {
  new_survey(object, id, filename, doi)
}

## Validator for survey class, not to be exported ---------------------

#' @keywords internal
#' @importFrom assertthat assert_that
validate_survey <- function(object, id, filename, doi) {
  if (!is.null(doi)) {
    if (!is.character(doi) || length(doi) != 1) {
      stop("The 'doi' must be a character of length 1L (or NULL)")
    }
  }

  if (!is.null(filename)) {
    if (!is.character(filename) || length(filename) != 1) {
      stop("The 'filename' must be a character of length 1L (or NULL)")
    }
  }

  if (!is.null(id)) {
    if (!is.character(id) || length(id) != 1) {
      stop("The 'id' must be a character of length 1L (or NULL)")
    }
  }

  assertthat::assert_that(
    inherits(object, "data.frame"),
    msg = "df in validate_survey(df) must be a data.frame like object."
  )
}

## Constructor for new survey, not to be exported ---------------------
#' @keywords internal
#' @importFrom tibble tibble
new_survey <- function(object = tibble::tibble(),
                       id = character(1),
                       filename = character(1),
                       doi = character(1)) {
  validate_survey(
    object = object,
    id = id,
    filename = filename,
    doi = doi
  )

  structure(object,
    id = id,
    filename = filename,
    doi = doi,
    class = c("survey", class(object))
  )
}


#' @rdname survey
#' @export
is.survey <- function(object) {
  inherits(object, "survey") | inherits(object, "survey_df")
}

#' @rdname survey_df
is.survey_df <- function(object) {
  inherits(object, "survey") | inherits(object, "survey_df")
}

#' @rdname survey
#' @param ... Arguments passed to summary method.
#' @importFrom tibble as_tibble
#' @importFrom dplyr bind_cols mutate_all all_of
#' @importFrom purrr set_names
#' @export
summary.survey <- function(object, ...) {
  if (!is.null(attr(object, "label"))) {
    cat(attr(object, "label"))
  }

  print(summary(tibble::as_tibble(object)))

  not_yet_implement <- function() {
    labelled_types <- names(object)[vapply(object, function(x) inherits(x, "haven_labelled"), logical(1))]
    not_labelled_types <- object[!names(object) %in% labelled_types]

    labelled_cols <- object %>%
      select(all_of(labelled_types))

    numeric_cols <- labelled_cols %>%
      set_names(paste0(names(labelled_cols), "_numeric")) %>%
      mutate_all(as_numeric)

    factor_cols <- labelled_cols %>%
      set_names(paste0(names(labelled_cols), "_factor")) %>%
      mutate_all(as_factor)

    object %>%
      select(-all_of(labelled_types)) %>%
      bind_cols(numeric_cols) %>%
      bind_cols(factor_cols) %>%
      as_tibble() %>%
      summary() %>%
      print()
  }
}

test_that("correct casting", {
  example_survey <- survey(
    object = data.frame(
      rowid = 1:6,
      observations = runif(6)
    ),
    id = "example",
    filename = "no_file"
  )
  expect_s3_class(example_survey, "survey")
  expect_true(is.survey(example_survey))
  expect_equal(attr(example_survey, "filename"), "no_file")
})

test_that("read_spss_survey() with attributes work", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav"),
    id = "my_iris"
  )
  expect_equal(attr(read_example, "id"), "my_iris")
  expect_equal(attr(read_example, "filename"), "iris1.sav")
  expect_equal(attr(read_example, "doi"), NULL)
})

test_that("exception handling works", {
  expect_error(read_spss(file.path(examples_dir, "not_iris.sav")))
})

test_that("read_spss() preserves core attributes", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav"),
    id = "my_iris"
  )

  expect_equal(
    attr(read_example, "id"),
    "my_iris"
  )

  expect_equal(
    attr(read_example, "filename"),
    "iris1.sav"
  )

  expect_null(
    attr(read_example, "doi")
  )

  expect_true(
    is.survey(read_example)
  )
})

test_that("read_spss() creates rowid", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav"),
    id = "my_iris"
  )

  expect_true(
    "rowid" %in% names(read_example)
  )

  expect_true(
    all(grepl("^my_iris_", read_example$rowid))
  )
})

test_that("read_spss() preserves variable labels", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav")
  )

  expect_true(
    !is.null(labelled::var_label(read_example$Sepal.Length))
  )
})

test_that("read_spss() stores provenance metadata", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav")
  )

  expect_true(
    attr(read_example, "object_size") > 0
  )

  expect_true(
    attr(read_example, "source_file_size") > 0
  )
})

test_that("read_spss() handles missing files", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  expect_error(
    read_spss(
      file.path(examples_dir, "not_iris.sav")
    )
  )
})

test_that("read_spss() returns empty survey on read failure", {
  tmp <- tempfile(fileext = ".sav")

  writeLines(
    "this is not an SPSS file",
    con = tmp
  )

  expect_warning(
    result <- read_spss(tmp)
  )

  expect_true(
    is.survey(result)
  )

  expect_equal(
    ncol(result),
    0
  )
})

test_that("read_spss() preserves column names", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  read_example <- read_spss(
    file = file.path(examples_dir, "iris1.sav")
  )

  expect_true(
    all(
      c(
        "rowid",
        "Sepal.Length",
        "Sepal.Width",
        "Petal.Length",
        "Petal.Width",
        "Species"
      ) %in% names(read_example)
    )
  )
})

## Displaying methods -----------------------------------------
#' @keywords internal
get_labeltext <- function(x, prefix = ": ") {
  label <- attr(x, "label", exact = TRUE)
  if (!is.null(label)) {
    paste0(prefix, label)
  }
}

#' @keywords internal
print_attributes <- function(x, full = TRUE) {
  na_values <- attr(x, "na_values")
  if (!is.null(na_values)) {
    cli::cat_line("Missing values: ", paste(na_values, collapse = ", "))
  }
  na_range <- attr(x, "na_range")
  if (!is.null(na_range)) {
    cli::cat_line("Missing range:  [", paste(na_range, collapse = ", "), "]")
  }

  if (full == FALSE) invisible(x)
  ## full printing goes on below ---------------------

  history_attributes <- names(attributes(x))
  history_attributes <- history_attributes[
    !history_attributes %in% c(
      "label", "labels",
      "na_values", "na_range", "class", "id"
    )
  ]

  if (length(history_attributes) > 0) {
    last_attribute <- history_attributes[length(history_attributes)]
    history_attributes <- c(history_attributes[1:3], "...", last_attribute)
    history_attributes <- paste(history_attributes, collapse = ", ")
    history_attributes <- gsub("\\,\\s\\.\\.\\.", " [...]", history_attributes)
  }

  cli::cat_line(paste0(
    "See all attributes ",
    history_attributes,
    " with attributes(",
    deparse(substitute(x)),
    ")"
  ))
}


#' @export
#' @importFrom cli cat_line
obj_print_header.retroharmonize_labelled_spss_survey <- function(x, ...) {
  cli::cat_line(
    "<",
    vec_ptype_full(x),
    "[",
    vctrs::vec_size(x),
    "]>", get_labeltext(x)
  )
  invisible(x)
}

#' @export
obj_print_footer.retroharmonize_labelled_spss_survey <- function(x, ...) {
  print_attributes(x)
  invisible(x)
}


#' @rdname labelled_spss_survey
#' @importFrom cli cat_line
#' @importFrom utils head
#' @export
print.retroharmonize_labelled_spss_survey <- function(x, ...) {
  cli::cat_line(
    "<", vec_ptype_full(x),
    "[", vec_size(x), "]>", get_labeltext(x)
  )
  cat(head(vctrs::vec_data(x), 20))
  cat("\n")
  print_attributes(x)
  invisible(x)
}


#' @export
vec_ptype_full.retroharmonize_labelled_spss_survey <- function(x, ...) {
  paste0(
    "labelled_spss_survey<",
    vctrs::vec_ptype_full(vctrs::vec_data(x)),
    ">"
  )
}


#' @export
vec_ptype_abbr.retroharmonize_labelled_spss_survey <- function(x, ...) {
  if (vctrs::vec_ptype_full(vctrs::vec_data(x)) == "character") {
    "retroh_chr"
  } else if (vec_ptype_full(vctrs::vec_data(x)) == "integer") {
    "retroh_int"
  } else if (vec_ptype_full(vctrs::vec_data(x)) == "double") {
    "retroh_dbl"
  } else {
    "retroh"
  }
}

#' @title Collect labels from metadata file
#'
#' @param metadata A metadata data frame created by
#' \code{\link{metadata_create}}.
#' @return The unique valid labels or the user-defined missing
#' labels found in all the files analyzed in \code{metadata}.
#' @family harmonization functions
#' @examples
#' test_survey <- retroharmonize::read_rds(
#'   file = system.file("examples", "ZA7576.rds",
#'     package = "retroharmonize"
#'   ),
#'   id = "test"
#' )
#' example_metadata <- metadata_create(test_survey)
#'
#' collect_val_labels(metadata = example_metadata)
#' collect_na_labels(metadata = example_metadata)
#' @export

collect_val_labels <- function(metadata) {
  . <- NULL

  valid_labels <- lapply(metadata$valid_labels, unlist) %>%
    sapply(., names)
  unique(as.character(unlist(valid_labels)))
}

#' @rdname collect_val_labels
#' @export
collect_na_labels <- function(metadata) {
  . <- NULL

  na_labels <- lapply(metadata$na_labels, unlist) %>%
    sapply(., names)
  unique(as.character(unlist(na_labels)))
}

test_that("val label collection works", {
  test_survey <- retroharmonize::read_rds(
    file = system.file("examples", "ZA7576.rds",
      package = "retroharmonize"
    ),
    id = "test"
  )

  example_metadata <- metadata_create(survey_list = test_survey)

  trust_media_metadata <- example_metadata[27, ]
  expect_equal(
    collect_val_labels(metadata = trust_media_metadata),
    c("Tend to trust", "Tend not to trust", "DK")
  )
  expect_equal(
    collect_na_labels(metadata = trust_media_metadata),
    c("Inap. (not CY-TCC in isocntry)")
  )
})

#' Read survey files into memory or save as `.rds`
#'
#' Import one or more survey files into R using a consistent survey
#' import workflow. The function supports SPSS (`.sav`, `.por`),
#' Stata (`.dta`), R (`.rds`), and CSV files.
#'
#' Use [read_survey()] to import a single survey file and
#' `read_surveys()` to import multiple files in a loop.
#'
#' When `export_path` is `NULL`, imported surveys are returned as
#' a list in memory. When `export_path` is a valid directory,
#' imported surveys are saved as `.rds` files with
#' [base::saveRDS()].
#'
#' Files that cannot be imported are skipped gracefully. A message
#' is printed and `NULL` is returned for the affected file.
#'
#' @param survey_paths A character vector containing full or relative
#'   paths to survey files.
#'
#' @param .f Import function to use. When `NULL`, the appropriate
#'   import function is selected automatically from the file
#'   extension.
#'
#'   Supported formats are:
#'
#'   \describe{
#'     \item{`.sav`, `.por`}{[read_spss()]}
#'     \item{`.dta`}{[read_dta()]}
#'     \item{`.rds`}{[read_rds()]}
#'     \item{`.csv`}{[read_csv()]}
#'   }
#'
#' @param export_path Optional path where imported surveys should
#'   be saved as `.rds` files. Defaults to `NULL`.
#'
#' @param ids Optional survey identifiers.
#'
#' @param dois Optional DOI identifiers for the imported surveys.
#'
#' @param ... Additional arguments passed to the import function.
#'
#' @return
#'
#' If `export_path = NULL`, a list of imported survey objects.
#'
#' If `export_path` is provided, a character vector containing
#' exported `.rds` file names.
#'
#' Imported surveys are returned as data frame-like
#' [survey()] objects with metadata attributes retained for
#' reproducible workflows.
#'
#' @examples
#' file1 <- system.file(
#'   "examples",
#'   "ZA7576.rds",
#'   package = "retroharmonize"
#' )
#'
#' file2 <- system.file(
#'   "examples",
#'   "ZA5913.rds",
#'   package = "retroharmonize"
#' )
#'
#' surveys <- read_surveys(
#'   c(file1, file2),
#'   .f = "read_rds"
#' )
#'
#' @importFrom assertthat assert_that
#' @importFrom fs path_file
#' @importFrom purrr safely
#'
#' @family import functions
#' @seealso [read_survey()], [survey()]
#' @export

read_surveys <- function(survey_paths,
                         .f = NULL,
                         export_path = NULL,
                         ids = NULL,
                         dois = NULL,
                         ...) {
  arguments <- list(...)

  import_file_vector <- survey_paths
  existing_files <- which(file.exists(import_file_vector))
  not_existing_files <- which(!file.exists(import_file_vector))

  if (length(existing_files) == 0) {
    stop("None of the files on read_surveys(survey_paths=...) exist.")
  }

  if (length(not_existing_files) > 0) {
    missing_files <- paste(import_file_vector[not_existing_files], collapse = ";\n")
    warning("Some files on 'survey_paths' do not exist:\n", missing_files)
  }

  import_file_vector <- import_file_vector[existing_files]

  if (!is.null(ids)) {
    ids <- ids[existing_files]
  } else {
    ids <- rep(NULL, length(import_file_vector))
  }

  if (!is.null(dois)) {
    dois <- dois[existing_files]
  } else {
    dois <- rep(NULL, length(import_file_vector))
  }


  return_survey_list <- lapply(
    seq_along(import_file_vector),
    function(x) {
      read_survey(
        file_path = import_file_vector[x],
        .f = .f,
        export_path = export_path,
        doi = dois[x],
        id = ids[x],
        ... = ...
      )
    }
  )

  return_survey_list
}

#' @rdname read_surveys
#' @importFrom fs file_exists dir_exists path_ext_remove
#' @importFrom glue glue
#' @importFrom assertthat assert_that
#' @importFrom purrr safely
#' @keywords internal
read_survey <- function(file_path,
                        .f = NULL,
                        export_path = NULL,
                        doi = NULL,
                        id = NULL,
                        ...) {
  arguments <- list(...)

  assert_that(fs::file_exists(file_path),
    msg = glue::glue("The file {file_path} does not exist.")
  )

  if (is.null(.f)) .f <- find_import_function(file_path) ## See definition below

  if (.f == "read_rds") {
    res <- safely(read_rds)(file_path)
  } else if (.f == "read_spss") {
    res <- safely(read_spss)(file_path)
  } else if (.f == "read_dta") {
    res <- safely(read_dta)(file_path)
  } else if (.f == "read_csv") {
    res <- safely(read_csv)(file = file_path, doi = doi, id = id, ...)
  }

  if (is.null(res$error)) {
    # No problem reading and should be saved --------------------------------------
    if (!is.null(export_path)) {
      if (fs::dir_exists(export_path)) {
        # Returned survey ----------------------------------------------------------------
        imported_survey <- res$result
        source_file_name <- attr(imported_survey, "filename")

        # Saving location exists, return file name after saving --------------------
        new_file_name <- paste0(fs::path_ext_remove(source_file_name), ".rds")
        saveRDS(res$result,
          file = file.path(export_path, new_file_name),
          version = 2
        )
        return(new_file_name)
      } else {
        # Exception: cannot be exported, returning to  memory -------------------------------
        warning("Cannot save to ", export_path, ", returning to memory instead.")
        return(res$result)
      }
    }
    return(res$result)
  }

  if (!is.null(res$error)) {
    # There was a problem reading -------------------------------------------------
    # Even though the file exists (checked in the beginning of the function) ------
    message(res$error)
    message("This is an error in read_survey(", file_path, ", ", .f, ")")
    message("Returning NULL for this file.")
    return(NULL)
  }
}


#' @title Find import function by file extension
#' @description This is an internal utility to select the appropriate importing function.
#' @return The name of the function that should read \code{file_path} based on the file
#' extension.
#' @importFrom fs path_ext path_ext_remove path_file
#' @importFrom glue glue
#' @inheritParams read_surveys
#' @keywords internal
find_import_function <- function(file_path) {
  survey_file_ext <- fs::path_ext(file_path)

  if (survey_file_ext %in% c("sav", "por")) {
    "read_spss"
  } else if (survey_file_ext == "rds") {
    "read_rds"
  } else if (survey_file_ext == "dta") {
    "read_dta"
  } else if (survey_file_ext == "csv") {
    "read_csv"
  } else {
    stop(glue("No adequate importing function was found for {file_path}."))
  }
}

test_that("read_surveys() reads the correct file", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  my_rds_files <- file.path(examples_dir, dir(examples_dir)[grepl(
    ".rds",
    dir(examples_dir)
  )])
  example_surveys <- read_surveys(survey_paths = my_rds_files)
  expect_equal(
    attr(read_survey(file_path = my_rds_files[1]), "filename"),
    fs::path_file(my_rds_files[1])
  )
})

test_that("read_surveys() error if file does not exists", {
  expect_error(read_surveys(tempfile(), .f = "read_csv", export_path = NULL))
})


test_that("read_surveys() reads all files", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  my_rds_files <- file.path(examples_dir, dir(examples_dir)[grepl(
    ".rds",
    dir(examples_dir)
  )])
  example_surveys <- read_surveys(survey_paths = my_rds_files)
  expect_equal(length(example_surveys), 3)
})


test_that("find_import_function() detects file formats", {
  expect_equal(
    find_import_function("test.sav"),
    "read_spss"
  )

  expect_equal(
    find_import_function("test.por"),
    "read_spss"
  )

  expect_equal(
    find_import_function("test.dta"),
    "read_dta"
  )

  expect_equal(
    find_import_function("test.rds"),
    "read_rds"
  )

  expect_equal(
    find_import_function("test.csv"),
    "read_csv"
  )
})


test_that("find_import_function() errors on unsupported formats", {
  expect_error(
    find_import_function("test.xlsx")
  )
})


test_that("read_surveys() skips missing files with warning", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  existing_file <- file.path(
    examples_dir,
    "ZA7576.rds"
  )

  missing_file <- tempfile(fileext = ".rds")

  expect_warning({
    surveys <- read_surveys(
      c(existing_file, missing_file)
    )
  })

  expect_equal(length(surveys), 1)
})


test_that("read_surveys() exports surveys to rds", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  my_rds_files <- file.path(
    examples_dir,
    dir(examples_dir)[grepl(
      ".rds",
      dir(examples_dir)
    )]
  )

  export_dir <- tempdir()

  exported <- read_surveys(
    survey_paths = my_rds_files,
    export_path = export_dir
  )

  expect_true(
    all(file.exists(
      file.path(export_dir, exported)
    ))
  )
})


test_that("read_survey() retains filename metadata", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  file1 <- file.path(
    examples_dir,
    "ZA7576.rds"
  )

  survey <- read_survey(file1)

  expect_equal(
    attr(survey, "filename"),
    "ZA7576.rds"
  )
})


test_that("read_surveys() propagates ids and dois", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  file1 <- file.path(
    examples_dir,
    "ZA7576.rds"
  )

  surveys <- read_surveys(
    survey_paths = file1,
    ids = "ZA7576",
    dois = "10.4232/example"
  )

  expect_equal(
    attr(surveys[[1]], "id"),
    "ZA7576"
  )
})

# See also test-read_surveys


test_that("read_csv works", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  test_csv_file <- tempfile()
  test_read <- read_rds(file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )
  write.csv(
    x = test_read,
    file = test_csv_file, row.names = FALSE
  )

  re_read <- read_csv(
    file = test_csv_file,
    id = "ZA7576",
    doi = "test_doi"
  )
  expect_equal(attr(re_read, "doi"), "test_doi")
  expect_equal(attr(re_read, "id"), "ZA7576")
  expect_true(is.survey(re_read))
})

test_that("read_survey(...) passes on ...", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  test_csv_file <- tempfile()
  test_read <- read_rds(file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )
  write.csv(x = test_read, file = test_csv_file, row.names = FALSE)
  re_read_2 <- read_survey(
    file_path = test_csv_file,
    .f = "read_csv",
    id = "ZA7576",
    doi = "test_doi"
  )
  expect_equal(attr(re_read_2, "doi"), "test_doi")
  expect_equal(attr(re_read_2, "id"), "ZA7576")
  expect_true(is.survey(re_read_2))
})


test_that("read_surveys(...) passes on ...", {
  examples_dir <- system.file("examples", package = "retroharmonize")
  test_csv_file <- tempfile()
  test_read <- read_rds(file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576",
    doi = "test_doi"
  )
  write.csv(x = test_read, file = test_csv_file, row.names = FALSE)
  re_read_3 <- read_surveys(
    survey_paths = test_csv_file,
    .f = "read_csv",
    ids = "ZA7576",
    dois = "test_doi"
  )[[1]]
  expect_equal(attr(re_read_3, "doi"), "test_doi")
  expect_equal(attr(re_read_3, "id"), "ZA7576")
  expect_true(is.survey(re_read_3))
})


test_that("read_csv preserves rowid", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  test_csv_file <- tempfile(fileext = ".csv")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds"),
    id = "ZA7576"
  )

  write.csv(
    x = test_read,
    file = test_csv_file,
    row.names = FALSE
  )

  re_read <- read_csv(
    file = test_csv_file,
    id = "ZA7576"
  )

  expect_true("rowid" %in% names(re_read))
})


test_that("read_csv errors without rowid column", {
  test_csv_file <- tempfile(fileext = ".csv")

  test_data <- data.frame(
    x = 1:3,
    y = letters[1:3]
  )

  write.csv(
    test_data,
    test_csv_file,
    row.names = FALSE
  )

  expect_error(
    read_csv(test_csv_file)
  )
})


test_that("read_csv stores object size metadata", {
  examples_dir <- system.file(
    "examples",
    package = "retroharmonize"
  )

  test_csv_file <- tempfile(fileext = ".csv")

  test_read <- read_rds(
    file.path(examples_dir, "ZA7576.rds")
  )

  write.csv(
    x = test_read,
    file = test_csv_file,
    row.names = FALSE
  )

  re_read <- read_csv(test_csv_file)

  expect_true(
    !is.null(attr(re_read, "object_size"))
  )

  expect_true(
    !is.null(attr(re_read, "source_file_size"))
  )
})

#' Subset and optionally harmonize surveys
#'
#' @description
#' Subset one or more surveys by retaining a specified set of variables.
#' Subsetting can be performed either on surveys already loaded in memory
#' or directly from survey files on disk.
#'
#' If a crosswalk table is supplied, variables are selected based on the
#' variables listed for each survey in the crosswalk, and variable names
#' can optionally be harmonized using `var_name_target`.
#'
#' This function replaces the deprecated helpers
#' [subset_waves()] and [subset_save_surveys()].
#'
#' @details
#' The function supports multiple workflows:
#'
#' * **In-memory subsetting** using `survey_list`
#' * **File-based subsetting** using `survey_paths` or `import_path`
#' * **Crosswalk-driven subsetting**, where variables are selected
#'   per survey using a crosswalk table created by
#'   [crosswalk_table_create()]
#'
#' If `export_path` is provided, subsetted surveys are written to disk
#' as `.rds` files. Otherwise, subsetted surveys are returned in memory.
#'
#' @param survey_list A list of survey objects created by
#'   [read_surveys()]. If `NULL`, surveys are read from disk.
#'
#' @param survey_paths A character vector of full file paths to survey files.
#'   Used when `survey_list` is `NULL`.
#'
#' @param rowid Name of the unique observation identifier column.
#'   Defaults to `"rowid"`.
#'
#' @param subset_name Character string appended to filenames of
#'   subsetted surveys. Defaults to `"subset"`.
#'
#' @param subset_vars Character vector of variable names to retain.
#'   If `NULL`, all variables are retained.
#'
#' @param crosswalk_table Optional crosswalk table created with
#'   [crosswalk_table_create()]. If supplied, variables are selected
#'   per survey based on `var_name_orig`, and variable names may be
#'   harmonized using `var_name_target`.
#'
#' @param import_path Optional directory containing survey files.
#'   Used to resolve filenames when subsetting from disk.
#'
#' @param export_path Optional directory where subsetted surveys
#'   are saved as `.rds` files. If `NULL`, surveys are returned in memory.
#'
#' @return
#' Either:
#' * a list of subsetted survey objects (if `export_path = NULL`), or
#' * a character vector of filenames written to `export_path`.
#'
#' @family subsetting functions
#'
#' @importFrom dplyr select any_of
#'
#' @seealso
#' [crosswalk_table_create()],
#' [harmonize_survey_variables()],
#' [read_surveys()]
#'
#' @examples
#' examples_dir <- system.file("examples", package = "retroharmonize")
#' survey_files <- dir(examples_dir, pattern = "\\.rds$")
#'
#' surveys <- read_surveys(
#'   file.path(examples_dir, survey_files),
#'   export_path = NULL
#' )
#'
#' subset_surveys(
#'   survey_list = surveys,
#'   subset_vars = c("rowid", "isocntry", "qa10_1", "qa14_1"),
#'   subset_name = "example_subset"
#' )
#'
#' @export

subset_surveys <- function(survey_list,
                           survey_paths = NULL,
                           rowid = "rowid",
                           subset_name = "subset",
                           subset_vars = NULL,
                           crosswalk_table = NULL,
                           import_path = NULL,
                           export_path = NULL) {
  if (!is.null(survey_list)) {
    validate_survey_list(survey_list)
    subset_from_files <- FALSE
    files_to_subset <- NULL

    if (!"list" %in% class(survey_list) & is.survey(survey_list)) {
      survey_list <- list(i = survey_list)
      names(survey_list)[1] <- attr(survey_list[[1]], "id")
    }
  } else {
    subset_from_files <- TRUE
  }
  if (!is.null(survey_paths)) {
    validate_survey_files(survey_paths)
  }

  if (!is.null(crosswalk_table)) {
    is.crosswalk_table(crosswalk_table)
    if (!is.null(survey_paths)) {
      files_to_subset <- survey_paths[fs::path_file(survey_paths) %in% fs::path_file(crosswalk_table$filename)]
    } else if (!is.null(import_path)) {
      files_to_subset <- dir(import_path)[dir(import_path) %in% fs::path_file(crosswalk_table$filename)]
    } else {
      files_to_subset <- unique(crosswalk_table$filename)
    }
  }
  if (!is.null(import_path)) {
    assert_that(fs::dir_exists(import_path) == TRUE,
      msg = " in subset_surveys: the 'import_path' is not a valid path to a directory."
    )

    files_to_subset <- dir(import_path)[dir(import_path) %in% fs::path_file(crosswalk_table$filename)]
    files_to_subset <- file.path(import_path, files_to_subset)
  }

  if (!is.null(export_path)) {
    assert_that(fs::dir_exists(export_path) == TRUE,
      msg = " in subset_surveys: the 'export_path' is not a valid path to a directory."
    )
  }

  if (subset_from_files) {
    ## Subsetting from files ------------------------------------------------------------------
    if (length(files_to_subset) == 0) {
      message("subset_surveys(): No files to subset.")
      return(NULL)
    }

    ### Subsetting with crosswalk table ----------------------
    if (!is.null(crosswalk_table)) {
      get_survey <- function(x) {
        this_path <- files_to_subset[x]

        this_id <- crosswalk_table %>%
          filter(
            filename == fs::path_file(this_path)
          ) %>%
          dplyr::distinct(id) %>%
          unlist() %>%
          as.character()

        subset_vars <- crosswalk_table %>%
          filter(id == this_id) %>%
          select(var_name_orig) %>%
          unlist() %>%
          as.character() %>%
          unique()

        subset_survey_file(
          file_path = this_path,
          subset_vars = subset_vars,
          id = this_id,
          export_path = export_path
        )
      }

      if (!is.null(export_path)) {
        saved_file_names <- lapply(
          seq_along(files_to_subset),
          function(x) get_survey(x)
        )
        return(unlist(saved_file_names))
      }
      return_list <- lapply(
        seq_along(files_to_subset),
        function(x) get_survey(x)
      )
    } else {
      ## Subsetting without crosswalk table -----------------

      get_survey_no_ctable <- function(x) {
        this_path <- files_to_subset[x]

        ### issue
        this_id <- NULL

        subset_survey_file(
          file_path = this_path,
          subset_vars = subset_vars,
          id = this_id,
          export_path = export_path
        )
      }

      if (!is.null(export_path)) {
        saved_file_names <- lapply(seq_along(files_to_subset), function(x) get_survey_no_ctable(x))
        return(unlist(saved_file_names))
      }
      return_list <- lapply(seq_along(files_to_subset), function(x) get_survey_no_ctable(x))
    }
  } else {
    ### Subsetting from memory, with cross_table ---------------------------------------------

    if (!is.null(crosswalk_table)) {
      available_surveys <- vapply(
        survey_list,
        function(x) attr(x, "id"),
        character(1)
      )

      surveys_in_ctable <- unique(
        crosswalk_table$id
      )[unique(crosswalk_table$id) %in%
        available_surveys]

      s <- which(surveys_in_ctable == available_surveys)

      get_survey_memory <- function(x) {
        this_survey <- survey_list[[x]]

        subset_vars <- crosswalk_table %>%
          filter(id == attr(this_survey, "id")) %>%
          select(var_name_orig) %>%
          unlist() %>%
          as.character() %>%
          unique()

        subset_survey_memory(
          this_survey = this_survey,
          subset_vars = subset_vars,
          subset_name = subset_name,
          export_path = export_path
        )
      }

      if (!is.null(export_path)) {
        saved_file_names <- lapply(
          seq_along(s),
          function(x) get_survey_memory(x)
        )
        return(unlist(saved_file_names))
      } else {
        return_list <- lapply(
          seq_along(s),
          function(x) {
            get_survey_memory(x)
          }
        )
      }
    } else {
      ### Subsetting from memory, without cross_table ------------------------------
      get_survey_no_ctable_memory <- function(x) {
        subset_survey_memory(
          this_survey = survey_list[[x]],
          subset_vars = subset_vars,
          subset_name = subset_name,
          export_path = export_path
        )
      }

      if (!is.null(export_path)) {
        saved_file_names <- lapply(
          seq_along(survey_list),
          function(x) get_survey_no_ctable_memory(x)
        )
        return(unlist(saved_file_names))
      } else {
        return_list <- lapply(
          seq_along(survey_list),
          function(x) get_survey_no_ctable_memory(x)
        )
      }
    }
  }
  return_list
}

#' @rdname subset_surveys
#' @param waves A list of surveys imported with [read_surveys()].
#' @export
subset_waves <- function(waves, subset_vars = NULL) {
  .Deprecated(new = "subset_surveys", msg = "subset_waves is deprecated, use subset_surveys instead.")
  subset_surveys(survey_list = waves, subset_vars = subset_vars)
}


#' @rdname subset_surveys
#' @export
subset_save_surveys <- function(crosswalk_table,
                                subset_name = "subset",
                                survey_list = NULL,
                                subset_vars = NULL,
                                survey_paths = NULL,
                                import_path = NULL,
                                export_path = NULL) {
  .Deprecated(
    new = "subset_surveys",
    msg = "subset_save_surveys is deprecated, use subset_surveys instead."
  )
  subset_surveys(
    crosswalk_table = crosswalk_table,
    subset_name = subset_name,
    survey_list = survey_list,
    subset_vars = subset_vars,
    import_path = import_path,
    export_path = export_path
  )
}

#' @title Subset surveys from files
#' @inheritParams subset_surveys
#' @param file_path A single survey files.
#' @keywords internal
subset_survey_file <- function(file_path,
                               subset_vars,
                               subset_name = "subset",
                               id = NULL,
                               export_path = NULL) {
  subset_vars <- unique(c("rowid", subset_vars))

  survey_file_ext <- fs::path_ext(file_path)
  if (is.null(id)) id <- fs::path_ext_remove(fs::path_file(file_path))

  if (survey_file_ext %in% c("sav", "por")) {
    this_survey <- read_spss(file_path, id = id)
  } else if (survey_file_ext == "rds") {
    this_survey <- read_rds(file = file_path, id = id)
  } else if (survey_file_ext == "dta") {
    this_survey <- read_dta(file = file_path, id = id)
  } else if (survey_file_ext == "csv") {
    this_survey <- read_csv(file = file_path, id = id)
  } else {
    return(NULL)
  }
  subset_survey_memory(this_survey,
    subset_vars = subset_vars,
    subset_name = subset_name,
    export_path = export_path
  )
}

#' @title Subset surveys in memory
#' @inheritParams subset_surveys
#' @importFrom tibble as_tibble
#' @importFrom fs path_ext path_ext_remove
#' @keywords internal
subset_survey_memory <- function(this_survey,
                                 subset_vars,
                                 subset_name = "subset",
                                 export_path = NULL) {
  subset_vars <- unique(c("rowid", subset_vars))

  subset_survey <- this_survey %>%
    select(any_of(subset_vars))

  attr(subset_survey, "subset_size") <- as.numeric(object.size(as_tibble(subset_survey)))

  this_file_name <- paste0(
    fs::path_ext_remove(attr(subset_survey, "filename")),
    "_", subset_name, ".",
    fs::path_ext(attr(subset_survey, "filename"))
  )

  attr(subset_survey, "filename") <- this_file_name

  if (!is.null(export_path)) {
    save_file_name <- paste0(fs::path_ext_remove(fs::path_file(this_file_name)), ".rds")
    message("Saving ", paste0(fs::path_ext_remove(fs::path_file(this_file_name)), ".rds"))
    saveRDS(
      object = subset_survey_memory(this_survey, subset_vars),
      file = file.path(export_path, save_file_name),
      version = 2
    )
    save_file_name
  } else {
    subset_survey
  }
}


#' @rdname subset_surveys
#' @importfrom dplyr add_count
#' @examples
#' test_survey <- read_rds(
#'   file = system.file("examples", "ZA7576.rds",
#'     package = "retroharmonize"
#'   )
#' )
#'
#' test_metadata <- metadata_create(test_survey)
#' test_metadata <- test_metadata[c(1, 7, 18), ]
#' ctable_2 <- crosswalk_table_create(test_metadata)
#' ctable_2$var_name_target <- ifelse(ctable$var_name_orig == "qa14_3",
#'   "trust_ecb",
#'   ctable$var_name_orig
#' )
#'
#' subset_save_surveys(
#'   crosswalk_table = ctable_2,
#'   subset_name = "tested",
#'   survey_list = test_survey,
#'   import_path = NULL
#' )
#' @export

#' @title Here
#'
#' @description A utility to make sure the system files of the package and other
#' files are always found, regardless if they are in an example or
#' vignette context.
#'
#' @details See \code{here::\link[here:here]{here}} for details.
#'
#' @name here
#' @keywords internal
#' @export
#' @importFrom here here
#' @examples
#' dir(here("inst", "examples"))
NULL

#' vctrs type and casting methods for labelled survey vectors
#'
#' These methods define how
#' \code{retroharmonize_labelled_spss_survey} objects interact with
#' base vectors and with each other in vctrs-based operations such as
#' concatenation, binding, and coercion.
#'
#' They ensure that labelled survey vectors:
#' \itemize{
#'   \item combine safely with numeric vectors,
#'   \item cast consistently to base types,
#'   \item error on incompatible coercions.
#' }
#'
#' These functions are part of the internal type system and are not
#' intended to be called directly by users.
#'
#' @name labelled_spss_survey_vctrs
#' @keywords internal
NULL


#' @keywords internal
vec_cast_named <- function(x, to, ...) {
  # identical to haven:::vec_cast_named()
  stats::setNames(vctrs::vec_cast(x, to, ...), names(x))
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_ptype2.retroharmonize_labelled_spss_survey.double <- function(x,
                                                                  y,
                                                                  ...) {
  double()
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_ptype2.double.retroharmonize_labelled_spss_survey <- function(x,
                                                                  y,
                                                                  ...) {
  double()
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_ptype2.integer.retroharmonize_labelled_spss_survey <- function(x,
                                                                   y,
                                                                   ...) {
  double()
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_ptype2.retroharmonize_labelled_spss_survey.integer <- function(x,
                                                                   y,
                                                                   ...) {
  double()
}


#' @rdname labelled_spss_survey_vctrs
#' @export
vec_cast.double.retroharmonize_labelled_spss_survey <- function(x,
                                                                to,
                                                                ...) {
  vctrs::vec_cast(vctrs::vec_data(x), to)
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_cast.integer.retroharmonize_labelled_spss_survey <- function(x,
                                                                 to,
                                                                 ...) {
  vctrs::vec_cast(vctrs::vec_data(x), to)
}

#' @rdname labelled_spss_survey_vctrs
#' @export
vec_cast.character.retroharmonize_labelled_spss_survey <- function(
  x,
  to,
  ...
) {
  if (is.character(x)) {
    vctrs::vec_cast(vctrs::vec_data(x), to, ...)
  } else {
    stop_incompatible_cast(x, to, ...)
  }
}


## Prototype --------------------------------------
# see registration.R
#' @export
vec_ptype2.retroharmonize_labelled_spss_survey.retroharmonize_labelled_spss_survey <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = ""
) {
  data_type <- vec_ptype2(vec_data(x),
    vctrs::vec_data(y),
    x_arg = x_arg,
    y_arg = y_arg
  )

  x_labels <- labelled::val_labels(x)
  y_labels <- labelled::val_labels(y)

  dots <- list2(...)

  if (is.null(dots)) {
    dots <- list(orig_name = "")

    same_in <- paste0(
      attr(x, "id"), " and ",
      attr(y, "id"), "."
    )
  } else {
    same_in <- paste0(
      attr(x, "id"), "$", dots$orig_name, " and ",
      attr(y, "id"), "$", dots$orig_name
    )
  }

  if (length(x_labels) == 0 || length(y_labels) == 0) {
    stop_incompatible_type(
      x, y,
      x_arg = dots$orig_name,
      y_arg = paste(names(y_labels), collapse = ", "),
      details = paste0(
        "Must be labelled in ", dots$orig_name
      )
    )
  }

  if (!setequal(x_labels, y_labels)) {
    stop_incompatible_type(
      x, y,
      x_arg = paste(names(x_labels), collapse = ", "),
      y_arg = paste(names(y_labels), collapse = ", "),
      details = paste0(
        "The labelled numeric values must be the same in ", same_in
      )
    )
  }


  if (!setequal(names(x_labels), names(y_labels))) {
    stop_incompatible_type(
      x, y,
      x_arg = paste(names(x_labels), collapse = ", "),
      y_arg = paste(names(y_labels), collapse = ", "),
      details = paste0(
        "The labels must be the same in ", same_in
      )
    )
  }

  if (!setequal(attr(x, "na_values"), attr(y, "na_values"))) {
    stop_incompatible_type(
      x, y,
      x_arg = paste(names(attr(x, "na_values")), collapse = ", "),
      y_arg = paste(names(attr(y, "na_values")), collapse = ", "),
      message = paste0(
        "The na_values attributes are not the same in ",
        same_in
      )
    )
  }

  if (!setequal(attr(x, "na_range"), attr(y, "na_range"))) {
    stop_incompatible_type(
      x, y,
      x_arg = paste(names(attr(x, "na_range")), collapse = ", "),
      y_arg = paste(names(attr(y, "na_range")), collapse = ", "),
      message = paste0(
        "The na_range attributes are not the same in ", same_in
      )
    )
  }

  x_labels <- vec_cast_named(attr(x, "labels"), data_type, x_arg = x_arg)
  y_labels <- vec_cast_named(attr(y, "labels"), data_type, x_arg = y_arg)

  x_label <- attr(x, "label")
  y_label <- attr(y, "label")

  x_id <- attr(x, "id")
  y_id <- attr(y, "id")

  x_attr_names <- names(attributes(x))
  y_attr_names <- names(attributes(y))

  x_orig_attr <- x_attr_names[which(x_attr_names == paste0(x_id, "_name"))]
  y_orig_attr <- y_attr_names[which(y_attr_names == paste0(y_id, "_name"))]

  x_orig_name <- as.character(attr(x, x_orig_attr[1]))
  y_orig_name <- as.character(attr(y, y_orig_attr[1]))

  name_orig <- paste(vec_c(x_orig_name, y_orig_name), collapse = ", ")

  x_na_values <- attr(x, "na_values")
  y_na_values <- attr(y, "na_values")

  x_na_range <- attr(x, "na_range")
  y_na_range <- attr(y, "na_range")

  label <- x_label

  if (!identical(x_label, y_label)) {
    # strip labels if not compatible
    if (is.null(x_label)) {
      label <- y_label
    }
  }

  id <- "multi-wave"

  s1 <- attributes(x)
  s2 <- attributes(y)

  c_vector <- new_labelled_spss_survey(
    x = vec_c(vctrs::vec_data(x), vctrs::vec_data(y)),
    labels = x_labels,
    label = label,
    id = id,
    na_values = x_na_values,
    na_range = x_na_range,
    name_orig = name_orig
  )

  for (x_attr in setdiff(names(s1), names(s2))) {
    # Copy the history of x to the new vector
    attr(c_vector, x_attr) <- attr(x, x_attr)
  }

  for (y_attr in setdiff(names(s2), names(s1))) {
    # Copy the history of y to the new vector
    attr(c_vector, y_attr) <- attr(y, y_attr)
  }

  c_vector
}

#' @export
vec_cast.retroharmonize_labelled_spss_survey.retroharmonize_labelled_spss_survey <- function(
  x,
  to, ...,
  x_arg = "",
  to_arg = ""
) {
  out_data <- vec_cast(vctrs::vec_data(x),
    vctrs::vec_data(to),
    x_arg = x_arg,
    to_arg = to_arg
  )
  x_labels <- labelled::val_labels(x)

  x_label <- attr(x, "label")
  to_label <- attr(to, "label")

  x_id <- attr(x, "id")
  to_id <- attr(to, "id")

  x_attr_names <- names(attributes(x))
  to_attr_names <- names(attributes(to))

  x_orig_attr <- x_attr_names[which(x_attr_names == paste0(x_id, "_name"))]
  to_orig_attr <- to_attr_names[which(to_attr_names == paste0(to_id, "_name"))]

  x_orig_name <- as.character(attr(x, x_orig_attr[1]))
  to_orig_name <- as.character(attr(to, to_orig_attr[1]))

  name_orig <- paste(vec_c(x_orig_name, to_orig_name), collapse = ", ")

  x_na_values <- attr(x, "na_values")
  to_na_values <- attr(to, "na_values")

  x_na_range <- attr(x, "na_range")
  to_na_range <- attr(to, "na_range")

  label <- x_label

  if (!identical(x_label, to_label)) {
    # strip labels if not compatible
    if (is.null(x_label)) {
      label <- to_label
    }
  }

  id <- paste(vec_c(x_id, to_id), collapse = ", ")

  out <- new_labelled_spss_survey(
    out_data,
    labels = x_labels,
    label = label,
    id = id,
    na_values = x_na_values,
    na_range = x_na_range,
    name_orig = name_orig
  )

  # do we lose tagged na values? from haven
  if (is.double(x) && !is.double(out)) {
    lossy <- haven::is_tagged_na(x)
    maybe_lossy_cast(out, x, to, lossy,
      x_arg = x_arg,
      to_arg = to_arg,
      details = "Only doubles can hold tagged na values."
    )
  }

  out
}

#' @title Concatenate haven_labelled_spss vectors
#'
#' @param x A haven_labelled_spss vector.
#' @param y A haven_labelled_spss vector.
#' @return A concatenated haven_labelled_spss vector. Returns an error
#' if the attributes do not match. Gives a warning when only the variable
#' label do not match.
#' @importFrom labelled labelled val_labels
#' @importFrom haven labelled_spss is.labelled
#' @examples
#' v1 <- labelled::labelled(
#'   c(3, 4, 4, 3, 8, 9),
#'   c(YES = 3, NO = 4, `WRONG LABEL` = 8, REFUSED = 9)
#' )
#' v2 <- labelled::labelled(
#'   c(4, 3, 3, 9),
#'   c(YES = 3, NO = 4, `WRONG LABEL` = 8, REFUSED = 9)
#' )
#' s1 <- haven::labelled_spss(
#'   x = unclass(v1), # remove labels from earlier defined
#'   labels = labelled::val_labels(v1), # use the labels from earlier defined
#'   na_values = NULL,
#'   na_range = 8:9,
#'   label = "Variable Example"
#' )
#'
#' s2 <- haven::labelled_spss(
#'   x = unclass(v2), # remove labels from earlier defined
#'   labels = labelled::val_labels(v2), # use the labels from earlier defined
#'   na_values = NULL,
#'   na_range = 8:9,
#'   label = "Variable Example"
#' )
#' concatenate(s1, s2)
#' @importFrom haven labelled_spss labelled
#' @family joining functions
#' @export

concatenate <- function(x, y) {
  validate_concatenate(x, y)

  label_x <- attr(x, "label")
  label_y <- attr(y, "label")

  if (xor(is.null(label_x), is.null(label_y))) {
    if (is.null(label_x)) {
      attr(x, "label") <- label_y

      warning(
        "The variable label of y <",
        label_y,
        "> will be used as variable label."
      )
    } else {
      warning(
        "The variable label of x <",
        label_x,
        "> will be used as variable label."
      )
    }
  } else if (!isTRUE(all.equal(label_x, label_y))) {
    warning(
      "The variable labels are not the same, <",
      label_x,
      "> of x will be used."
    )
  }

  joining_attributes(x, y)
}

#' @keywords internal
validate_concatenate <- function(x, y) {
  if (!all(c(haven::is.labelled(x), haven::is.labelled(y)))) {
    stop("Both arguments must be labelled")
  }

  if (sum(is.labelled_spss(x), is.labelled_spss(y)) == 1) {
    if (!is.labelled_spss(x)) {
      x_na_values <- labelled::na_values(y)
      y_labels <- labelled::val_labels(y)

      x <- convert_to_labelled_spss(x, y_labels[y_labels %in% x_na_values])
    }
  }

  if (is.null(attr(x, "na_range"))) {
    # na_range consistency check
    if (!is.null(attr(y, "na_range"))) {
      stop("The first (x) na_range is NULL, the second (y) is not.")
    }
  } else if (is.null(attr(y, "na_range"))) {
    stop("The second (y) na_range is NULL, the first (x) is not.")
  } else {
    if (any(attr(x, "na_range") != attr(y, "na_range"))) {
      stop("The na_range attribute must be equal.")
    }
  }

  if (is.null(attr(x, "na_values"))) {
    if (!is.null(attr(y, "na_values"))) {
      stop("The first (x) na_values is NULL, the second (y) is not.")
    }
  } else if (is.null(attr(y, "na_values"))) {
    stop("The second (y) na_values is NULL, the first (x) is not.")
  } else {
    if (any(attr(x, "na_values") != attr(y, "na_values"))) {
      stop("The na_values attribute must be equal.")
    }
  }

  if (!haven::is.labelled(x) && haven::is.labelled(y)) {
    stop("Both x and y must be labelled.")
  }

  if (!all(class(x) == class(y))) {
    stop(
      "class(x)=",
      paste(class(x), collapse = ","),
      "\nbut class(y)=", paste(class(y), collapse = ",")
    )
  }
}

#' @keywords internal
joining_attributes <- function(x, y) {
  s1 <- attributes(x)
  s2 <- attributes(y)

  same_attributes <- intersect(
    names(s1),
    names(s2)
  )

  compare_attributes <- vapply(
    same_attributes,
    function(attr_name) {
      isTRUE(
        setequal(
          s1[[attr_name]],
          s2[[attr_name]]
        )
      )
    },
    logical(1)
  )

  matching_arguments <- names(
    compare_attributes[compare_attributes]
  )

  c_vector <- structure(
    vctrs::vec_c(
      vctrs::vec_data(x),
      vctrs::vec_data(y)
    ),
    id = c(
      attr(x, "id"),
      attr(y, "id")
    )
  )

  for (i in matching_arguments) {
    attr(c_vector, i) <- attr(x, i)
  }

  for (x_attr in setdiff(names(s1), names(s2))) {
    attr(c_vector, x_attr) <- attr(x, x_attr)
  }

  for (y_attr in setdiff(names(s2), names(s1))) {
    attr(c_vector, y_attr) <- attr(y, y_attr)
  }

  c_vector
}

#' Harmonize na_values in haven_labelled_spss
#'
#' @param df A data frame that contains haven_labelled_spss vectors.
#' @return A tibble where the na_values are consistent
#' @importFrom labelled to_character labelled na_values val_labels
#' @importFrom tibble deframe
#' @importFrom dplyr case_when
#' @examples
#' \donttest{
#' examples_dir <- system.file(
#'   "examples",
#'   package = "retroharmonize"
#' )
#'
#' test_read <- read_rds(
#'   file.path(examples_dir, "ZA7576.rds"),
#'   id = "ZA7576",
#'   doi = "test_doi"
#' )
#'
#' harmonize_na_values(test_read)
#' }
#' @export
#' @family harmonization functions

harmonize_na_values <- function(df) {
  missing_value_labels <- NULL

  # should be paramteric

  vars_with_spss_missings <- vapply(df, is.labelled_spss, logical(1))

  ## If there are no haven_labelled_spss in the tibble, return it ----
  if (!any(vars_with_spss_missings)) {
    return(df)
  }

  vars_with_spss_missings <- names(df)[which(vars_with_spss_missings)]

  # tibble::deframe ( df[,i])
  for (i in vars_with_spss_missings) {
    this_var <- tibble::deframe(df[, i])
    summary(this_var)
    these_values <- unique(this_var)

    these_labels <- labelled::val_labels(this_var)
    user_na <- labelled::na_values(this_var)
    not_missing_labels <- these_labels[!these_labels %in% user_na]
    missing_values <- these_labels[which(these_labels %in% user_na)]
    h_missing_value_labels <- tolower(names(missing_values))

    h_missing_value_labels <- dplyr::case_when(
      grepl("^inap", h_missing_value_labels) ~ "inap",
      grepl("^declin", h_missing_value_labels) ~ "declined",
      grepl("^na", h_missing_value_labels) ~ "missing",
      grepl("^dk", h_missing_value_labels) ~ "do_not_know",
      grepl("do_not", h_missing_value_labels) ~ "do_not_know",
      grepl("_inap", h_missing_value_labels) ~ "inap",
      grepl("_declin", h_missing_value_labels) ~ "declined",
      TRUE ~ paste0("<missing>_", h_missing_value_labels)
    )

    for (m in seq_along(missing_value_labels)) {
      change_label <- which(names(these_labels) == names(missing_values[m]))
      names(these_labels)[change_label] <- h_missing_value_labels[m]
    }

    harmonized <- labelled::labelled(unclass(this_var), these_labels)
    new_labels <- labelled::to_character(harmonized)

    harmonized_missing_labels <- vector(
      "numeric",
      length = length(h_missing_value_labels)
    )

    harmonized_missing_labels <- dplyr::case_when(
      grepl("inap", h_missing_value_labels) ~ 99999,
      grepl("^declin", h_missing_value_labels) ~ 99998,
      grepl("^na", h_missing_value_labels) ~ 99996,
      grepl("^do_not", h_missing_value_labels) ~ 99996,
      grepl("^<missing>", h_missing_value_labels) ~ 99901
    )

    harmonized_numeric_value <- dplyr::case_when(
      grepl("inap", new_labels) ~ 99999,
      grepl("^declin", new_labels) ~ 99998,
      grepl("^na", new_labels) ~ 99996,
      grepl("^do_not", new_labels) ~ 99996,
      grepl("^<missing>", new_labels) ~ 99901,
      TRUE ~ unclass(harmonized)
    )

    harmonized_labels <- dplyr::case_when(
      grepl("inap", names(these_labels)) ~ 99999,
      grepl("^declin", names(these_labels)) ~ 99998,
      grepl("^na", names(these_labels)) ~ 99996,
      grepl("^do_not_know", names(these_labels)) ~ 99996,
      grepl("^<missing>", names(these_labels)) ~ 99901,
      TRUE ~ these_labels
    )

    names(harmonized_labels) <- names(these_labels)

    replace_var <- labelled::labelled(
      harmonized_numeric_value,
      harmonized_labels
    )
    df[, i] <- replace_var
  }

  df
}

---
title: "Working with a Crosswalk Table"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Working with a Crosswalk Table}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r knitrsetup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message=FALSE}
library(retroharmonize)
library(dplyr)
```

Let's read in the package's small examples, and create a metadata table from them. These steps were explained in the vignette [Working With Survey Metadata](https://retroharmonize.dataobservatory.eu/articles/metadata.html).

From a technical perspective, the aim of the survey harmonization is to create a single, tidy, joined table in the form of a data frame that contains a row identifier, which is truly unique across all observations and the concatenated and harmonized variables.

```{r surveyfiles}
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
survey_files
```

```{r surveypaths}
survey_paths <- file.path(examples_dir, survey_files)
```

```{r examplemetadata}
example_metadata <- metadata_create(survey_paths = survey_paths)
```
## Crosswalk Table

A schema crosswalk, or a crosswalk table is a table that shows equivalent elements (or "fields") in more than one survey. In this example, we will create a crosswalk table of a subset of the tree example surveys. We will use three variables: the unique row identifier, the Trust in the European Parliament concept, and the country code.

By filtering out the other variables, we have the basic information in our metadata table.

```{r subsetmetadata}
subset_example_metadata <- example_metadata %>%
  filter(grepl("^unique_identifier_in|trust|country_code", var_label_orig)) %>%
  filter(grepl(
    "^unique_identifier_in|european_parliament|country_code",
    var_label_orig
  )) %>%
  filter(var_name_orig != "uniqid")

subset_example_metadata
```

### Create a crosswalk table

You can easily create a crosswalk table with `crosswalk_table_create()`. Crosswalk tables are validated with `is.crosswalk()`. You can create or modify your own crosswalk table in any spreadsheet program. The mandatory elements of a crosswalk table are 

* `id`: a unique identifier for a survey
* `filename`: the original source data file
* `var_name_orig`: the original variable name at source
* `var_name_target`: the original variable name after using the crosswalk, by default, it equal to     `var_name_orig`.  

All other columns are optional because they are needed for specific tasks, namely to change the numeric coding and labelling of valid and special survey response values, or to harmonize the eventual R type representation of the data.

```{r ct1}
ct <- crosswalk_table_create(subset_example_metadata)
ct
```

### Variable name harmonization

The harmonization of the variable names requires a single, unambiguous name for the variables that represent the same concept: 

```{r varnames}
ct %>%
  mutate(var_name_target = case_when(
    var_name_orig == "rowid" ~ .data$var_name_orig,
    var_name_orig == "isocntry" ~ "geo",
    TRUE ~ "trust_ep"
  )) %>%
  distinct(across(all_of(c("filename", "var_name_orig", "var_name_target"))))
```

### Value numeric code and label harmonization

For code and label harmonization, the crosswalk table should optionally contain instructions on harmonizing the numeric value codes and value labels. 

* The `crosswalk_table_create()` will create identical source and target columns for labeling (and type setting.) 
* You can save this table into `.csv` or `.xls` and edit it in a spreadsheet application if you want. If you do not change the default values (`source`=`target`) then the harmonization in those aspects will not take place. 

In this example, for full reproducability, we do not work in a spreadsheet program like Excel or Numbers or OpenOffice but we create programatically an unambiguous coding. To be on the safe side, the special missing value is placed far away from the normal values. 

```{r vallabels}
ct %>%
  mutate(val_numeric_target = case_when(
    val_numeric_orig == 1 ~ .data$val_numeric_orig,
    val_numeric_orig == 2 ~ 0,
    TRUE ~ 99999
  )) %>%
  mutate(val_label_target = case_when(
    val_numeric_orig == 1 ~ "trust",
    val_numeric_orig == 2 ~ "distrust",
    TRUE ~ "declined"
  )) %>%
  distinct(across(all_of(c(
    "filename",
    "val_numeric_orig", "val_numeric_target",
    "val_label_orig", "val_label_target"
  ))))
```
And now let's turn our attention to the special (missing) cases. Sometimes there may be other special cases in the code range, but the most likely suspects are values that represent some form of missing answers to a question or a missing questionnaire item selection.

```{r nalabels}
ct %>%
  mutate(na_numeric_target = case_when(
    na_numeric_orig == 3 ~ 99999,
    TRUE ~ NA_real_
  )) %>%
  mutate(na_label_target = case_when(
    na_numeric_target == 99999 ~ "declined",
    TRUE ~ NA_character_
  )) %>%
  distinct(across(all_of(c(
    "filename", "val_numeric_orig",
    "na_numeric_orig", "na_numeric_target",
    "na_label_orig", "na_label_target"
  ))))
```

### A reproducible recoding and relabelling

Let's put the entire process into a tidyverse pipeline with [dplyr](https://dplyr.tidyverse.org/):

```{r crosswalktarget}
example_crosswalk_table <- ct %>%
  mutate(var_name_target = case_when(
    var_name_orig == "rowid" ~ .data$var_name_orig,
    var_name_orig == "isocntry" ~ "geo",
    TRUE ~ "trust_ep"
  )) %>%
  mutate(val_numeric_target = case_when(
    val_numeric_orig == 1 ~ .data$val_numeric_orig,
    val_numeric_orig == 2 ~ 0,
    TRUE ~ 99999
  )) %>%
  mutate(val_label_target = case_when(
    val_numeric_orig == 1 ~ "trust",
    val_numeric_orig == 2 ~ "distrust",
    TRUE ~ "declined"
  )) %>%
  mutate(na_numeric_target = case_when(
    na_numeric_orig == 3 ~ 99999,
    TRUE ~ NA_real_
  )) %>%
  mutate(na_label_target = case_when(
    na_numeric_target == 99999 ~ "declined",
    TRUE ~ NA_character_
  ))

example_crosswalk_table
```

Needless to say that you do not need to work with *dplyr*. 

Reading the crosswalk table is very simple, for example, the last two (15-16) rows read like this:

* In `ZA6863.rds` rename the variable storing questionnaire item `qa14_1` to `trust_ep` (`Trust in European Parliament`).  
* Recode the numeric values from `2` to `0` and label them as `distrust`.
* Recode the special values from `3` to `99999` and label them as `declined`.

### Subsetting

From a technical perspective, the aim of the survey harmonization is to create a single, tidy, joined table. For making joining possible (and to reduce memory use), a first processing step is to remove irrelevant variables that will not be harmonized. 

There are several ways how the subsetting can be made.  With smaller tasks all the surveys can be stored in memory and the subsequent processing made fast in memory. With many surveys, we provide a slower but memory-saving way of importing and subsetting consecutive surveys from files. 

Recall from  the vignette [Working With Survey Metadata](https://retroharmonize.dataobservatory.eu/articles/metadata.html) that you can read files into a list of surveys: 

```{r readtomemory}
example_surveys <- read_surveys(survey_paths, .f = "read_rds")
```

Now let's focus of our attention to a small subset of the variables.

```{r subsetsave1}
subset_survey_list_1 <- subset_surveys(
  survey_list = example_surveys,
  subset_vars = c("rowid", "isocntry", "qa10_1", "qa14_1"),
  subset_name = "subset_example"
)
```

We still have the three surveys in a list: 

```{r subsetsave11}
vapply(subset_survey_list_1, function(x) attr(x, "id"), character(1))
``` 

The top few rows of the first subsetted survey to see if the subsetting took place:

```{r subsetsave13}
head(subset_survey_list_1[[1]])
```

While `qa10_1` and `qa14_1` refer to trust in the European Parliament, they cannot be joined because they have dissimilar names. These names are not too easy to remember, either. 

```{r subsetsave14}
lapply(subset_survey_list_1, names)
```

The next step is to harmonize the names of those variables that represent the same concept, in this case, *Trust in the European Parliament*.  The next steps, i.e. the harmonization of the numerical codes of answers and their labels will be discussed in the [Harmonize Value Labels](https://retroharmonize.dataobservatory.eu/articles/harmonize_labels.html) vignette.

## Variable Name Harmonization

It is very practical to do the subsetting and the variable name harmonization in one step.  The `subset_save_surveys()` function will do this optionally, and a wrapper function `harmonize_surveys()` will validate that all metadata (i.e. the original, source variable names and the new, target variable names) are present. 

```{r subsetsave21}
subset_survey_list_2 <- subset_surveys(
  crosswalk_table = example_crosswalk_table,
  survey_list = example_surveys,
  subset_name = "trust_ep"
)
```

We have again the three surveys in a list: 

```{r subsetsave22}
vapply(subset_survey_list_2, function(x) attr(x, "id"), character(1))
``` 

The top few rows of the first subsetted survey now show that we have new, harmonized names.

```{r subsetsave23}
head(subset_survey_list_2[[1]])
```

Our variable names are harmonized:  

```{r subsetsave24}
lapply(subset_survey_list_2, names)
```

While this example easily fits in the memory, when working with several dozens of SPSS files, it is better to sequentially import the surveys from file, and save the output to files. By default, the parameters `import_path` and `export_path` are set to `NULL`. If you enter a valid path to a directory, the function will look for the files specified in the `crosswalk_table$filename` on this path.

* If you do not specify the `export_path`, a list of surveys will be returned.
* If you specify the `export_path`, a vector of the saved file names (with full path) will be returned.

The `subset_surveys()` is a versatile function that fits with several workflows. It works in memory or with larger tasks, with sequentially read survey files; it for simple tasks it can use a simple vector for names to keep, or it can use an entire crosswalk table. You can read more about it with `?subset_surveys`.

```{r subsetsave}
subset_surveys(
  survey_list = example_surveys,
  crosswalk_table = example_crosswalk_table,
  subset_name = "trust_ep",
  import_path = examples_dir,
  export_path = tempdir()
)
```

The subsetted surveys are saved with a common name element, `trust_ep` to the export file location, in this case a temporary directory created with `tempdir()`.

```{r reread}
readRDS(file.path(tempdir(), "ZA5913_trust_ep.rds")) %>%
  head()
```

## Further Steps

Having a subsetted list of surveys with harmonized variable names is usually not yet an output that is ready for statistical analysis.  In the next step, the numerical codes, the variable labels need to be made consistent, with a special attention given to special values, particularly to missing values.

At last, if the statistical analysis will take place in R, a conversion to basic R classes is necessary to rely on the vast arsenal of R’s statistical packages. This means that the survey data must be brought to a consistent numeric or a consistent factor format (and in some cases, for visualization, to a character format).

We created a special s3 class (see `?labelled_spss_survey`), which retains the metadata about coding and special values, and created three methods that take into consideration the retained metadata. For example, if 999 is the code for declined answers, then the base are `as.numeric()` will coerce observations (survey responses) with a value `999` to a numerical value of `999`, but the `as_numeric()` method will give a `NA_real_` representation to this observation.  Averaging numerical, coded values will give a logically wrong result with the basic as.numeric, but a correct with the `as_numeric()` method.

The crosswalk table is a map for value code and label harmonization, and for type conversion, too.  This is the topic of the [Harmonize Value Labels](https://retroharmonize.dataobservatory.eu/articles/harmonize_labels.html) vignette.

---
title: "Case Study: Working With Afrobarometer surveys"
output: rmarkdown::html_vignette
resource_files:
  - vignettes/ab_plot1.png
vignette: >
  %\VignetteIndexEntry{Case Study: Working With Afrobarometer surveys}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, include=FALSE}
## https://github.com/tidyverse/rvest/blob/master/vignettes/selectorgadget.Rmd
requireNamespace("png", quietly = TRUE)
embed_png <- function(path, dpi = NULL) {
  meta <- attr(png::readPNG(path, native = TRUE, info = TRUE), "info")
  if (!is.null(dpi)) meta$dpi <- rep(dpi, 2)
  knitr::asis_output(paste0(
    "<img src='", path, "'",
    " width=", round(meta$dim[1] / (meta$dpi[1] / 96)),
    " height=", round(meta$dim[2] / (meta$dpi[2] / 96)),
    " />"
  ))
}
```

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(retroharmonize)
library(dplyr)
library(tidyr)
load(file = system.file(
  "afrob", "afrob_vignette.rda",
  package = "retroharmonize"
))
```

The goal of this case study is to explore the variation in trust in various state institutions among African societies, as well as changes in trust over time.

To do this, we use data from [Afrobarometer](https://www.afrobarometer.org/data/), a cross-national survey project measuring attitudes and opinions about democracy, the economy and society in African countries based on general population samples. Currently, seven rounds of the Afrobarometer are available, covering the period between 1999 and 2019. 

`retroharmonize` is not affiliated with Afrobarometer. To fully reproduce this example, you must acquire the data files from them, which is free of charge. Afrobarometer data is protected by copyright. Authors of any published work based on Afrobarometer data or papers are required to acknowledge the source, including, where applicable, citations to data sets posted on this website. Please acknowledge the copyright holders in all publications resulting from its use by means of bibliographic citation in this form:

In this vignette, we harmonize data from rounds [Afrobarometer Data Round 5 (all countries)](https://www.afrobarometer.org/survey-resource/merged-round-5-data-34-countries-2015/), [Afrobarometer Data Round 6 (all countries)](https://www.afrobarometer.org/survey-resource/merged-round-6-codebook-36-countries-2016/) and [Afrobarometer Data Round 7 (all countries)](https://www.afrobarometer.org/survey-resource/merged-round-7-data-34-countries-2019/).  Some elements of the vignette are not “live”, because we want to avoid re-publishing the original microdata files from Afrobarometer, but you can access the data directly from the [www.afrobarometer.org website](https://www.afrobarometer.org/).

File names (to identify file versions):  
Round 5: `merged-round-5-data-34-countries-2011-2013-last-update-july-2015.sav`  
Round 6: `merged_r6_data_2016_36countries2.sav`  
Round 7: `r7_merged_data_34ctry.release.sav`  

For reproducibility, we are storing only a small subsample from the files and the metadata. We assume that you extracted and copied all `.sav` files into a single folder that we will call in this vignette the `afrobarometer_dir`. Define your `afrobarometer_dir` with `file.path()` in your own system.

## Importing Afrobarometer Files

We start by reading in the three rounds of the Afrobarometer.

```{r setup, eval=FALSE}
library(retroharmonize)
library(dplyr)
library(tidyr)
```

```{r import, eval=FALSE}
### use here your own directory
ab <- dir(afrobarometer_dir, pattern = "sav$")
afrobarometer_rounds <- file.path(afrobarometer_dir, ab)

ab_waves <- read_surveys(afrobarometer_rounds, .f = "read_spss")
```

Let's give a bit more meaningful identifiers than the file names:

```{r set-id, eval=FALSE}
attr(ab_waves[[1]], "id") <- "Afrobarometer_R5"
attr(ab_waves[[2]], "id") <- "Afrobarometer_R6"
attr(ab_waves[[3]], "id") <- "Afrobarometer_R7"
```

We can review if the main descriptive metadata is correctly present with `document_surveys()`.

```{r document-ab, eval=FALSE}
documented_ab_waves <- document_surveys(ab_waves)
```
```{r}
print(documented_ab_waves)
```

Create a metadata table, or a data map, that contains information about the variable names and labels, and where each row correspnds to one variable in the survey data file. We do this with the function `metadata_survey_create()`, which extracts the metadata from the survery data files, normalizes variable labels, identifies ranges of substantive responses and missing value codes. Its wrapper, `metadata_create()`, loops the function on a list of surveys, or on a set of files containing surveys.

```{r create-metadata, eval=FALSE}
ab_metadata <- metadata_create(survey_list = ab_waves)
```

## Working with metadata

From the metadata file we select only those rows that correspond to the variables that we're interested in: the `rowid` being the unique case identifier, `DATEINTR` with the interview date, `COUNTRY` containing information about the country where the interview was conducted, `REGION` with the region (sub-national unit) where the interview was conducted, and `withinwt` being the weighting factor. 

Further, we select all variables that have the word "trust" in the normalized variable label.

For these variables, we create normalized variable names (`var_name`) and labels (`var_label`).

```{r, selection, eval=FALSE}
library(dplyr)
to_harmonize <- ab_metadata %>%
  filter(var_name_orig %in%
    c("rowid", "DATEINTR", "COUNTRY", "REGION", "withinwt") |
    grepl("trust ", label_orig)) %>%
  mutate(var_label = var_label_normalize(label_orig)) %>%
  mutate(var_label = case_when(
    grepl("^unique identifier", var_label) ~ "unique_id",
    TRUE ~ var_label
  )) %>%
  mutate(var_name = val_label_normalize(var_label))
```

To further reduce the size of the analysis, we only choose a few variables based on the normalized variable names (`var_name`).

```{r}
to_harmonize <- to_harmonize %>%
  filter(
    grepl("president|parliament|religious|traditional|unique_id|weight|country|date_of_int", var_name)
  )
```

The resulting table with information about the variables selected for harmonization looks as follows:

```{r}
head(to_harmonize %>%
  select(all_of(c("id", "var_name", "var_label"))), 10)
```

The `merge_surveys()` function harmonizes the variable names, the variable labels and survey identifiers and returns a list of surveys (of class `survey()`.) The parameter `var_harmonization` must be a list or a data frame that contains at least the original file name (`filename`), original variable names (`var_name_orig`), the new variable names (`var_name`) and their labels (`var_label`), so that the program knows which variables to take from what files and how to call and label them after transformation.


```{r merge, eval=FALSE}
merged_ab <- merge_surveys(
  survey_list = ab_waves,
  var_harmonization = to_harmonize
)

# country will be a character variable, and doesn't need a label
merged_ab <- lapply(merged_ab,
  FUN = function(x) {
    x %>%
      mutate(country = as_character(country))
  }
)
```

Review the most important metadata with `document_surveys()`:

```{r, eval=FALSE}
documenteded_merged_ab <- document_surveys(merged_ab)
```
```{r}
print(documenteded_merged_ab)
```

## Harmonization

In the Afrobarometer trust is measured with four-point ordinal rating scales. Such data are best analyzed with ordinal models, which do not assume that the points are equidistant. However, to get a quick idea of how the data look like, we will assign numbers 0-3, with 0 corresponding to the least trust, and 3 corresponding to the most trust, and for the time-being analyze the data as if they were metric.

To review the harmonization on a single survey use `pull_survey()`. Here we select Afrobarometer Round 6.

```{r check, eval=FALSE}
R6 <- pull_survey(merged_ab, id = "Afrobarometer_R6")
```
```{r pulled-attributes}
attributes(R6$trust_president[1:20])
```

The `document_survey_item()` function shows the metadata of a single variable. 

```{r document-item}
document_survey_item(R6$trust_president)
```

Afrobarometer's SPSS files do not mark the missing values, so we have to be careful. The set of labels that are identified as missing:

```{r}
collect_na_labels(to_harmonize)
```

The set of valid category labels and missing value labels are as follows:
  
```{r}
collect_val_labels(to_harmonize %>%
  filter(grepl("trust", var_name)))
```

We create a harmonization function from the `harmonize_values()` prototype function. In fact, this is just a re-setting the default values of the original function.  It makes future reference in pipelines easier, or it can be used for a question block only, in this case to variables with `starts_with("trust")`.

```{r specify}
harmonize_ab_trust <- function(x) {
  label_list <- list(
    from = c(
      "^not", "^just", "^somewhat",
      "^a", "^don", "^ref", "^miss", "^not", "^inap"
    ),
    to = c(
      "not_at_all", "little", "somewhat",
      "a_lot", "do_not_know", "declined", "inap", "inap",
      "inap"
    ),
    numeric_values = c(0, 1, 2, 3, 99997, 99998, 99999, 99999, 99999)
  )

  harmonize_values(
    x,
    harmonize_labels = label_list,
    na_values = c(
      "do_not_know" = 99997,
      "declined" = 99998,
      "inap" = 99999
    )
  )
}
```

We apply this function to the trust variables. The `harmonize_surveys()` function binds all variables that are present in all surveys.

```{r harmonize, eval=FALSE}
harmonized_ab_waves <- harmonize_surveys(
  survey_list = merged_ab,
  .f = harmonize_ab_trust
)
```

Let's look at the attributes of `harmonized_ab_waves`. 

```{r, eval=FALSE}
h_ab_structure <- attributes(harmonized_ab_waves)
```
```{r}
h_ab_structure$row.names <- NULL # We have over 100K row names
h_ab_structure
```

Let's add the year of the interview:

```{r year, eval=FALSE}
harmonized_ab_waves <- harmonized_ab_waves %>%
  mutate(year = as.integer(substr(as.character(
    date_of_interview
  ), 1, 4)))
```

To keep our example manageable, we subset the datasets to include only five countries.

```{r, eval=FALSE}
harmonized_ab_waves <- harmonized_ab_waves %>%
  filter(country %in% c(
    "Niger", "Nigeria", "Algeria",
    "South Africa", "Madagascar"
  ))
```


## Analyzing the harmonized data

The harmonized data can be exported and analyzed in another statistical program.  The labelled survey data is stored in `labelled_spss_survey()` vectors, which is a complex class that retains metadata for reproducibility. Most statistical R packages do not know it. The data should be presented either as numeric data with `as_numeric()` or as categorical with `as_factor()`.  (See more why you should not fall back on the more generic `as.factor()` or `as.numeric()` methods in [The labelled_spss_survey class vignette.](https://retroharmonize.dataobservatory.eu/articles/labelled_spss_survey.html))

Please note that the numeric form of these trust variables is not directly comparable with the numeric averages of the Eurobarometer trust variables, because the middle of the range is at `r mean(0:3)` and not `r mean(0:1)`.

```{r numeric}
harmonized_ab_waves %>%
  mutate_at(
    vars(starts_with("trust")),
    ~ as_numeric(.) * within_country_weighting_factor
  ) %>%
  select(-all_of("within_country_weighting_factor")) %>%
  group_by(country, year) %>%
  summarize_if(is.numeric, mean, na.rm = TRUE)
```

And the factor representation, without weighting:

```{r factor}
library(tidyr) ## tidyr::pivot_longer()
harmonized_ab_waves %>%
  select(-all_of("within_country_weighting_factor")) %>%
  mutate_if(is.labelled_spss_survey, as_factor) %>%
  pivot_longer(starts_with("trust"),
    names_to  = "institution",
    values_to = "category"
  ) %>%
  mutate(institution = gsub("^trust_", "", institution)) %>%
  group_by(country, year, institution, category) %>%
  summarize(n = n())
```

We subsetted the datasets to meet the vignette size limitations. If you are following our code without reducing the number of countries, you get the following results:

```{r, out.width='95%', echo=FALSE}
knitr::include_graphics("ab_plot1.png")
```

---
title: "documentation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{documentation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(retroharmonize)

examples_dir <- system.file("examples", package = "retroharmonize")
my_rds_files <- dir(examples_dir)[grepl(
  ".rds",
  dir(examples_dir)
)]
```

The `document_surveys()` function works with either a list of surveys in the memory, or a vector of paths to survey files. 

The function has two alternative input parameters. If `survey_list` is the 
input, it returns  the name of the original source data file, the number of rows and 
columns, and the size of the object as stored in memory. In case `survey_paths`
contains the source data files, it will sequentially read those files, and add the file
size, the last access and the last modified time attributes. 

```{r memory}
example_surveys <- read_surveys(file.path(examples_dir, my_rds_files))
documented_surveys <- document_surveys(survey_list = example_surveys)

attr(documented_surveys, "original_list")
documented_surveys
```
```{r files}
document_surveys(survey_paths = file.path(examples_dir, my_rds_files))
```

---
title: "Survey Harmonization"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Survey Harmonization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(retroharmonize)
```

Survey data harmonization refers to procedures that improve the data comparability or the inferential capacity of multiple surveys. *Ex ante* survey harmonization refers to planning and design steps to make sure that not yet answered questionnaires can be better compared, or data derived from them joined, integrated. Such procedures include the harmonization of the questionnaire, the harmonization of the sample design, and other aspects of carrying out multiple surveys. *Ex post* or *retrospective harmonization* refers to procedures to data that has been derived from surveys---i.e., survey that have been carried out. 

Naturally, better *ex ante* harmonization makes eventual data integration or data comparison easier; yet often we can still harmonize retrospectively survey data that has not been carefully pre-harmonized before respondents have answered the questionnaire items.

Our aim with the `retroharmonize` R package is to provide assistance to a reproducible research workflow in carrying out important computational aspects of retrospective survey harmonization. 

Let's start with a very simple example.

```{r simple-example-def}
library(labelled)
survey_1 <- data.frame(
  sex = labelled(c(1, 1, 0, NA_real_), c(Male = 1, Female = 0))
)
attr(survey_1, "id") <- "Survey 1"

survey_2 <- data.frame(
  gender = labelled(c(1, 3, 9, 1, 2), c(male = 1, female = 2, other = 3, declined = 9))
)
attr(survey_2, "id") <- "Survey 2"
```


```{r ex1}
library(dplyr, quietly = TRUE)
survey_1 %>%
  mutate(
    sex_numeric = as_numeric(.data$sex),
    sex_factor = as_factor(.data$sex)
  )
```

## Tasks in the harmonization workflow

The ordering of the survey harmonization workflow is flexible, and it is likely that even the same researcher would choose a different workflow in the case of smaller, simpler harmonization tasks and more complex harmonization tasks.

The data science aspect of a successful survey harmonization task is the creation of a consistent data frame that contains harmonized information from multiple surveys. It practically means that questionnaire items are mapped into variables with a consistent numerical coding, descriptive metadata (variable and value labels) and a consistent handling of missing and special values. This may be very laborous task when surveys are conducted in different years, saved in different file formats with a different metadata structure, missing and special values are handled differently, and the metadata contains potentially different natural language descriptions or spelling.

`Survey 1` labels the sex of respondents as `Male` and `Female`, and has cases that are neither `Male` or `Female`, but we do not know why.

```{r example2}
survey_2 %>%
  mutate(
    gender_numeric = as_numeric(.data$gender),
    gender_factor = as_factor(.data$gender)
  )
```

`Survey 2` records gender, which contains the same information as sex in `Survey 1` (`Male` and `Female`), but allows people to identify as `Other`, and labels cases when people decline to identify with any of these three categories.

In practice, you want to end up with the following joined representation of your survey:

```{r manually-joined}
survey_joined <- data.frame(
  id = c(1, 2, 3, 4, 1, 2, 3, 4, 5),
  survey = c(rep(1, 4), rep(2, 5)),
  gender = labelled(c(1, 1, 0, 9, 1, 3, 9, 1, 0), c(male = 1, female = 0, other = 3, declined = 9))
)

survey_joined %>%
  mutate(
    id = paste0("survey_", .data$survey, "_", .data$id),
    gender_numeric = c(1, 1, 0, NA_real_, 1, 3, NA_real_, 1, 0),
    gender_factor = as_factor(.data$gender),
    is_female = ifelse(.data$gender_numeric == 0, 1, 0)
  )
```


1.  Harmonization of concepts
    *  Create a mental map of the measured concepts that needs to be harmonized. Which variables contain sufficiently similar information that can be harmonized? In our simple example, we want to harmonize a binary sex with missing cases and a four-level categorical variable on gender identification, and concatenate the harmonized vectors by binding or joining the Survey 1 and Survey 2 data frames.
    *  Our metadata function help mapping the information stored in the file representations of multiple surveys. We want to create a simple inventory of numerical codes, value ranges, missing cases and variable labels.
2.  Variable names
    *  Data measuring sufficiently similar concepts, i.e. data that can be harmonized, is stored in variables that have the same name in different data frames representing the survey, therefore they can be bind or joined together. We want to join or bind by rows `survey_1` with `survey_2`, or, we want to concatenate `survey_1$sex` with `survey_2$gender`.
    *  Descriptive metadata about the variable, such as "variable labels" in SPSS files, is recorded for documentation, and if needed, harmonized across surveys. In SPSSS,  `survey_1$sex` may come with a variable label something like *SEX OF RESPONDENT*, and `survey_2$gender` may be labelled as *GENDER IDENTIFICATION*. This label should be harmonized to *Sex or gender or the respondent*. 
3.  Variable coding and labels
    * Variables recording or measuring the same concept, such as the gender of the respondent, are coded exactly the same way, for example, females with 0, males with 1, non-binary respondents with 3, and people declining to reveal their gender with 9. This means that observations in `survey_2$gender` coded with a numeric 2 must be changed to a numeric 0.
    * Variable labels are used consistently and in exactly the same way, i.e. `survey_1$sex` *Female* respondents and `survey_2$gender` *female* respondents will be consistently labelled as *female*.
4.  Variable types
    * Variables recording or measuring the same concept are stored in exactly the same R type, and they can be consistently concatenated across surveys, or they can be subsetted, cross-cutting surveys, for example, all female respondents from Survey 1 and Survey 2 can be subsetted into a female vector.
    * The labelled class of [labelled](https://larmarange.github.io/labelled/articles/labelled.html) is not sufficiently strict, because it allows inconsistent special (missing) values.  Our inherited [labelled_spss_survey](https://retroharmonize.dataobservatory.eu/reference/labelled_spss_survey.html) consistently contains codes, labels, missing ranges and missing values, and therefore it can be concatenated.
    * The numeric or factor representation of `survey_1$sex` and `survey_2$gender` can be technically concatenated, but before harmonization this will create logical errors, because females will be either coded with 0 or with 2. The `as_numeric()` and `as_factor()` methods of our [labelled_spss_survey](https://retroharmonize.dataobservatory.eu/reference/labelled_spss_survey.html)  class handle consistency issues.
5.  Reproducibility
    * The revision, checking, external review and audit of the data requires that the steps can be replicated by a third party. This requires a documentation of the harmonization steps, i.e., 1=Women in Survey 1, and 0=female in Survey 2 became 0=females in the harmonized dataset.
    * Our [survey](https://retroharmonize.dataobservatory.eu/reference/survey.html) class is derived from [tibble](https://tibble.tidyverse.org/), the modernized version of the base `data.frame()`. It contains various descriptive metadata about the survey among attributes.
    
The joining of the not harmonized datasets results in the following data frame.
    
```{r}
library(dplyr)

survey_1 %>%
  mutate(
    survey = 1,
    sex_numeric = as_numeric(.data$sex),
    sex_factor = as_factor(.data$sex)
  ) %>%
  full_join(
    survey_2 %>%
      mutate(
        survey = 2,
        gender_numeric = as_numeric(.data$gender),
        gender_factor = as_factor(.data$gender)
      )
  )
```  
    
Performing only variable harmonization yields to a data frame that has the correct dimensions, but it is not usable for statistical analysis. 

    
```{r naive-join}
library(dplyr)

survey_var_harmonized <- survey_1 %>%
  rename(gender = .data$sex) %>%
  mutate(
    survey = 1,
    gender_numeric = as_numeric(.data$gender),
    gender_factor = as_factor(.data$gender)
  ) %>%
  full_join(
    survey_2 %>%
      mutate(
        survey = 2,
        gender_numeric = as_numeric(.data$gender),
        gender_factor = as_factor(.data$gender)
      ),
    by = c("gender", "survey", "gender_numeric", "gender_factor")
  )
```      

Apart from the simple, descriptive variable of the survey identification, non of the descriptive statistics are meaningful.    

```{r}
summary(survey_var_harmonized)
```
The variable labels must be harmonized for a successful factor representation.  The numerical coding must be harmonized, and the missing cases must be consistently handled to achieve any useful numerical representation. 

```{r}
survey_joined %>%
  mutate(
    id = paste0("survey_", .data$survey, "_", .data$id),
    gender_numeric = c(1, 1, 0, NA_real_, 1, 3, NA_real_, 1, 0),
    gender_factor = as_factor(.data$gender),
    female_ratio = ifelse(.data$gender_numeric == 0, 1, 0)
  ) %>%
  summary()
```
## How we help harmonization

1. The *data importing* functions make sure that survey data and metadata are carefully translated to R data classes and variable types.  

2. The *metadata functions* help the analysis, normalization and joining of the metadata aspects (variable and value labels, original variable names, unique identifiers) across surveys.

3. *Harmonization functions* help the harmonization of responses to questionnaire items, i.e. making sure that coded values, the labelling of values, and missing data are handled consistently across multiple surveys.

Our package was tested on multiple, international, harmonized surveys, particularly the Eurobarometer, the Afrobarometer and the Arab Barometer survey programs. Different users, and different task call for different workflows.  We created a number of helper functions to assist various workflows.  

---
title: "The labelled_spss_survey class"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The labelled_spss_survey class}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Large international survey programs that provide data access to their data, such as the aforementioned Eurobarometer, Afrobarometer and Arab Barometer, or the European Values Survey, or Lationbarometro usually give access to the data in SPSS files. In some cases, Stata files are available, too. Our examples always start with SPSS files.

In R, the [haven](https://haven.tidyverse.org/) package provides functions to import and export data to and from IBM's proprietary SPSS files. We had to realize that while [haven::read_spss](https://haven.tidyverse.org/reference/read_spss.html) works perfectly with importing the data and metadata of a single survey, it is not always suitable for multiple survey files, for two important reasons. SPSS files contain the survey data in a coded form, with the coding (labelling) metadata optionally included to each variable.  

1. Variables imported with inconsistent labelling cannot always be concatenated.  For example, unlabelled age variables are imported to an R numeric vector, whilst when the age 18 is labelled *18 years* or *18 éves*, it is imported to a labelled class. 

2. The SPSS variables do not handle various missing cases in a complete and unambiguous form.  In an age variable, *998* and *999* may be labelled as *not asked* and *declined to answer*, or simply the numerical range between 120-999 may be marked as a range of numeric values representing missing cases.

One practical problem in the Eurobarometer surveys, which targets the population of at least 15 years old Europeans is that in some standard questions, such as the age of finishing full-time education, 10 represents a special missing value, whilst in other surveys it may be a perfectly valid numerical answer. The real coding problem is that SPSS users can freely chose to use explicit labelling of missing cases, using a numerical range for missing cases, or not providing any missing case metadata at all.

Our importing functions rely on two new S3 classes.  A single survey is imported into a `survey()` class, which inherits all the properties of a modern data frame, i.e., it is a `tibble` or `tbl_df` from [tibble](https://tibble.tidyverse.org/) in the tidyverse, but it includes as much metadata from the original SPSS file as possible. These metadata attributes are handled in a way that they can facilitate proper documentation and a reproducible workflow. Furthermore, it converts labelled variables into the *retroharmonize_labelled_spss* class inherited from [haven::labelled_spss](https://haven.tidyverse.org/reference/labelled_spss.html) with more consistent handling of missing value ranges and labels. See `?labelled_spss_survey`.

Working with our new *retroharmonize_labelled_spss class* can be very cumbersome, particularly with simple harmonization tasks. In the case of harmonizing a single question from two surveys, this may not be practical, and a simple crosswalk table can help with spotting and correcting inconsistent codes.

Our survey objects or retroharmonize_labelled_spss vectors can be converted to base R classes with the `as_numeric()`, `as_factor(`) or `as_character()` methods.  When computing a numerical average,  the special age value of 10 is converted to `NA_real`_ as a numeric. In other statistical applications, missing and special values are best represented as categories---this calls for the factor representation. The character representation is often more useful for visualizing the data then the factor representation.

## Create A labelled_spss_survey Vector

```{r setup}
library(retroharmonize)
```

Use the `labelled_spss_survey()` helper function to create vectors of class *retroharmonize_labelled_spss_survey*.

```{r}
sl1 <- labelled_spss_survey(
  x = c(1, 1, 0, 8, 8, 8),
  labels = c(
    "yes" = 1,
    "no" = 0,
    "declined" = 8
  ),
  label = "Do you agree?",
  na_values = 8,
  id = "survey1"
)

print(sl1)
```

You can check the type: 

```{r}
is.labelled_spss_survey(sl1)
```

The `labelled_spss_survey()` class inherits some properties from `haven::labelled()`, which can be manipulated by the `labelled` package (See particularly the vignette *Introduction to labelled* by Joseph Larmarange.)

```{r}
haven::is.labelled(sl1)
```
```{r}
labelled::val_labels(sl1)
```
```{r}
labelled::na_values(sl1)
```
It can also be subsetted:

```{r}
sl1[3:4]
```

When used within the modernized version of *data.frame*, `tibble::tibble()`, the summary of the variable content prints in an informative way.

```{r}
df <- tibble::tibble(v1 = sl1)
## Use tibble instead of data.frame(v1=sl1) ...
print(df)
## ... which inherits the methods of a data.frame
subset(df, v1 == 1)
```

## Coercion rules and type casting

To avoid any confusion with mis-labelled surveys, coercion with double or integer vectors will result in a double or integer vector. The use of `vctrs::vec_c` is generally safer than base R `c()`.

```{r}
# double
c(sl1, 1 / 7)
vctrs::vec_c(sl1, 1 / 7)
```
```{r integer}
c(sl1, 1:3)
```

Conversion to character works as expected:

```{r character}
as.character(sl1)
```
The base `as.factor` converts to integer and uses the integers as levels, because base R factors are integers with a `levels` attribute.

```{r as.factor}
as.factor(sl1)
```

Conversion to factor with `as_factor` converts the value labels to factor levels:

```{r as_factor}
as_factor(sl1)
```
Similarly, when converting to numeric types, we have to convert the user-defined missing values to `NA` values used in the R language. For numerical analysis, convert with `as_numeric`.

```{r numerics}
as.numeric(sl1)
as_numeric(sl1)
```
## Arithmetics 

The median value is correctly displayed, because user-defined missing values are removed from the calculation. Only a few arithmetic methods are implemented, such as 

* median()

```{r}
median(as.numeric(sl1))
median(sl1)
```

* quantile()

```{r}
quantile(as.numeric(sl1), 0.9)
quantile(sl1, 0.9)
```

* mean()

```{r}
mean(as.numeric(sl1))
mean(sl1)
mean(sl1, na.rm = TRUE)
```

* weighted.mean() - always removes NA values.

```{r}
weights1 <- runif(n = 6, min = 0, max = 1)
weighted.mean(as.numeric(sl1), weights1)
weighted.mean(sl1, weights1)
```

* sum()

```{r}
sum(as.numeric(sl1))
sum(sl1, na.rm = TRUE)
```

The result of the conversion to numeric can be used for other mathematical / statistical function. 

```{r}
as_numeric(sl1)
min(as_numeric(sl1))
min(as_numeric(sl1), na.rm = TRUE)
```

---
title: "Working With Survey Metadata"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Working With Survey Metadata}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r knitropts, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(retroharmonize)
library(dplyr)
```

```{r surveyfiles}
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
survey_files
```
## Working With a Single Survey

```{r readfiles}
survey_1 <- read_rds(file.path(examples_dir, survey_files[1]))
```

This function should be renamed and slightly rewritten, it does too many things.  

```{r metadatacreate}
metadata_create(survey_1) %>% head()
```

## Working With Multiple Surveys

```{r surveypaths}
survey_paths <- file.path(examples_dir, survey_files)
```

With smaller data frames representing your surveys, the most efficient way to work with the information is to read them into a list of surveys. 

Read the surveys into a list object in the memory:

```{r readtolist}
example_surveys <- read_surveys(survey_paths, .f = "read_rds")
```

Map the metadata contents of the files:

```{r processlist}
set.seed(2022)
metadata_create(survey_list = example_surveys) %>%
  sample_n(12)
```

If you may ran out of memory, you can work with files.  The advantage of keeping the surveys in memory is that later it will be much faster to continue working with them, but from the metadata point of view, the returned object is the same either way.

```{r createmetadatasurveys}
example_metadata <- metadata_create(survey_paths = survey_paths, .f = "read_rds")
```
```{r printexample}
set.seed(2022)
example_metadata %>%
  sample_n(12)
```
A quick glance at some metadata:

```{r subsetmetadata}
library(dplyr)
subset_example_metadata <- example_metadata %>%
  filter(grepl("trust", .data$var_label_orig)) %>%
  filter(grepl("european_parliament", .data$var_label_orig)) %>%
  select(all_of(c("filename", "var_label_orig", "var_name_orig", "valid_labels", "na_labels", "class_orig")))

subset_example_metadata
```

In `ZA5913.rds` the Trust in European Parliament variable is called `qa10_1`, in the other surveys it is called `qa14_1`.  

In the first survey, the variable has two values (coded as 1 and 2, and labelled as `Tend to trust` and `Tend not to trust`. )

```{r examplelabels}
unlist(subset_example_metadata$valid_labels[1])
```

In the first survey, the variable has two values (coded as 1 and 2, and labelled as `Tend to trust` and `Tend not to trust`.) In the second survey, we have three values, and non of them are marked as special, missing values. This is not surprising, because they were not SPSS files. They have related, but not exactly matching classes, too.  Therefore, these variables need to be harmonized.

```{r examplenalabels2}
unlist(subset_example_metadata$valid_labels[2])
unlist(subset_example_metadata$na_labels[2])
```
The metadata created by the `metadata_create()` and its version for multiple surveys, `metadata_create`, gives a first overview for the harmonization of concepts, the necessary harmonization of variable names and variable labels. In this case:

* Variable name harmonization is required: For a successful join, you must use a common name for
    `qa10_1` and `qa14_1`, for example, `trust_european_parliament`, because the variable refers to the same concept.
* Variable label harmonization is required: You must make sure that the variable has three
    categories, even if one category, `Declined` (to answer) is missing from `ZA5913.rds`.
* Type harmonization is needed: For various statistical procedures you must convert the concatenated contents of  `qa10_1` and `qa14_1` into a numerical variable or factor variable. It is practical to use *1="Tend to Trust"*, *0="Tend not to trust"* for calculating the percentage of people trusting the European Parliament, making sure that Decline will get a `NA_real_` value for averaging, or creating a factor variable with three levels, for example *trust*, *not_trust*, *declined*.

---
title: "Harmonizing Concepts, Questions, and Variables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Harmonizing Concepts, Questions, and Variables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message=FALSE}
library(retroharmonize)
library(dplyr)
```

The first step of retrospective harmonization is finding the relevant concepts, operationalized in questions that need to be harmonized among two or more surveys.

## Concept


## Questions

```{r surveyfiles}
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir)[grepl("\\.rds", dir(examples_dir))]
survey_files
```

```{r surveypaths}
survey_paths <- file.path(examples_dir, survey_files)
```

With smaller data frames representing your surveys, the most efficient way to work with the information is to read them into a list of surveys. 

Read the surveys into a list object in the memory:

```{r readtolist}
example_surveys <- read_surveys(survey_paths, .f = "read_rds")
```


If you may ran out of memory, you can work with files.  The advantage of keeping the surveys in memory is that later it will be much faster to continue working with them, but from the metadata point of view, the returned object is the same either way.

```{r createmetadatasurveys}
# not evaluated
example_metadata <- metadata_create(survey_paths = survey_paths, .f = "read_rds")
```

Let's work in the memory now. Map the metadata contents of the files:

```{r processlist}
set.seed(2022)
metadata_create(survey_list = example_surveys) %>%
  dplyr::sample_n(12)
```
The current retroharmonize uses the metadata_create() function to restore the encoded metadata into a tidy table that can be the start of further steps.  This function should be revised after much use, and brought to a simpler format, and renamed, preferably choosing a DDI Glossary term. (Ingest?  Or just mapping?  Should not contain any tidyverse verbs.)
C2: The selected variables from the metadata table (which needs a better word) we subset the surveys either in memory or, in case of many files, sequentially from file.  This the subset_survey() function. It will need a thorough upgrade to correctly retain the attributes of the datacube-inheritted new survey class, but it functions well.

This stage should be harmonized with the [DDI Codebook](https://zenodo.org/records/14447460). One problem appears to me is that DDI calls a “codebook” differently than we do.  DDI uses the term codebook on the level of file (survey), and we use it on the level of individual observations. 

> Codebook: A document that provides information on the structure, contents, and layout of a data file. Source: [DDI Glossary](https://docs.ddialliance.org/DDI-Codebook/2.5/xmlschema/schemas/codebook_xsd/elements/codeBook.html).

Here is a DDI Codebook example in [PDF](https://ddialliance.org/hubfs/sites/default/files/National%20Household%20Survey,%202011%20%5BCanada%5D%20Public%20Use%20Microdata%20File%20(PUMF)-%20Individuals%20File.pdf).

Because normally we want to use standardized codes, and we started to harmonize with the SDMX statistical metadata standard, a good resolution seems to be to differentiate between a Codebook (DDI term) and a Codelist (SDMX term, but I am sure it has a more general RDF definition.)

We roughly have a DDI Codebook regarding the concepts and question items, but the number of Valid and Invalid responses were not collected at ingestion:

```{r}
set.seed(12)
example_metadata %>%
  select(
    Filename = .data$filename,
    Name = .data$var_name_orig,
    Label = .data$var_label_orig,
    Type = .data$class_orig,
    Format = .data$labels
  ) %>%
  mutate(
    Valid = NA_real_,
    Invalid = NA_real_,
    Question = NA_character_
  ) %>%
  sample_n(12)
```

The DDI Codebook is however, a lot more, because it contains survey-level metadata that we did not use in retroharmonize so far. We assumed that the user (researcher) did a comparison of sampling methods, collection modes, etc, which are all part of the DDI Codebook standard.

It would be very easy to write a codebook_create() function that would create a partial DDI codebook as a component of a future DDI Codebook function codebook_create_ddi() and keep working with this.

However, we have a problem, the current, released retroharmonize has a more complex `create_codebook()` function. This should be depracted.

```{r}
set.seed(12)
my_codebook <- create_codebook(
  survey = read_rds(
    system.file("examples", "ZA7576.rds",
      package = "retroharmonize"
    )
  )
)

sample_n(my_codebook, 12)
```


## Reproducible research tasks

The tasks that we do with this information is variable name and variable label harmonization.

```{r}
metadata <- metadata_create(example_surveys)
metadata$var_name_suggested <- label_normalize(metadata$var_name)
metadata$var_name_suggested[metadata$label_orig == "age_education"] <- "age_education"

harmonized_example_surveys <- harmonize_var_names(
  survey_list = example_surveys,
  metadata = metadata
)

lapply(harmonized_example_surveys, names)
```
There is, however, an important extra step, what the DDI codebook calls Type and Format matching.  This is software/computer language dependent, but our codebook could easily accommodate this with containing the generic DDI Codebook

```{r, results='asis', message=FALSE}
data.frame(
  Type = rep("discrete", 3),
  Format = c("numeric-1.0", "numeric-2.0", "numeric-6.0"),
  r_type = rep("integer", 3),
  range = c("0..9", "10..99", "100000..999999")
) %>% knitr::kable()
```

These variables can be mapped either to our [labelled_spss_survey](https://retroharmonize.dataobservatory.eu/articles/labelled_spss_survey.html) class or Adrian Dusa's [declared](https://github.com/dusadrian/declared).

Considerations:
- The *labelled_spss_survey* or *declared* is necessary because R does not have a missing case identifier that can distinguish declined answers or answers that were not collected.
- There must be a clear coercion (without "lazy" and ambiguous coercion) to at least R integer, numeric, character or factor classes for further use in R's statistical functions or visualization functions.
- Integers can easily be coerced into characters, but this is not necessarily a good idea, because some functions anyway want a numeric input, and characters require a lot more space to be stored in memory or in a file. 

```{r}
as.integer(1982)
as.character(as.integer(1982))
```
we can assume that we only use integer representation for coded questionnaire items, but we still may have open text responses or observation identifiers that are character vectors.  It is likely that the use of character-represented identifiers is a better idea in later stages. So we must work with a class that can be converted (coerced) into both integer (numeric) and character formats. 

The choice has profound consequences for variable label harmonization and the harmonization of codelists, but not at the level of concepts, questions and codebooks.

```{r, results='asis', message=FALSE}
data.frame(
  Type = rep("discrete", 3),
  Format = c("numeric-1.0", "numeric-2.0", "numeric-6.0"),
  r_type = rep("declared", 3),
  range = c("Male|Female|DK", "10..99", "100000..999999")
) %>% knitr::kable()
```

## Question Banks

Question banks contain information about questions asked about the same concepts in different surveys.

"Using DDI as a foundation for a question bank enables you to reuse metadata and to find identical and similar questions and or response sets across surveys for purposes of data comparison, harmonization, or new questionnaire development." 

[Create a Question Bank](https://ddialliance.org/create-a-codebook)

**Social Science Variables Database**: (located at ICPSR) 
Search over 4 million variables. 
Also able to compare questions across studies and series.
<https://www.icpsr.umich.edu/sites/icpsr/find-data>

**UK Data Service Variable and Question Bank**:
Search hundreds of surveys.
<https://datacatalogue.ukdataservice.ac.uk/searchresults?sort=2&tab=3>

**Survey Data Netherlands**:
Over 36,000 questions to search. 
<https://surveydata.nl>

Obviously we should facilitate the use of existing question banks, and create question banks that interoperable with existing ones.

Let’s take a look at concerts in the Eurobarometer series.
<https://www.icpsr.umich.edu/web/ICPSR/series/26/variables?q=concert>
Here is the variable that we use in our use case:
<https://www.icpsr.umich.edu/web/ICPSR/studies/35505/datasets/0001/variables/QB1_4?archive=icpsr>

A short caveat:  the questionnaire item may or may not be copyright protected.  The reuse of the questionnaire requires further research.

Here we have Values (1…5) and their labels (Not in the last 12 months, 1-2 times, etc)

- We must clarify with Eurobarometer, Afrobarometer, etc, if we can reuse their questions in question banks, and are researchers to use it in harmonized surveys?

The question bank information already contains information for the next step, the harmonization of value labels and codelists not covered in this vignette.

## Literature review

- Standard view in the literature on concept and question harmonization. Any difference with ex ante and ex post harmonization?
- Comparison of the DDI Constructs: Concept, Question --- correctly representing this in our function descriptions and vignettes.

## Coding tasks {#concept=coding}

We should follow the [rOpenSci Packages: Development, Maintenance, and Peer Review](https://devguide.ropensci.org/) for future changes.  In designing and deprecating functions, the relevant parts are 

- [1.3.1 Function and argument naming](https://devguide.ropensci.org/building.html#function-and-argument-naming)
- [15 Package evolution - changing stuff in your package](https://devguide.ropensci.org/evolution.html)

1. `create_codebook()` will be deprecated, luckily, it does not meet the rOpenSci object_verb suggestion.
2. `codebook_create()` will create a DDI-Codebook compatible, partial codebook, only covering tasks that are relevant for retroharmonize. The core of the codebook will be compatible with DDI-Codebook, but further information about the R specific implementation of the codebook will be added.
3. `codebook_export_ddi()` will add further data (whatever we have but do not use) to make a more complete, but not necessarily complete DDI Codebook object.  [Not a high priority now.]

---
title: "Harmonize Value Labels"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Harmonize Value Labels}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(retroharmonize)
```

This vignette follows up on the [Working With A Crosswalk Table](https://retroharmonize.dataobservatory.eu/articles/crosswalk.html). In the that vignette, you learned how to remove variables that cannot be harmonized with `subset_surveys()` and harmonize variable names with `harmonize_survey_variables()`.

As a result of these steps, you have a list of surveys, or surveys saved in files that are harmonization candidates. They now need a consistent numerical coding, labelling with special attention given to missing values and other special values.


## Harmonize value codes and labels

The function `harmonize_values()` solves problems in the following situations:

1. When the data are read from an SPSS file, in one dataset the variable `survey1$trust` has no user-defined missing values, but in another dataset the variable `survey2$trust` does have missing values defined. The two variables cannot be combined. We add harmonized missing values to the missing value range, even if they are not present among the observations.   

2. The labels are not matching in `survey1$trust` and `survey2$trust`. We harmonize the labels, and record their initial values for reproducibility.

3. The missing value ranges in `survey1$trust` and `survey2$trust` do not match. We harmonize the missing values, and record their initial values for reproducibility.

4. There are unexpected labels present in the range of substantive or missing values.  They are taken out from the value range with a special code and marked with a special label.


### Scenario 1

All values are present, and only the missing values are recoded. 

```{r}
v1 <- labelled_spss_survey(
  c(1, 0, 1, 9),
  labels = c(
    "yes" = 1,
    "no" = 0,
    "inap" = 9
  ),
  na_values = 9
)

h1 <- harmonize_values(
  x = v1,
  harmonize_labels = list(
    from = c("^yes", "^no", "^inap"),
    to = c("trust", "not_trust", "inap"),
    numeric_values = c(1, 0, 99999)
  ),
  id = "survey1"
)

str(h1)
```
* the attribute `survey1_values` may be used to restore the original coding.

* the attribute `survey1_labels` may be used to restore the original labelling.

* the attribute `na_values` can re-define if a category should be treated as missing.

The `to_numeric()` method converts the missing value range to `NA_real_`.

### Scenario 2

The original variable is of class `haven::labelled_spss()`. It has an invalid missing value.

```{r}
v2 <- haven::labelled_spss(
  c(1, 1, 0, 8),
  labels = c(
    "yes" = 1,
    "no" = 0,
    "declined" = 8
  ),
  na_values = 8
)

h2 <- harmonize_values(
  v2,
  harmonize_labels = list(
    from = c("^yes", "^no", "^inap"),
    to = c("trust", "not_trust", "inap"),
    numeric_values = c(1, 0, 99999)
  ),
  id = "survey2"
)
str(h2)
```

We apply the code `99901` for this value and label it as `invalid_label`. 

After modifying the user-defined missing value labels:

```{r}
h2b <- harmonize_values(
  v2,
  harmonize_labels = list(
    from = c("^yes", "^no", "^decline"),
    to = c("trust", "not_trust", "inap"),
    numeric_values = c(1, 0, 99999)
  ),
  id = "survey2"
)

str(h2b)
```

### Scenario 3

The original vector is of class `haven_labelled`, therefore it has no defined missing value range. We want to remove `DK` from the value range to the missing range as `do_not_know`. The original vector also has an unlabelled value (9). Because we believe that in this vector all values should have a value label, we treat it as an invalid observation.

```{r}
var3 <- labelled::labelled(
  x = c(1, 6, 2, 9, 1, 1, 2),
  labels = c(
    "Tend to trust" = 1,
    "Tend not to trust" = 2,
    "DK" = 6
  )
)

h3 <- harmonize_values(
  x = var3,
  harmonize_labels = list(
    from = c(
      "^tend\\sto|^trust",
      "^tend\\snot|not\\strust", "^dk",
      "^inap"
    ),
    to = c(
      "trust",
      "not_trust", "do_not_know",
      "inap"
    ),
    numeric_values = c(1, 0, 99997, 99999)
  ),
  id = "S3_"
)

str(h3)
```

### Base Types & Summary

* as factor:

```{r}
summary(as_factor(h3))
levels(as_factor(h3))
unique(as_factor(h3))
```
* as numeric:

```{r}
summary(as_numeric(h3))
unique(as_numeric(h3))
```

* as character:

```{r}
summary(as_character(h3))
unique(as_character(h3))
```

## Combination of harmonized values

You can combine `labelled_spss_survey` vectors if the metadata describing their current state is an exact match. This means that the labels, missing values and missing range are defined the same way, and the base type of the vector is matching numeric or character --- though labelling character vectors makes little sense.

The historic metadata, i.e. the earlier naming and coding of the variable do not have to match, they are added to all "inherited vectors".


```{r}
var1 <- labelled::labelled_spss(
  x = c(1, 0, 1, 1, 0, 8, 9),
  labels = c(
    "TRUST" = 1,
    "NOT TRUST" = 0,
    "DON'T KNOW" = 8,
    "INAP. HERE" = 9
  ),
  na_values = c(8, 9)
)

var2 <- labelled::labelled_spss(
  x = c(2, 2, 8, 9, 1, 1),
  labels = c(
    "Tend to trust" = 1,
    "Tend not to trust" = 2,
    "DK" = 8,
    "Inap" = 9
  ),
  na_values = c(8, 9)
)


h1 <- harmonize_values(
  x = var1,
  harmonize_label = "Do you trust the European Union?",
  harmonize_labels = list(
    from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
    to = c("trust", "not_trust", "do_not_know", "inap"),
    numeric_values = c(1, 0, 99997, 99999)
  ),
  na_values = c(
    "do_not_know" = 99997,
    "inap" = 99999
  ),
  id = "survey1"
)

h2 <- harmonize_values(
  x = var2,
  harmonize_label = "Do you trust the European Union?",
  harmonize_labels = list(
    from = c("^tend\\sto|^trust", "^tend\\snot|not\\strust", "^dk|^don", "^inap"),
    to = c("trust", "not_trust", "do_not_know", "inap"),
    numeric_values = c(1, 0, 99997, 99999)
  ),
  na_values = c(
    "do_not_know" = 99997,
    "inap" = 99999
  ),
  id = "survey2"
)
```

For a single vector, you can use the `concatenate()` function, which, under the hood, calls the `vctrs::vec_c` method with some additional validation.

```{r}
vctrs::vec_c(h1, h2)
```

## Binding surveys together

As soon as you have only compatible variables with matching names in two data frames, you can bind them together in a way that their history is preserved.  You can do this with `vctrs::vec_rbind` or `dplyr::bind_rows()`. The generic `rbind()` will lose the labelling information.

```{r}
a <- tibble::tibble(
  rowid = paste0("survey1", 1:length(h1)),
  hvar = h1,
  w = runif(n = length(h1), 0, 1)
)
b <- tibble::tibble(
  rowid = paste0("survey2", 1:length(h2)),
  hvar = h2,
  w = runif(n = length(h2), 0, 1)
)

c <- dplyr::bind_rows(a, b)
```

```{r}
summary(c)
```

```{r}
print(c)
```

While dplyr's join functions may result in correct values, the metadata get lost. A new join method will be developed.

# print method works

    <labelled_spss_survey<double>[3]>: Question text
    1 2 9
    Missing values: 9
    See all attributes survey1_name, survey1_values, survey1_label [...], survey1_na_values with attributes(x)

# pillar shaft works

    <pillar_ornament>
    # A tibble: 10 x 1
       v1             
       <retroh_int>   
     1  1 [Good]      
     2  2             
     3  3             
     4  4             
     5  5             
     6  6             
     7  7             
     8  8 [Bad]       
     9  9 (NA)        
    10 10 (NA)        

# pillar display works

    <pillar>
    <retroh_int>
     1 [Good]   
     2          
     3          
     4          
     5          
     6          
     7          
     8 [Bad]    
     9 (NA)     
    10 (NA)     

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{convert_to_labelled_spss}
\alias{convert_to_labelled_spss}
\title{Convert to haven_labelled_spss}
\usage{
convert_to_labelled_spss(x, na_labels = NULL)
}
\arguments{
\item{x}{A vector}

\item{na_labels}{A named vector of missing values, defaults to
\code{c( "inap" = "inap")} for character vectors and
\code{c( 99999 = "inap")} for numeric vectors.}
}
\value{
A haven_labelled_spss vector
}
\description{
Convert to haven_labelled_spss
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosswalk.R
\name{crosswalk_surveys}
\alias{crosswalk_surveys}
\alias{crosswalk}
\title{Crosswalk and harmonize surveys}
\usage{
crosswalk_surveys(
  crosswalk_table,
  survey_list = NULL,
  survey_paths = NULL,
  import_path = NULL,
  na_values = NULL
)

crosswalk(survey_list, crosswalk_table, na_values = NULL)
}
\arguments{
\item{crosswalk_table}{A crosswalk table created with
  [crosswalk_table_create()] or a data frame containing at least the columns
  `id`, `var_name_orig`, and `var_name_target`.

  If the columns `val_label_orig` and `val_label_target` are present,
  value labels are harmonized. If `val_numeric_orig` and
  `val_numeric_target` are present, numeric codes are harmonized.
  If `class_target` is present, variables are coerced to the specified
  target class (`"factor"`, `"numeric"`, or `"character"`) using
  [as_factor()], [as_numeric()], or [as_character()].}

\item{survey_list}{A list of survey objects to be harmonized.}

\item{survey_paths}{Optional character vector of file paths to surveys.
Used when surveys must be read from disk before harmonization.}

\item{import_path}{Optional base directory used to resolve `survey_paths`.
This is primarily intended for workflows where surveys are stored outside
the current working directory.}

\item{na_values}{Optional named vector defining numeric codes to be treated
as missing values. Names correspond to missing-value labels.}
}
\value{
`crosswalk_surveys()` returns a list of harmonized survey data frames.
`crosswalk()` returns either a single data frame (if only one survey is
harmonized) or a merged data frame combining all harmonized surveys.
}
\description{
Harmonize one or more surveys using a crosswalk table that defines how
variable names, value labels, numeric codes, and variable classes should
be aligned across surveys.
}
\details{
A crosswalk table can be created with [crosswalk_table_create()] or supplied
manually as a data frame. At a minimum, the table must contain columns
`id`, `var_name_orig`, and `var_name_target`. Additional columns enable
harmonization of value labels, numeric codes, missing values, and variable
classes.
}
\examples{
\dontrun{
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir, pattern = "\\\\.rds$")

surveys <- read_surveys(
  file.path(examples_dir, survey_files),
  save_to_rds = FALSE
)

metadata <- metadata_create(survey_list = surveys)

crosswalk_table <- crosswalk_table_create(metadata)

harmonized <- crosswalk_surveys(
  crosswalk_table = crosswalk_table,
  survey_list = surveys
)
}

}
\seealso{
[crosswalk_table_create()] to create a crosswalk table,
[harmonize_survey_variables()] for lower-level variable harmonization.

Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_surveys.R
\name{read_surveys}
\alias{read_surveys}
\alias{read_survey}
\title{Read survey files into memory or save as `.rds`}
\usage{
read_surveys(
  survey_paths,
  .f = NULL,
  export_path = NULL,
  ids = NULL,
  dois = NULL,
  ...
)

read_survey(
  file_path,
  .f = NULL,
  export_path = NULL,
  doi = NULL,
  id = NULL,
  ...
)
}
\arguments{
\item{survey_paths}{A character vector containing full or relative
paths to survey files.}

\item{.f}{Import function to use. When `NULL`, the appropriate
  import function is selected automatically from the file
  extension.

  Supported formats are:

  \describe{
    \item{`.sav`, `.por`}{[read_spss()]}
    \item{`.dta`}{[read_dta()]}
    \item{`.rds`}{[read_rds()]}
    \item{`.csv`}{[read_csv()]}
  }}

\item{export_path}{Optional path where imported surveys should
be saved as `.rds` files. Defaults to `NULL`.}

\item{ids}{Optional survey identifiers.}

\item{dois}{Optional DOI identifiers for the imported surveys.}

\item{...}{Additional arguments passed to the import function.}
}
\value{
If `export_path = NULL`, a list of imported survey objects.

If `export_path` is provided, a character vector containing
exported `.rds` file names.

Imported surveys are returned as data frame-like
[survey()] objects with metadata attributes retained for
reproducible workflows.
}
\description{
Import one or more survey files into R using a consistent survey
import workflow. The function supports SPSS (`.sav`, `.por`),
Stata (`.dta`), R (`.rds`), and CSV files.
}
\details{
Use [read_survey()] to import a single survey file and
`read_surveys()` to import multiple files in a loop.

When `export_path` is `NULL`, imported surveys are returned as
a list in memory. When `export_path` is a valid directory,
imported surveys are saved as `.rds` files with
[base::saveRDS()].

Files that cannot be imported are skipped gracefully. A message
is printed and `NULL` is returned for the affected file.
}
\examples{
file1 <- system.file(
  "examples",
  "ZA7576.rds",
  package = "retroharmonize"
)

file2 <- system.file(
  "examples",
  "ZA5913.rds",
  package = "retroharmonize"
)

surveys <- read_surveys(
  c(file1, file2),
  .f = "read_rds"
)

}
\seealso{
[read_survey()], [survey()]

Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_spss]{read_spss()}}
}
\concept{import functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_create.R
\name{metadata_initialize}
\alias{metadata_initialize}
\title{Initialize a metadata data frame}
\usage{
metadata_initialize(filename, id)
}
\arguments{
\item{filename}{A file name}

\item{id}{An id.}
}
\value{
A nested data frame with metadata and the range of
labels, na_values and the na_range itself.
}
\description{
Initialize a metadata data frame
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labelled_spss_survey_arithmetic.R
\name{vec_convert_na}
\alias{vec_convert_na}
\title{Convert labelled missing values to NA}
\usage{
vec_convert_na(x)
}
\arguments{
\item{x}{A labelled survey vector.}
}
\value{
A numeric vector with missing values converted to `NA`.
}
\description{
Internal helper used by numeric summary methods to replace
SPSS-style missing values with `NA`.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_harmonize.R
\name{label_normalize}
\alias{label_normalize}
\alias{var_label_normalize}
\alias{val_label_normalize}
\title{Normalize value and variable labels}
\usage{
label_normalize(x)

var_label_normalize(x)

val_label_normalize(x)
}
\arguments{
\item{x}{A character vector of labels to be normalized.}
}
\value{
Returns a suggested, normalized label without special characters. The
\code{var_label_normalize} and \code{val_label_normalize} returns them in
\code{snake_case} for programmatic use.
}
\description{
\code{label_normalize} removes special characters, whitespace,
and other typical typing errors.
}
\details{
\code{var_label_normalize} and \code{val_label_normalize} removes possible
chunks from question identifiers.

The functions \code{var_label_normalize} and
\code{val_label_normalize} may
be differently implemented for various survey series.
}
\examples{
label_normalize(
  c(
    "Don't know", " TRUST", "DO NOT  TRUST",
    "inap in Q.3", "Not 100\%", "TRUST < 50\%",
    "TRUST >=90\%", "Verify & Check", "TRUST 99\%+"
  )
)

var_label_normalize(
  c(
    "Q1_Do you trust the national government?",
    " Do you trust the European Commission"
  )
)

val_label_normalize(
  c(
    "Q1_Do you trust the national government?",
    " Do you trust the European Commission"
  )
)
}
\seealso{
Other variable label harmonization functions:
\code{\link[=na_range_to_values]{na_range_to_values()}}

Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}}
}
\concept{harmonization functions}
\concept{variable label harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retroharmonize-package.R
\docType{package}
\name{retroharmonize-package}
\alias{retroharmonize-package}
\title{retroharmonize: Ex Post Survey Data Harmonization}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Assist in reproducible retrospective (ex-post) harmonization of data, particularly individual level survey data, by providing tools for organizing metadata, standardizing the coding of variables, and variable names and value labels, including missing values, and documenting the data transformations, with the help of comprehensive s3 classes.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://retroharmonize.dataobservatory.eu/}
  \item Report bugs at \url{https://github.com/dataobservatory-eu/retroharmonize/issues}
}

}
\author{
\strong{Maintainer}: Daniel Antal \email{daniel.antal@dataobservatory.eu} (\href{https://orcid.org/0000-0001-7513-6760}{ORCID})

Authors:
\itemize{
  \item Daniel Antal \email{daniel.antal@dataobservatory.eu} (\href{https://orcid.org/0000-0001-7513-6760}{ORCID})
}

Other contributors:
\itemize{
  \item Marta Kolczynska \email{mkolczynska@gmail.com} (\href{https://orcid.org/0000-0003-4981-0437}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_rds.R
\name{read_rds}
\alias{read_rds}
\title{Read a survey from an `.rds` file}
\usage{
read_rds(file, dataset_bibentry = NULL, id = NULL, doi = NULL)
}
\arguments{
\item{file}{Path to an `.rds` file containing a survey object.}

\item{dataset_bibentry}{Optional bibliographic metadata created with
[dataset::dublincore()] or [dataset::datacite()].}

\item{id}{Optional survey identifier. Defaults to the file name
without extension.}

\item{doi}{Optional DOI identifier for the survey.}
}
\value{
A `survey` object inheriting from `data.frame` and `tbl_df`
  with survey metadata attributes.
}
\description{
Import a serialized survey object stored in `.rds` format and return
it as a `survey` object with harmonized metadata attributes.
}
\details{
This function restores survey objects previously saved with
[base::saveRDS()] or exported from the `retroharmonize`
workflow. The returned object retains survey metadata and gains
additional provenance attributes such as source file name and file size.


If the file cannot be read, an empty `survey` object is returned
and a warning is emitted.

The function:

- restores the serialized object,
- validates source file information,
- normalizes `rowid`,
- records provenance metadata,
- and stores object and source file sizes as attributes.
}
\examples{
path <- system.file(
  "examples",
  "ZA7576.rds",
  package = "retroharmonize"
)

survey_object <- read_rds(path)

attr(survey_object, "id")
attr(survey_object, "filename")
attr(survey_object, "doi")

}
\seealso{
Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_spss]{read_spss()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_create.R
\name{metadata_create}
\alias{metadata_create}
\alias{metadata_waves_create}
\title{Create metadata tables from survey datasets}
\usage{
metadata_create(survey_list = NULL, survey_paths = NULL, .f = NULL)

metadata_waves_create(survey_list)
}
\arguments{
\item{survey_list}{Optional list of survey objects of class
[survey()].}

\item{survey_paths}{Optional character vector containing paths to
survey files.}

\item{.f}{Import function used to read surveys from
`survey_paths`. When `NULL`, the import function is inferred from
the file extension.}
}
\value{
A data frame containing variable-level survey metadata.
}
\description{
Create a variable-level metadata table from one or more survey
datasets. Metadata are extracted either from survey objects already
loaded into memory or directly from survey files.
}
\details{
The resulting metadata table contains information about:

\itemize{
  \item variable names and labels,
  \item storage classes,
  \item value labels,
  \item user-defined missing values,
  \item and missing value ranges.
}

`metadata_create()` is a convenience wrapper around repeated
[metadata_survey_create()] calls.

The form \code{metadata_waves_create} is deprecated.
}
\examples{
examples_dir <- system.file(
  "examples",
  package = "retroharmonize"
)

my_rds_files <- dir(examples_dir)[grepl(
  "\\\\.rds$",
  dir(examples_dir)
)]

example_surveys <- read_surveys(
  file.path(examples_dir, my_rds_files)
)

metadata_create(example_surveys)

}
\seealso{
[metadata_survey_create()], [create_variable_catalog()]

Other metadata functions:
\code{\link[=create_codebook]{create_codebook()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=metadata_survey_create]{metadata_survey_create()}}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na_range_to_values.R
\name{is.na_range_to_values}
\alias{is.na_range_to_values}
\title{Test whether missing values need harmonization}
\usage{
is.na_range_to_values(x)
}
\arguments{
\item{x}{A labelled vector.}
}
\value{
Logical scalar.
}
\description{
Checks whether both `na_values` and `na_range` attributes
are present on a labelled vector.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_surveys.R
\name{merge_surveys}
\alias{merge_surveys}
\title{Merge and harmonize surveys}
\usage{
merge_surveys(survey_list, var_harmonization)
}
\arguments{
\item{survey_list}{A list of survey objects.}

\item{var_harmonization}{A metadata table describing the
  harmonization rules. The table must contain at least:

  - `filename`
  - `var_name_orig`
  - `var_name_target`
  - `var_label`}
}
\value{
A list of harmonized survey objects with standardized variable names
and variable labels.
}
\description{
`merge_surveys()` applies a harmonization specification to a list
of survey objects and returns harmonized survey datasets with
aligned variable names and metadata.
}
\details{
Harmonize variable names, labels, and identifiers across multiple
surveys using a metadata crosswalk table.


Prior to version 0.2.0 this function was called `merge_waves()`,
reflecting terminology commonly used in Eurobarometer surveys.

The harmonization table supplied in `var_harmonization`
typically originates from [metadata_create()] and contains
mappings between original and harmonized variable names.
}
\examples{
\donttest{

examples_dir <- system.file(
  "examples",
  package = "retroharmonize"
)

survey_files <- dir(
  examples_dir,
  pattern = "\\\\.rds$",
  full.names = TRUE
)

example_surveys <- read_surveys(
  survey_files
)

metadata <- metadata_create(
  survey_list = example_surveys
)

to_harmonize <- metadata \%>\%
  dplyr::filter(
    var_name_orig \%in\% c("rowid", "w1") |
      grepl("^trust", var_label_orig)
  ) \%>\%
  dplyr::mutate(
    var_label = var_label_normalize(var_label_orig),
    var_name_target = val_label_normalize(var_label),
    var_name_target = ifelse(
      .data$var_name_orig \%in\%
        c("rowid", "w1", "wex"),
      .data$var_name_orig,
      .data$var_name_target
    )
  )

merged_surveys <- merge_surveys(
  survey_list = example_surveys,
  var_harmonization = to_harmonize
)

merged_surveys[[1]]
}

}
\seealso{
[metadata_create()]

Other survey harmonization functions:
\code{\link[=merge_waves]{merge_waves()}}
}
\concept{survey harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spss.R
\name{read_spss}
\alias{read_spss}
\title{Read SPSS survey files}
\usage{
read_spss(
  file,
  user_na = TRUE,
  dataset_bibentry = NULL,
  id = NULL,
  doi = NULL,
  .name_repair = "unique"
)
}
\arguments{
\item{file}{Path to an SPSS survey file.}

\item{user_na}{Logical. Should user-defined missing values be imported?
Defaults to `TRUE`.}

\item{dataset_bibentry}{Optional bibliographic metadata created with
[dataset::dublincore()] or [dataset::datacite()].}

\item{id}{Optional survey identifier. Defaults to the file name
without extension.}

\item{doi}{Optional DOI identifier.}

\item{.name_repair}{Strategy for repairing invalid or duplicated
column names. Passed to [haven::read_spss()].}
}
\value{
A `survey` object inheriting from `data.frame` and `tbl_df`.

Variable labels are stored in the `"label"` attribute of each variable.

Additional provenance metadata are stored as attributes, including:

- `"id"`
- `"doi"`
- `"object_size"`
- `"source_file_size"`
}
\description{
Import SPSS survey files in `.sav`, `.zsav`, or `.por` format and
convert them into harmonized `survey` objects with preserved metadata,
labelled variables, and provenance information.
}
\details{
This function wraps [haven::read_spss()] and adds:

- error handling,
- harmonized survey metadata,
- `rowid` creation and normalization,
- preservation of variable labels,
- conversion of labelled SPSS vectors,
- handling of malformed labelled variables,
- and provenance metadata.


`read_sav()` reads both `.sav` and `.zsav` files.
`read_por()` reads portable SPSS `.por` files.
`read_spss()` automatically dispatches to the appropriate importer
based on file extension.

Variables that inherit from `haven_labelled` but do not contain
valid label definitions are converted to standard numeric or
character vectors.

If a file cannot be imported, the function returns an empty
`survey` object and emits a warning.
}
\examples{
\donttest{
path <- system.file(
  "examples",
  "iris.sav",
  package = "haven"
)

survey_object <- read_spss(path)

attr(survey_object, "id")
attr(survey_object, "filename")
}

}
\seealso{
Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset_surveys.R
\name{subset_surveys}
\alias{subset_surveys}
\alias{subset_waves}
\alias{subset_save_surveys}
\title{Subset and optionally harmonize surveys}
\usage{
subset_surveys(
  survey_list,
  survey_paths = NULL,
  rowid = "rowid",
  subset_name = "subset",
  subset_vars = NULL,
  crosswalk_table = NULL,
  import_path = NULL,
  export_path = NULL
)

subset_waves(waves, subset_vars = NULL)

subset_save_surveys(
  crosswalk_table,
  subset_name = "subset",
  survey_list = NULL,
  subset_vars = NULL,
  survey_paths = NULL,
  import_path = NULL,
  export_path = NULL
)
}
\arguments{
\item{survey_list}{A list of survey objects created by
[read_surveys()]. If `NULL`, surveys are read from disk.}

\item{survey_paths}{A character vector of full file paths to survey files.
Used when `survey_list` is `NULL`.}

\item{rowid}{Name of the unique observation identifier column.
Defaults to `"rowid"`.}

\item{subset_name}{Character string appended to filenames of
subsetted surveys. Defaults to `"subset"`.}

\item{subset_vars}{Character vector of variable names to retain.
If `NULL`, all variables are retained.}

\item{crosswalk_table}{Optional crosswalk table created with
[crosswalk_table_create()]. If supplied, variables are selected
per survey based on `var_name_orig`, and variable names may be
harmonized using `var_name_target`.}

\item{import_path}{Optional directory containing survey files.
Used to resolve filenames when subsetting from disk.}

\item{export_path}{Optional directory where subsetted surveys
are saved as `.rds` files. If `NULL`, surveys are returned in memory.}

\item{waves}{A list of surveys imported with [read_surveys()].}
}
\value{
Either:
* a list of subsetted survey objects (if `export_path = NULL`), or
* a character vector of filenames written to `export_path`.
}
\description{
Subset one or more surveys by retaining a specified set of variables.
Subsetting can be performed either on surveys already loaded in memory
or directly from survey files on disk.

If a crosswalk table is supplied, variables are selected based on the
variables listed for each survey in the crosswalk, and variable names
can optionally be harmonized using `var_name_target`.

This function replaces the deprecated helpers
[subset_waves()] and [subset_save_surveys()].
}
\details{
The function supports multiple workflows:

* **In-memory subsetting** using `survey_list`
* **File-based subsetting** using `survey_paths` or `import_path`
* **Crosswalk-driven subsetting**, where variables are selected
  per survey using a crosswalk table created by
  [crosswalk_table_create()]

If `export_path` is provided, subsetted surveys are written to disk
as `.rds` files. Otherwise, subsetted surveys are returned in memory.
}
\examples{
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir, pattern = "\\\\.rds$")

surveys <- read_surveys(
  file.path(examples_dir, survey_files),
  export_path = NULL
)

subset_surveys(
  survey_list = surveys,
  subset_vars = c("rowid", "isocntry", "qa10_1", "qa14_1"),
  subset_name = "example_subset"
)

}
\seealso{
[crosswalk_table_create()],
[harmonize_survey_variables()],
[read_surveys()]
}
\concept{subsetting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{validate_harmonize_labels}
\alias{validate_harmonize_labels}
\title{Validate harmonize_labels parameter
Check if "from", "to", and "numeric_values" are of equal lengths.}
\usage{
validate_harmonize_labels(harmonize_labels)
}
\description{
Validate harmonize_labels parameter
Check if "from", "to", and "numeric_values" are of equal lengths.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concatenate.R
\name{concatenate}
\alias{concatenate}
\title{Concatenate haven_labelled_spss vectors}
\usage{
concatenate(x, y)
}
\arguments{
\item{x}{A haven_labelled_spss vector.}

\item{y}{A haven_labelled_spss vector.}
}
\value{
A concatenated haven_labelled_spss vector. Returns an error
if the attributes do not match. Gives a warning when only the variable
label do not match.
}
\description{
Concatenate haven_labelled_spss vectors
}
\examples{
v1 <- labelled::labelled(
  c(3, 4, 4, 3, 8, 9),
  c(YES = 3, NO = 4, `WRONG LABEL` = 8, REFUSED = 9)
)
v2 <- labelled::labelled(
  c(4, 3, 3, 9),
  c(YES = 3, NO = 4, `WRONG LABEL` = 8, REFUSED = 9)
)
s1 <- haven::labelled_spss(
  x = unclass(v1), # remove labels from earlier defined
  labels = labelled::val_labels(v1), # use the labels from earlier defined
  na_values = NULL,
  na_range = 8:9,
  label = "Variable Example"
)

s2 <- haven::labelled_spss(
  x = unclass(v2), # remove labels from earlier defined
  labels = labelled::val_labels(v2), # use the labels from earlier defined
  na_values = NULL,
  na_range = 8:9,
  label = "Variable Example"
)
concatenate(s1, s2)
}
\concept{joining functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/document_survey_item.R
\name{document_survey_item}
\alias{document_survey_item}
\title{Document survey item provenance}
\usage{
document_survey_item(x)
}
\arguments{
\item{x}{A `labelled_spss_survey` vector originating from a single
survey or concatenated from multiple surveys.}
}
\value{
A named list containing:
\itemize{
  \item current and historical value coding,
  \item variable labels,
  \item valid and missing value definitions,
  \item original variable names,
  \item survey identifiers.
}
}
\description{
Document the current and historical coding, labels, missing values,
and survey provenance of a harmonized survey variable.
}
\examples{
var1 <- labelled::labelled_spss(
  x = c(1, 0, 1, 1, 0, 8, 9),
  labels = c(
    "TRUST" = 1,
    "NOT TRUST" = 0,
    "DON'T KNOW" = 8,
    "INAP. HERE" = 9
  ),
  na_values = c(8, 9)
)

var2 <- labelled::labelled_spss(
  x = c(2, 2, 8, 9, 1, 1),
  labels = c(
    "Tend to trust" = 1,
    "Tend not to trust" = 2,
    "DK" = 8,
    "Inap" = 9
  ),
  na_values = c(8, 9)
)

harmonization <- list(
  from = c(
    "^tend\\\\sto|^trust",
    "^tend\\\\snot|not\\\\strust",
    "^dk|^don",
    "^inap"
  ),
  to = c(
    "trust",
    "not_trust",
    "do_not_know",
    "inap"
  ),
  numeric_values = c(1, 0, 99997, 99999)
)

missing_values <- c(
  "do_not_know" = 99997,
  "inap" = 99999
)

h1 <- harmonize_values(
  x = var1,
  harmonize_label = "Do you trust the European Union?",
  harmonize_labels = harmonization,
  na_values = missing_values,
  id = "survey1"
)

h2 <- harmonize_values(
  x = var2,
  harmonize_label = "Do you trust the European Union?",
  harmonize_labels = harmonization,
  na_values = missing_values,
  id = "survey2"
)

h3 <- concatenate(h1, h2)

document_survey_item(h3)

}
\seealso{
Other documentation functions:
\code{\link[=document_surveys]{document_surveys()}}
}
\concept{documentation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labelled_spss_survey-print.R,
%   R/labelled_spss_survey.R, R/labelled_spss_survey_arithmetic.R
\name{print.retroharmonize_labelled_spss_survey}
\alias{print.retroharmonize_labelled_spss_survey}
\alias{labelled_spss_survey}
\alias{[.retroharmonize_labelled_spss_survey}
\alias{summary.retroharmonize_labelled_spss_survey}
\alias{names<-.retroharmonize_labelled_spss_survey}
\alias{is.na.retroharmonize_labelled_spss_survey}
\alias{levels.retroharmonize_labelled_spss_survey}
\alias{format.retroharmonize_labelled_spss_survey}
\alias{is.labelled_spss_survey}
\alias{median.retroharmonize_labelled_spss_survey}
\alias{quantile.retroharmonize_labelled_spss_survey}
\alias{weighted.mean.retroharmonize_labelled_spss_survey}
\alias{mean.retroharmonize_labelled_spss_survey}
\alias{sum.retroharmonize_labelled_spss_survey}
\title{Labelled SPSS-style vectors with survey provenance}
\usage{
\method{print}{retroharmonize_labelled_spss_survey}(x, ...)

labelled_spss_survey(
  x = double(),
  labels = NULL,
  na_values = NULL,
  na_range = NULL,
  label = NULL,
  id = NULL,
  name_orig = NULL
)

\method{[}{retroharmonize_labelled_spss_survey}(x, i, ...)

\method{summary}{retroharmonize_labelled_spss_survey}(object, ...)

\method{names}{retroharmonize_labelled_spss_survey}(x) <- value

\method{is.na}{retroharmonize_labelled_spss_survey}(x)

\method{levels}{retroharmonize_labelled_spss_survey}(x)

\method{format}{retroharmonize_labelled_spss_survey}(x, ..., digits = getOption("digits"))

is.labelled_spss_survey(x)

\method{median}{retroharmonize_labelled_spss_survey}(x, na.rm = TRUE, ...)

\method{quantile}{retroharmonize_labelled_spss_survey}(x, probs, ...)

\method{weighted.mean}{retroharmonize_labelled_spss_survey}(x, w, ...)

\method{mean}{retroharmonize_labelled_spss_survey}(x, ...)

\method{sum}{retroharmonize_labelled_spss_survey}(x, ...)
}
\arguments{
\item{x}{A vector of values.}

\item{...}{potentially further arguments for methods; not used in
    the default method.}

\item{labels}{A named vector of value labels.}

\item{na_values}{A vector of values to be treated as missing.}

\item{na_range}{A numeric range defining missing values.}

\item{label}{A variable label.}

\item{id}{A character scalar identifying the survey.}

\item{name_orig}{Original variable name. Defaults to the name of `x`.}

\item{i}{Index vector used for subsetting.}

\item{object}{A labelled_spss_survey to summarize.}

\item{value}{Replacement values used when assigning names.}

\item{digits}{Number of digits to use in string representation in
the format method.}

\item{na.rm}{a logical value indicating whether \code{NA}
    values should be stripped before the computation proceeds.}

\item{probs}{numeric vector of probabilities with values in
    \eqn{[0,1]}.  (Values up to \samp{2e-14} outside that
    range are accepted and moved to the nearby endpoint.)}

\item{w}{a numerical vector of weights the same length as \code{x} giving
    the weights to use for elements of \code{x}.}
}
\value{
An object of class `"retroharmonize_labelled_spss_survey"`,
extending [haven::labelled_spss()].
}
\description{
Create a labelled vector compatible with
[haven::labelled_spss()] that carries additional
survey-level provenance metadata.
}
\details{
The resulting object behaves like a
`haven_labelled_spss` vector, but stores:
\itemize{
  \item a survey identifier;
  \item the original variable name;
  \item the original value coding.
}


Several arithmetic and statistical summary methods operate on
the numeric representation of labelled survey vectors,
converting SPSS-style missing values to `NA` before computation.

You can coerce `labelled_spss_survey` vectors to numeric,
character or factor representation.
}
\examples{
x <- labelled_spss_survey(
  x = c(1, 2, 9),
  labels = c(Yes = 1, No = 2),
  na_values = 9,
  id = "survey_1"
)

is.na(x)
as_factor(x)

}
\seealso{
[haven::labelled_spss()],
[as_factor()],
[as_numeric()],
[as_character()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_surveys.R
\name{find_import_function}
\alias{find_import_function}
\title{Find import function by file extension}
\usage{
find_import_function(file_path)
}
\value{
The name of the function that should read \code{file_path} based on the file
extension.
}
\description{
This is an internal utility to select the appropriate importing function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_csv.R
\name{read_csv}
\alias{read_csv}
\title{Read a survey dataset from a CSV file}
\usage{
read_csv(file, id = NULL, doi = NULL, dataset_bibentry = NULL, ...)
}
\arguments{
\item{file}{Path to a `.csv` file.}

\item{id}{Optional dataset identifier. When omitted, the file name
without extension is used.}

\item{doi}{Optional dataset DOI identifier.}

\item{dataset_bibentry}{Optional bibliographic metadata created
with [dataset::dublincore()] or [dataset::datacite()].}

\item{...}{Additional arguments passed to
[utils::read.csv()].}
}
\value{
A tibble-like survey object with metadata attributes
  retained for reproducible workflows.
}
\description{
Import a survey dataset stored in comma-separated value (`.csv`)
format and convert it into a survey-compatible tibble with
reproducibility metadata retained as attributes.
}
\details{
The imported object is returned as a tibble with additional survey
metadata such as identifiers, DOI references, and optional dataset
bibliographic metadata.
}
\examples{
# Create a temporary CSV file:
path <- system.file(
  "examples",
  "ZA7576.rds",
  package = "retroharmonize"
)

read_survey <- read_rds(path)

test_csv_file <- tempfile(fileext = ".csv")

write.csv(
  x = read_survey,
  file = test_csv_file,
  row.names = FALSE
)

# Read the CSV file:
re_read <- read_csv(
  file = test_csv_file,
  id = "ZA7576",
  doi = "test_doi"
)

}
\seealso{
Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_spss]{read_spss()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey.R
\name{survey}
\alias{survey}
\alias{is.survey}
\alias{summary.survey}
\title{Create a survey data frame}
\usage{
survey(object = data.frame(), id = "survey_id", filename = NULL, doi = NULL)

is.survey(object)

\method{summary}{survey}(object, ...)
}
\arguments{
\item{object}{A tibble or data frame that contains the survey data.}

\item{id}{A mandatory identifier for the survey.}

\item{filename}{The import file name.}

\item{doi}{Optional document object identifier (doi), can be omitted.}

\item{...}{Arguments passed to summary method.}
}
\value{
A tibble with \code{id},  \code{filename},  \code{doi}
metadata information.
}
\description{
Store the data of a survey in a tibble (data frame) with a unique
survey identifier, import filename, and optional document object identifier.
}
\details{
Whilst you can create a survey object with this helper function, it is most likely that
you will receive it with an importing function, i.e.
\code{\link{read_rds}}, \code{\link{read_spss}} \code{\link{read_dta}}, \code{\link{read_csv}} or
their common wrapper \code{\link{read_survey}}.
}
\examples{
example_survey <- survey(
  object = data.frame(
    rowid = 1:6,
    observations = runif(6)
  ),
  id = "example",
  filename = "no_file"
)
}
\seealso{
Other importing functions:
\code{\link[=is.survey_df]{is.survey_df()}}
}
\concept{importing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset_surveys.R
\name{subset_survey_file}
\alias{subset_survey_file}
\title{Subset surveys from files}
\usage{
subset_survey_file(
  file_path,
  subset_vars,
  subset_name = "subset",
  id = NULL,
  export_path = NULL
)
}
\arguments{
\item{file_path}{A single survey files.}

\item{subset_vars}{Character vector of variable names to retain.
If `NULL`, all variables are retained.}

\item{subset_name}{Character string appended to filenames of
subsetted surveys. Defaults to `"subset"`.}

\item{export_path}{Optional directory where subsetted surveys
are saved as `.rds` files. If `NULL`, surveys are returned in memory.}
}
\description{
Subset surveys from files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_survey_values.R
\name{harmonize_survey_values}
\alias{harmonize_survey_values}
\alias{harmonize_waves}
\title{Harmonize values in surveys}
\usage{
harmonize_survey_values(survey_list, .f, status_message = FALSE)

harmonize_waves(waves, .f, status_message = FALSE)
}
\arguments{
\item{survey_list}{A list of surveys (data frames). In earlier versions this argument was called
\code{waves}.}

\item{.f}{A function applied to each labelled variable
(class \code{"retroharmonize_labelled_spss_survey"}). The function must not
change the length of the input vector.}

\item{status_message}{Logical. If \code{TRUE}, prints the identifier of each survey as it is
processed.}

\item{waves}{A list of surveys. Deprecated.}
}
\value{
A data frame containing the row-wise combination of all surveys, with
harmonized labelled variables and preserved attributes describing the
original surveys.
}
\description{
Harmonize value codes and value labels across multiple surveys and
combine them into a single data frame.
}
\details{
The function first aligns the structure of all surveys by ensuring that
they contain the same set of variables. Missing variables are added and
filled with appropriate missing values depending on their type.

Variables of class \code{"retroharmonize_labelled_spss_survey"} are then
harmonized by applying a user-supplied function \code{.f} to each variable
separately within each survey.

The harmonization function \code{.f} must return a vector of the same length
as its input. If \code{.f} returns \code{NULL}, the original variable is kept
unchanged.

Prior to version 0.2.0 this function was called \code{harmonize_waves}.

The earlier form \code{harmonize_waves} is deprecated.
The function is currently called \code{\link{harmonize_waves}}.
}
\examples{
\donttest{
examples_dir <- system.file("examples", package = "retroharmonize")
survey_files <- dir(examples_dir, pattern = "\\\\.rds$", full.names = TRUE)

surveys <- read_surveys(
  survey_files,
  export_path = NULL
)

# Keep only supported variable types
surveys <- lapply(
  surveys,
  function(s) {
    s[, vapply(
      s,
      function(x) {
        inherits(x, c(
          "retroharmonize_labelled_spss_survey",
          "numeric",
          "character",
          "Date"
        ))
      },
      logical(1)
    )]
  }
)

# Identity harmonization (no-op)
harmonized <- harmonize_survey_values(
  survey_list = surveys,
  .f = function(x) x,
  status_message = FALSE
)

head(harmonized)
}
}
\seealso{
Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_create.R
\name{metadata_survey_create}
\alias{metadata_survey_create}
\title{Create variable-level metadata from a survey dataset}
\usage{
metadata_survey_create(survey)
}
\arguments{
\item{survey}{A survey object of class [survey()].

  Survey objects are typically created with:

  \itemize{
    \item [read_rds()]
    \item [read_spss()]
    \item [read_dta()]
    \item [read_csv()]
    \item [read_survey()]
  }

  Survey objects can also be created manually from a data frame
  with [survey()].}
}
\value{
A nested data frame containing:

\describe{
  \item{filename}{Original survey file name.}
  \item{id}{Survey identifier.}
  \item{var_name_orig}{Original variable name.}
  \item{class_orig}{Imported storage class.}
  \item{var_label_orig}{Original variable label.}
  \item{labels}{List column of value labels.}
  \item{valid_labels}{List column of non-missing value labels.}
  \item{na_labels}{List column of user-defined missing labels.}
  \item{na_range}{List column containing user-defined missing ranges.}
  \item{n_labels}{Number of labelled categories.}
  \item{n_valid_labels}{Number of non-missing categories.}
  \item{n_na_labels}{Number of missing categories.}
}
}
\description{
Extract variable-level metadata from a survey dataset and return
the result as a nested data frame.
}
\details{
The metadata table contains:

\itemize{
  \item variable names and labels,
  \item imported storage classes,
  \item value labels,
  \item user-defined missing values,
  \item missing value ranges,
  \item and summary counts of labelled categories.
}

For multiple surveys, use [metadata_create()], which applies
`metadata_survey_create()` across a list of surveys or survey files.
}
\examples{
metadata_survey_create(
  survey = read_rds(
    system.file(
      "examples",
      "ZA7576.rds",
      package = "retroharmonize"
    )
  )
)

}
\seealso{
[metadata_create()], [create_variable_catalog()]

Other metadata functions:
\code{\link[=create_codebook]{create_codebook()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=metadata_create]{metadata_create()}}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_var_names.R
\name{harmonize_var_names}
\alias{harmonize_var_names}
\title{Harmonize variable names across surveys}
\usage{
harmonize_var_names(
  survey_list,
  metadata,
  old = "var_name_orig",
  new = "var_name_suggested",
  rowids = TRUE
)
}
\arguments{
\item{survey_list}{A list of survey objects, typically imported
with [read_surveys()].}

\item{metadata}{A metadata table containing harmonization rules.
Typically created with [metadata_create()] and combined across
surveys.}

\item{old}{Name of the column in `metadata` containing
the original variable names.}

\item{new}{Name of the column in `metadata` containing
the harmonized variable names.}

\item{rowids}{Logical. Should original `rowid` variables
be renamed to `"uniqid"`?}
}
\value{
A list of surveys with harmonized variable names.
}
\description{
`harmonize_var_names()` renames variables across multiple
surveys to a shared harmonized naming scheme.

The harmonization rules are defined in a metadata table,
typically created with [metadata_create()].
}
\details{
Harmonize variable names in a list of survey objects using
a metadata crosswalk table.


The function can also be used for survey subsetting workflows.
If `metadata` contains only a subset of variables for a survey,
only those variables are retained in the harmonized output.
}
\examples{
examples_dir <- system.file(
  "examples",
  package = "retroharmonize"
)

survey_files <- dir(
  examples_dir,
  pattern = "\\\\.rds$"
)

example_surveys <- read_surveys(
  file.path(examples_dir, survey_files)
)

metadata <- metadata_create(
  example_surveys
)

metadata$var_name_suggested <-
  label_normalize(metadata$var_name)

metadata$var_name_suggested[
  metadata$label_orig == "age_education"
] <- "age_education"

harmonized_surveys <- harmonize_var_names(
  survey_list = example_surveys,
  metadata = metadata
)

harmonized_surveys[[1]]

}
\seealso{
[metadata_create()],
[crosswalk()]

Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_values.R
\name{harmonize_values}
\alias{harmonize_values}
\title{Harmonize values and labels of labelled vectors}
\usage{
harmonize_values(
  x,
  harmonize_label = NULL,
  harmonize_labels = NULL,
  na_values = c(do_not_know = 99997, declined = 99998, inap = 99999),
  na_range = NULL,
  id = "survey_id",
  name_orig = NULL,
  remove = NULL,
  perl = FALSE
)
}
\arguments{
\item{x}{A labelled vector, typically of class
`"haven_labelled"` or `"haven_labelled_spss"`.}

\item{harmonize_label}{Optional harmonized variable label.
Defaults to the original variable label.}

\item{harmonize_labels}{A list describing harmonization rules.
  Must contain the elements:

  - `from`
  - `to`
  - `numeric_values`}

\item{na_values}{Named numeric vector defining harmonized
missing value codes.}

\item{na_range}{Optional SPSS-style missing value range.
Usually left `NULL`.}

\item{id}{Survey identifier.
Defaults to `"survey_id"`.}

\item{name_orig}{Optional original variable name.
Defaults to the object name supplied to `x`.}

\item{remove}{Optional regex pattern removed from original labels
before harmonization.}

\item{perl}{Logical. Use Perl-compatible regular expressions?
Defaults to `FALSE`.}
}
\value{
A harmonized `haven_labelled_spss` vector.

The returned vector preserves:

- harmonized value labels;
- harmonized numeric coding;
- SPSS missing value metadata;
- original coding metadata;
- survey provenance metadata.
}
\description{
`harmonize_values()` converts heterogeneous labelled survey vectors
into a harmonized representation suitable for cross-survey integration.

The function:

- harmonizes value labels using regex-based matching;
- assigns harmonized numeric codes;
- preserves original coding metadata;
- standardizes user-defined missing values;
- preserves SPSS-style labelled metadata;
- and records provenance attributes.
}
\details{
Create a harmonized labelled vector with standardized value labels,
numeric coding, and missing value definitions.


Harmonization is performed using a harmonization table supplied via
`harmonize_labels`.

The harmonization table must contain:

- `from`: regex patterns matching original labels;
- `to`: harmonized labels;
- `numeric_values`: harmonized numeric codes.

Original labels and numeric codes are preserved in attributes
attached to the returned vector.

If no harmonization table is supplied, the function still attempts
to normalize common missing value labels such as:

- `"inap"`
- `"declined"`
- `"do_not_know"`
}
\examples{
var1 <- labelled::labelled_spss(
  x = c(1, 0, 1, 1, 0, 8, 9),
  labels = c(
    "TRUST" = 1,
    "NOT TRUST" = 0,
    "DON'T KNOW" = 8,
    "INAP. HERE" = 9
  ),
  na_values = c(8, 9)
)

harmonize_values(
  var1,
  harmonize_labels = list(
    from = c(
      "^tend\\\\sto|^trust",
      "^tend\\\\snot|not\\\\strust",
      "^dk|^don",
      "^inap"
    ),
    to = c(
      "trust",
      "not_trust",
      "do_not_know",
      "inap"
    ),
    numeric_values = c(
      1,
      0,
      99997,
      99999
    )
  ),
  na_values = c(
    "do_not_know" = 99997,
    "inap" = 99999
  ),
  id = "survey_id"
)

}
\seealso{
[harmonize_var_names()]

Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-here.R
\name{here}
\alias{here}
\title{Here}
\description{
A utility to make sure the system files of the package and other
files are always found, regardless if they are in an example or
vignette context.
}
\details{
See \code{here::\link[here:here]{here}} for details.
}
\examples{
dir(here("inst", "examples"))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_na_values.R
\name{harmonize_na_values}
\alias{harmonize_na_values}
\title{Harmonize na_values in haven_labelled_spss}
\usage{
harmonize_na_values(df)
}
\arguments{
\item{df}{A data frame that contains haven_labelled_spss vectors.}
}
\value{
A tibble where the na_values are consistent
}
\description{
Harmonize na_values in haven_labelled_spss
}
\examples{
\donttest{
examples_dir <- system.file(
  "examples",
  package = "retroharmonize"
)

test_read <- read_rds(
  file.path(examples_dir, "ZA7576.rds"),
  id = "ZA7576",
  doi = "test_doi"
)

harmonize_na_values(test_read)
}
}
\seealso{
Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labelled_spss_survey-vctrs.R
\name{labelled_spss_survey_vctrs}
\alias{labelled_spss_survey_vctrs}
\alias{vec_ptype2.retroharmonize_labelled_spss_survey.double}
\alias{vec_ptype2.double.retroharmonize_labelled_spss_survey}
\alias{vec_ptype2.integer.retroharmonize_labelled_spss_survey}
\alias{vec_ptype2.retroharmonize_labelled_spss_survey.integer}
\alias{vec_cast.double.retroharmonize_labelled_spss_survey}
\alias{vec_cast.integer.retroharmonize_labelled_spss_survey}
\alias{vec_cast.character.retroharmonize_labelled_spss_survey}
\title{vctrs type and casting methods for labelled survey vectors}
\usage{
\method{vec_ptype2}{retroharmonize_labelled_spss_survey.double}(x, y, ...)

\method{vec_ptype2}{double.retroharmonize_labelled_spss_survey}(x, y, ...)

\method{vec_ptype2}{integer.retroharmonize_labelled_spss_survey}(x, y, ...)

\method{vec_ptype2}{retroharmonize_labelled_spss_survey.integer}(x, y, ...)

\method{vec_cast}{double.retroharmonize_labelled_spss_survey}(x, to, ...)

\method{vec_cast}{integer.retroharmonize_labelled_spss_survey}(x, to, ...)

\method{vec_cast}{character.retroharmonize_labelled_spss_survey}(x, to, ...)
}
\description{
These methods define how
\code{retroharmonize_labelled_spss_survey} objects interact with
base vectors and with each other in vctrs-based operations such as
concatenation, binding, and coercion.
}
\details{
They ensure that labelled survey vectors:
\itemize{
  \item combine safely with numeric vectors,
  \item cast consistently to base types,
  \item error on incompatible coercions.
}

These functions are part of the internal type system and are not
intended to be called directly by users.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labelled_spss_survey_coercion.R
\name{labelled_spss_survey_coercion}
\alias{labelled_spss_survey_coercion}
\alias{as_numeric}
\alias{as_character}
\alias{as_factor}
\title{Coercion methods for labelled survey vectors}
\usage{
as_numeric(x)

as_character(x)

as_factor(x, levels = "default", ordered = FALSE)
}
\arguments{
\item{x}{A labelled survey vector created with
[labelled_spss_survey()].}

\item{levels}{Character string indicating how factor levels
should be constructed. Currently retained for compatibility.}

\item{ordered}{Logical; whether the resulting factor should be ordered.
Currently ignored.}
}
\value{
* `as_numeric()` returns a numeric vector with labelled missing
  values converted to `NA`.
* `as_character()` returns a character vector based on the
  factor representation of `x`.
* `as_factor()` returns a factor with levels derived from value
  labels.
}
\description{
Convert labelled SPSS-style survey vectors to common R data types.
These helpers provide consistent coercion behavior for
`"retroharmonize_labelled_spss_survey"` objects while respecting
labelled missing values.
}
\seealso{
[labelled_spss_survey()],
[haven::as_factor()]

Other type conversion functions:
\code{\link[=as_labelled_spss_survey]{as_labelled_spss_survey()}}
}
\concept{type conversion functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_labelled_spss_survey.R
\name{as_labelled_spss_survey}
\alias{as_labelled_spss_survey}
\title{Labelled to labelled_spss_survey}
\usage{
as_labelled_spss_survey(x, id)
}
\arguments{
\item{x}{A vector of class haven_labelled or haven_labelled_spss.}

\item{id}{The survey identifier.}
}
\value{
A vector of labelled_spss_survey
}
\description{
Labelled to labelled_spss_survey
}
\seealso{
Other type conversion functions:
\code{\link{labelled_spss_survey_coercion}}
}
\concept{type conversion functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collect_val_labels.R
\name{collect_val_labels}
\alias{collect_val_labels}
\alias{collect_na_labels}
\title{Collect labels from metadata file}
\usage{
collect_val_labels(metadata)

collect_na_labels(metadata)
}
\arguments{
\item{metadata}{A metadata data frame created by
\code{\link{metadata_create}}.}
}
\value{
The unique valid labels or the user-defined missing
labels found in all the files analyzed in \code{metadata}.
}
\description{
Collect labels from metadata file
}
\examples{
test_survey <- retroharmonize::read_rds(
  file = system.file("examples", "ZA7576.rds",
    package = "retroharmonize"
  ),
  id = "test"
)
example_metadata <- metadata_create(test_survey)

collect_val_labels(metadata = example_metadata)
collect_na_labels(metadata = example_metadata)
}
\seealso{
Other harmonization functions:
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pull_survey.R
\name{pull_survey}
\alias{pull_survey}
\title{Retrieve a survey from a survey list}
\usage{
pull_survey(survey_list, id = NULL, filename = NULL)
}
\arguments{
\item{survey_list}{A list of `survey` objects.}

\item{id}{Optional survey identifier.}

\item{filename}{Optional source file name.}
}
\value{
A single `survey` object.
}
\description{
`pull_survey()` retrieves a survey object from a list created with
[read_surveys()].

Surveys can be selected using:

- the survey identifier stored in the `"id"` attribute, or
- the original source file name stored in the `"filename"` attribute.
}
\details{
Extract a single `survey` object from a list of surveys using either
its survey identifier or source file name.


Either `id` or `filename` must be supplied.

The function throws an error if:

- neither argument is provided;
- the requested survey cannot be found;
- or multiple surveys match the query.
}
\examples{
examples_dir <- system.file(
  "examples",
  package = "retroharmonize"
)

survey_files <- dir(
  examples_dir,
  pattern = "\\\\.rds$"
)

example_surveys <- read_surveys(
  file.path(examples_dir, survey_files)
)

pull_survey(
  example_surveys,
  id = "ZA5913"
)

}
\seealso{
[read_surveys()]

Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_spss]{read_spss()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_surveys.R
\name{merge_waves}
\alias{merge_waves}
\title{Deprecated wrapper for `merge_surveys()`}
\usage{
merge_waves(waves, var_harmonization)
}
\arguments{
\item{waves}{Deprecated alias for `survey_list`.}

\item{var_harmonization}{A metadata table describing the
  harmonization rules. The table must contain at least:

  - `filename`
  - `var_name_orig`
  - `var_name_target`
  - `var_label`}
}
\value{
A list of harmonized survey objects.
}
\description{
`merge_waves()` has been renamed to [merge_surveys()]
for more general survey harmonization workflows.
}
\seealso{
[merge_surveys()]

Other survey harmonization functions:
\code{\link[=merge_surveys]{merge_surveys()}}
}
\concept{survey harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_survey_variables.R
\name{harmonize_survey_variables}
\alias{harmonize_survey_variables}
\title{Read a survey from a CSV file}
\usage{
harmonize_survey_variables(
  crosswalk_table,
  subset_name = "subset",
  survey_list = NULL,
  survey_paths = NULL,
  import_path = NULL,
  export_path = NULL
)
}
\arguments{
\item{crosswalk_table}{A crosswalk table created with
[crosswalk_table_create()].}

\item{subset_name}{Character string appended to filenames of
subsetted surveys. Defaults to `"subset"`.}

\item{survey_list}{A list containing surveys of class survey.}

\item{survey_paths}{Optional character vector of file paths to surveys.}

\item{import_path}{Optional base directory used to resolve `survey_paths`.}

\item{export_path}{Optional directory where subsetted surveys are
exported to}
}
\value{
An object of class `"survey"`, which is a data frame with attached
survey- and dataset-level metadata.
}
\description{
Import a survey stored in a CSV file and return it as a survey object
with attached dataset- and survey-level metadata.
}
\details{
The CSV file is read using [utils::read.csv()]. Character variables
with more than one unique value are automatically converted to
labelled factors. A unique row identifier is added and labelled.


If the file cannot be read, an empty survey object is returned with
a warning.

If a column named `"X"` is present (commonly created by
`write.csv()`), it is removed automatically.
}
\examples{
# Create a temporary CSV file from an example survey
path <- system.file("examples", "ZA7576.rds",
  package = "retroharmonize"
)
survey <- read_rds(path)

tmp <- tempfile(fileext = ".csv")
write.csv(survey, tmp, row.names = FALSE)

# Read the CSV file back as a survey
re_read <- read_csv(
  file = tmp,
  id = "ZA7576",
  doi = "10.0000/example"
)

}
\seealso{
[read_rds()] for importing surveys from RDS files,
[survey_df()] for constructing survey objects manually.

Other import functions:
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_dta]{read_dta()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_spss]{read_spss()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_codebook.R
\name{create_codebook}
\alias{create_codebook}
\alias{codebook_waves_create}
\alias{codebook_surveys_create}
\title{Create a survey codebook}
\usage{
create_codebook(metadata = NULL, survey = NULL)

codebook_waves_create(waves)

codebook_surveys_create(survey_list)
}
\arguments{
\item{metadata}{A metadata table created by [metadata_create()].
If supplied, `survey` must be `NULL`.}

\item{survey}{A survey object of class `"survey"`.
If supplied, metadata is generated internally using
[metadata_create()].}

\item{waves}{A list of surveys.}

\item{survey_list}{A list containing surveys of class survey.}
}
\value{
A data frame with one row per value label, including:
\itemize{
  \item survey identifiers (`id`, `filename`)
  \item original variable names and labels
  \item value codes and value labels
  \item label type (`"valid"` or `"missing"`)
  \item summary counts of labels
}

Additional user-defined metadata columns present in the input
metadata are preserved.
}
\description{
Expand survey metadata into a long-format codebook of value labels.
}
\details{
`create_codebook()` takes survey-level metadata and returns a
tidy data frame describing all labelled variables and their
associated value labels. Each row corresponds to a single
value label, classified as either a valid value or a missing value.

Unlabelled numeric and character variables are excluded.


For multiple survey waves, use [codebook_surveys_create()].

If both `metadata` and `survey` are provided, `survey` takes precedence.
}
\examples{
survey <- read_rds(
  system.file("examples", "ZA7576.rds", package = "retroharmonize")
)

cb <- create_codebook(survey = survey)
head(cb)

\donttest{
examples_dir <- system.file("examples", package = "retroharmonize")
survey_list <- dir(examples_dir)[grepl("\\\\.rds", dir(examples_dir))]

example_surveys <- read_surveys(
  file.path(examples_dir, survey_list),
  save_to_rds = FALSE
)

codebook_surveys_create(example_surveys)
}
}
\seealso{
[metadata_create()],
[codebook_surveys_create()]

Other metadata functions:
\code{\link[=is.crosswalk_table]{is.crosswalk_table()}},
\code{\link[=metadata_create]{metadata_create()}},
\code{\link[=metadata_survey_create]{metadata_survey_create()}}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retroharmonize.R
\docType{package}
\name{retroharmonize}
\alias{retroharmonize}
\title{retroharmonize: Retrospective harmonization of survey data files}
\description{
The goal of \code{retroharmonize} is to facilitate retrospective (ex-post)
harmonization of data, particularly survey data, in a reproducible manner.
The package provides tools for organizing the metadata, standardizing the
coding of variables, variable names and value labels, including missing
values, and for documenting all transformations, with the help of
comprehensive S3 classes.
}
\section{import functions}{

Read data stored in formats with rich metadata, such as SPSS (.sav) files,
and make them usable in a programmatic context.\cr
\code{\link{read_spss}}: read an SPSS file and record metadata for reproducibility\cr
\code{\link{read_rds}}: read an rds file and record metadata for reproducibility\cr
\code{\link{read_surveys}}: programmatically read a list of surveys\cr
\code{\link{pull_survey}}: pull a single survey from a survey list.\cr
}

\section{subsetting functions}{

\code{\link{subset_surveys}}: remove variables from surveys that cannot be harmonized.
}

\section{variable name harmonization functions}{

\code{\link{harmonize_survey_variables}}: Create a list of surveys with harmonized variable names.\cr
}

\section{variable label harmonization functions}{

Create consistent coding and labelling.\cr
\code{\link{harmonize_values}}: Harmonize the label list across surveys.\cr
\code{\link{harmonize_survey_values}}: Create a list of surveys with harmonized value labels.\cr
\code{\link{na_range_to_values}}: Make the \code{na_range} attributes,
as imported from SPSS, consistent with the \code{na_values} attributes.\cr
\code{\link{label_normalize}} removes special characters, whitespace,
and other typical typing errors and helps the uniformization of labels
and variable names.\cr
}

\section{survey harmonization functions}{

\code{\link{merge_surveys}}: Create a list of surveys with harmonized names and variable labels.\cr
\code{\link{crosswalk_surveys}}: Create a list of surveys with harmonized variable names, harmonized
value labels and harmonize R classes.\cr
\code{\link{crosswalk}}: Create a joined data frame of surveys with harmonized variable names, harmonized
value labels and harmonize R classes.\cr
}

\section{metadata functions}{

\code{\link{metadata_create}}: Createa metadata dataa from one or more \code{\link{survey}}.\cr
\code{\link{metadata_survey_create}}: Create a joined metadata data frame from one survey.\cr
\code{\link{create_codebook}} and \code{\link{codebook_waves_create}}
\code{\link{crosswalk_table_create}}: Create an initial crosswalk table from a metadata data frame.\cr
}

\section{documentation functions}{

Make the workflow reproducible by recording the harmonization process.
\code{\link{document_survey_item}}: Returns a list of the current and historic coding,
labelling of the valid range and missing values or range, the history of the variable names
and the history of the survey IDs.
\code{\link{document_surveys}}: Document the key attributes surveys in a survey list.
}

\section{type conversion functions}{

Consistently treat labels and SPSS-style user-defined missing
values in the R language.
\code{\link{survey}} helps constructing a valid survey data frame, and
\code{\link{labelled_spss_survey}} helps creating a vector for a
questionnaire item.
\code{\link{as_numeric}}: convert to numeric values.\cr
\code{\link{as_factor}}: convert to labels to factor levels.\cr
\code{\link{as_character}}: convert to labels to characters.\cr
\code{\link{as_labelled_spss_survey}}: convert labelled and labelled_spss
vectors to labelled_spss_survey vectors.\cr
}

\seealso{
Useful links:
\itemize{
  \item \url{https://retroharmonize.dataobservatory.eu/}
  \item Report bugs at \url{https://github.com/dataobservatory-eu/retroharmonize/issues}
}

}
\author{
\strong{Maintainer}: Daniel Antal \email{daniel.antal@dataobservatory.eu} (\href{https://orcid.org/0000-0001-7513-6760}{ORCID})

Authors:
\itemize{
  \item Daniel Antal \email{daniel.antal@dataobservatory.eu} (\href{https://orcid.org/0000-0001-7513-6760}{ORCID})
}

Other contributors:
\itemize{
  \item Marta Kolczynska \email{mkolczynska@gmail.com} (\href{https://orcid.org/0000-0003-4981-0437}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey.R, R/survey_df.R
\name{is.survey_df}
\alias{is.survey_df}
\alias{survey_df}
\alias{print.survey_df}
\title{Create a survey object}
\usage{
is.survey_df(x)

survey_df(
  x,
  title = NULL,
  creator = person("Unknown", "Creator"),
  dataset_bibentry = NULL,
  dataset_subject = NULL,
  identifier,
  filename
)

is.survey_df(x)

\method{print}{survey_df}(x, ...)
}
\arguments{
\item{x}{A data frame or tibble containing the survey data.}

\item{title}{Optional title for the survey. Defaults to `"Untitled Survey"`.}

\item{creator}{A [utils::person()] object describing the dataset creator.
Defaults to `person("Unknown", "Creator")`.}

\item{dataset_bibentry}{Optional dataset-level bibliographic metadata.
If `NULL`, a minimal DataCite entry is created automatically using
`title`, `creator`, and `dataset_subject`.}

\item{dataset_subject}{Dataset subject metadata.
If `NULL`, defaults to the Library of Congress Subject Heading
\href{https://id.loc.gov/authorities/subjects/sh85130875}{Surveys}.}

\item{identifier}{A character scalar identifying the survey.}

\item{filename}{A character scalar giving the source filename, or `NULL`
if unknown.}

\item{...}{potentially further arguments for methods.}
}
\value{
An object of class `"survey_df"`, which is a data frame with additional
survey-level metadata stored as attributes and dataset-level metadata
stored using the `dataset` package.
}
\description{
Construct a survey object from a data frame or tibble by attaching
survey-level metadata such as an identifier, source filename, and
basic dataset-level descriptive metadata.
}
\details{
This function is primarily intended for use by import helpers such as
[read_rds()], [read_spss()], [read_dta()], and [read_csv()].
Most users will not need to call it directly.
}
\examples{
survey_df(
  x = data.frame(
    rowid = 1:6,
    observations = runif(6)
  ),
  identifier = "example",
  filename = "no_file"
)

}
\seealso{
[read_survey()] for importing survey data from external files.

Other importing functions:
\code{\link[=survey]{survey()}}
}
\concept{importing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-read_csv.R
\name{read.csv}
\alias{read.csv}
\title{Data Input}
\description{
See \code{utils::\link[utils:read.table]{read.csv}} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_dta.R
\name{read_dta}
\alias{read_dta}
\title{Read a Stata `.dta` survey file}
\usage{
read_dta(file, id = NULL, doi = NULL, .name_repair = "unique")
}
\arguments{
\item{file}{Path to a Stata `.dta` file.}

\item{id}{Optional survey identifier. Defaults to the file name
without extension.}

\item{doi}{Optional DOI identifier for the survey.}

\item{.name_repair}{Strategy for repairing invalid or duplicated
column names. Passed to [haven::read_dta()].}
}
\value{
A `survey` object inheriting from `data.frame` and `tbl_df`.
}
\description{
Import a survey dataset stored in Stata `.dta` format and convert it
into a `survey` object with harmonized metadata and labelled variables.
}
\details{
This function wraps [haven::read_dta()] and adds:

- error handling,
- survey metadata creation,
- `rowid` normalization,
- preservation of variable labels,
- conversion of labelled variables,
- and provenance metadata.


Variable labels are preserved using the `"label"` attribute.

Labelled variables are converted to harmonized labelled survey vectors
where possible. Variables that inherit from `haven_labelled` but do not
contain valid label definitions are converted back to standard vectors.

If the file cannot be read, the function returns an empty `survey`
object and emits a warning.
}
\examples{
\donttest{
path <- system.file(
  "examples",
  "iris.dta",
  package = "haven"
)

survey_object <- read_dta(path)

attr(survey_object, "id")
attr(survey_object, "filename")
}

}
\seealso{
Other import functions:
\code{\link[=harmonize_survey_variables]{harmonize_survey_variables()}},
\code{\link[=pull_survey]{pull_survey()}},
\code{\link[=read_csv]{read_csv()}},
\code{\link[=read_rds]{read_rds()}},
\code{\link[=read_spss]{read_spss()}},
\code{\link[=read_surveys]{read_surveys()}}
}
\concept{import functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na_range_to_values.R
\name{na_range_to_values}
\alias{na_range_to_values}
\title{Harmonize SPSS-style missing value ranges}
\usage{
na_range_to_values(x)
}
\arguments{
\item{x}{A labelled vector created with
[haven::labelled_spss()] or
`retroharmonize_labelled_spss_survey`.}
}
\value{
The input vector with harmonized `na_values` and `na_range`
attributes. If no harmonization is needed, `x` is returned
unchanged.
}
\description{
Ensure consistency between SPSS-style missing value ranges
(`na_range`) and explicit missing values (`na_values`)
for labelled survey vectors.
}
\details{
When both attributes are present, this function:
\itemize{
  \item adjusts the missing range if it conflicts with existing missing values,
  \item derives missing values from the range when necessary,
  \item leaves non-SPSS-labelled vectors unchanged.
}

This harmonization is important before joining, binding, or
summarizing survey data.
}
\seealso{
[labelled::na_range()],
[labelled::na_values()],
[as_numeric()]

Other variable label harmonization functions:
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{variable label harmonization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosswalk.R
\name{is.crosswalk_table}
\alias{is.crosswalk_table}
\alias{crosswalk_table_create}
\title{Validate a crosswalk table}
\usage{
is.crosswalk_table(ctable)

crosswalk_table_create(metadata)
}
\arguments{
\item{ctable}{A table to validate if it is a crosswalk table.}

\item{metadata}{A metadata table created by
[metadata_create()].}
}
\value{
A tibble with raw crosswalk table. It contains all
harmonization tasks, but the
target values need to be set by further manipulations.
}
\description{
Create a crosswalk table with the source variable names
and variable labels.
}
\details{
The table contains a \code{var_name_target} and
\code{val_label_target} column, but
these values need to be set by further manual or
reproducible harmonization steps.
}
\seealso{
Other metadata functions:
\code{\link[=create_codebook]{create_codebook()}},
\code{\link[=metadata_create]{metadata_create()}},
\code{\link[=metadata_survey_create]{metadata_survey_create()}}

Other harmonization functions:
\code{\link[=collect_val_labels]{collect_val_labels()}},
\code{\link[=crosswalk_surveys]{crosswalk_surveys()}},
\code{\link[=harmonize_na_values]{harmonize_na_values()}},
\code{\link[=harmonize_survey_values]{harmonize_survey_values()}},
\code{\link[=harmonize_values]{harmonize_values()}},
\code{\link[=harmonize_var_names]{harmonize_var_names()}},
\code{\link[=label_normalize]{label_normalize()}}
}
\concept{harmonization functions}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/document_surveys.R
\name{document_surveys}
\alias{document_surveys}
\alias{document_waves}
\title{Document survey lists}
\usage{
document_surveys(survey_list = NULL, survey_paths = NULL, .f = NULL)

document_waves(waves)
}
\arguments{
\item{survey_list}{A list of \code{\link{survey}} objects.}

\item{survey_paths}{A vector of full file paths to the surveys to subset, defaults to
\code{NULL}.}

\item{.f}{A function to import the surveys with.
Defaults to \code{'read_rds'}. For SPSS files,
\code{read_spss} is recommended, which is a
well-parameterized version of \code{\link[haven]{read_spss}} that
saves some metadata, too. For STATA files use \code{read_dta}.}

\item{waves}{A list of \code{\link{survey}} objects.}
}
\value{
Returns a data frame with the key attributes of the surveys
in a survey list: the name of the data file, the number of rows and
columns, and the size of the object as stored in memory.
}
\description{
Document the key attributes surveys in a survey list.
}
\details{
The function has two alternative input parameters. If \code{survey_list} is the
input, it returns  the name of the original source data file, the number of rows and
columns, and the size of the object as stored in memory. In case \code{survey_paths}
contains the source data files, it will sequentially read those files, and add the file
size, the last access and the last modified time attributes.

The earlier form \code{document_waves} is deprecated.
Currently called \code{\link{document_surveys}}.
}
\examples{
examples_dir <- system.file("examples", package = "retroharmonize")

my_rds_files <- dir(examples_dir)[grepl(
  ".rds",
  dir(examples_dir)
)]

example_surveys <- read_surveys(file.path(examples_dir, my_rds_files))

documented <- document_surveys(example_surveys)

attr(documented, "original_list")
documented

document_surveys(survey_paths = file.path(examples_dir, my_rds_files))

}
\seealso{
Other documentation functions:
\code{\link[=document_survey_item]{document_survey_item()}}
}
\concept{documentation functions}
